/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



#include <iostream.h>
#include <stdlib.h>
#include "AddOnScriptAnalyser.h"
#include "util.h"
#include "TextFileModifier.h"
#include "LineFileModifier.h"
#include "SectionFileModifier.h"
#include "File.h"
#include "Ini.h"

#define KEY_WORD_version_needed "version needed:"

#define KEY_WORD_name "name:"
#define KEY_WORD_template "template:"
#define KEY_WORD_comment "comment:"
#define KEY_WORD_end_comment "end comment:"
#define KEY_WORD_type "type:"
#define KEY_WORD_initialised_with "initialised with:"

#define KEY_WORD_add_class "add class:"
#define KEY_WORD_end_add_class "end add class:"

#define KEY_WORD_add_parameter "add parameter:"
#define KEY_WORD_end_add_parameter "end add parameter:"

#define KEY_WORD_virtuality "virtuality:"
#define KEY_WORD_declaration "declaration:"
#define KEY_WORD_body "body:"
#define KEY_WORD_end_body "end body:"

#define KEY_WORD_add_function "add function:"
#define KEY_WORD_end_add_function "end add function:"

#define KEY_WORD_id "id:"
#define KEY_WORD_value "value:"

#define KEY_WORD_add_define "add define:"
#define KEY_WORD_end_add_define "end add define:"

#define KEY_WORD_file "file:"

#define KEY_WORD_add_include "add include:"
#define KEY_WORD_end_add_include "end add include:"

#define KEY_WORD_after "after:"
#define KEY_WORD_end_after "end after:"
#define KEY_WORD_before "before:"
#define KEY_WORD_end_before "end before:"
#define KEY_WORD_at_the_beginning "at the beginning:"
#define KEY_WORD_at_the_end "at the end:"

#define KEY_WORD_add_in_enum "add in enum:"
#define KEY_WORD_end_add_in_enum "end add in enum:"

#define KEY_WORD_mother_class "mother class:"

#define KEY_WORD_add_inheritance "add inheritance:"
#define KEY_WORD_end_add_inheritance "end add inheritance:"

#define KEY_WORD_wall_declaration "wall declaration:"
#define KEY_WORD_end_wall_declaration "end wall declaration:"

#define KEY_WORD_add_subclass "add subclass:"
#define KEY_WORD_end_add_subclass "end add subclass:"

#define KEY_WORD_add_roughly "add roughly:"
#define KEY_WORD_end_add_roughly "end add roughly:"

#define KEY_WORD_added_body "added body:"
#define KEY_WORD_end_added_body "end added body:"

#define KEY_WORD_complete_function "complete function:"
#define KEY_WORD_end_complete_function "end complete function:"

#define KEY_WORD_complete_class "complete class:"
#define KEY_WORD_end_complete_class "end complete class:"

#define KEY_WORD_complete_ini_files "complete .ini files:"
#define KEY_WORD_end_complete_ini_files "end complete .ini files:"

#define KEY_WORD_add_roughly_section "add roughly section:"
#define KEY_WORD_end_add_roughly_section "end add roughly section:"

#define KEY_WORD_add_id "add id:"
#define KEY_WORD_in_section "in section:"
#define KEY_WORD_section_number "section number:"
#define KEY_WORD_id_number "id number:"
#define KEY_WORD_end_add_id "end add id:"

#define KEY_WORD_needed_addon "needed addon:"
#define KEY_WORD_end_needed_addon "end needed addon:"
#define KEY_WORD_addon_name "addon name:"
#define KEY_WORD_author "author:"

#define KEY_WORD_include_script "include script:"
#define KEY_WORD_end_include_script "end include script:"
#define KEY_WORD_script_file "script file:"
#define KEY_WORD_parameters "parameters:"
#define KEY_WORD_end_parameters "end parameters:"

#define KEY_WORD_if "if:"
#define KEY_WORD_end_if "end if:"

#define KEY_WORD_addon_parameter "addon parameter:"
#define KEY_WORD_end_addon_parameter "end addon parameter:"



enum LocalisationPos {
  LOCALISATION_POS_AFTER,
  LOCALISATION_POS_BEFORE,
  LOCALISATION_POS_AT_THE_BEGINNING,
  LOCALISATION_POS_AT_THE_END
};
enum LimitedLocalisationPos {
  LIMITED_LOCALISATION_POS_AFTER,
  LIMITED_LOCALISATION_POS_BEFORE,
  LIMITED_LOCALISATION_POS_AT_THE_END
};


AddOnScriptAnalyser::AddOnScriptAnalyser(const String& nameFile,bool verbose) throw(Exception)
  :ScriptAnalyser(nameFile),
   verbose(verbose),
   name(0)
{
}

AddOnScriptAnalyser::~AddOnScriptAnalyser() throw()
{
}

void AddOnScriptAnalyser::messageErr(const String& m) const throw()
{
  //  if(!verbose) return;
  cerr<<"Warning ! "<<(const char*)(*nameOfFile)<<":"<<(const char*)m<<"\n";
  cout.flush();
}

void AddOnScriptAnalyser::message(const String& m) const throw()
{
  if(!verbose) return;
  cout<<(const char*)(*nameOfFile)<<":"<<(const char*)m<<"\n";
  cout.flush();
}

void AddOnScriptAnalyser::message(const String& m0,const String& m1) const throw()
{
  if(!verbose) return;
  cout<<(const char*)(*nameOfFile)<<":"<<(const char*)m0<<":"<<(const char*)m1<<"\n";
  cout.flush();
}

const String *AddOnScriptAnalyser::getString(const String& before) throw(Exception)
{
  ignoreEmptyLines();
  check(before);
  ignoreEmptyLines();
  const String *ret=getLine();
  message(before,*ret);
  return ret;
}

List<const String *> *AddOnScriptAnalyser::getPureBlock(const String& begin,const String& end,bool checkEnds) throw(Exception)
{
  ignoreEmptyLines();
  check(begin);
  return getPureBlockWithoutBegin(begin,end,checkEnds);
}

void AddOnScriptAnalyser::checkNotEnd(const String& s) throw(Exception)
{
  if(endOfFile) return;

  String *end=new String("end");
  String *r=simplifyBlank(s);
  if(r->size()<end->size()) return;

  //prendre les trois premieres lettres de s (dans t)
  char t[end->size()+1];
  for(int i=0;i<end->size();i++) t[i]=(*r)[i];
  t[end->size()]=0;
  delete r;
  
  if(equal(*new String(t),*end)) {
    //%%    list.walkHeadToQueue();
    error(ERROR_ADDON_SCRIPT_INVALID_END,-1);
  }

  delete end;

  return;
}

List<const String *> *AddOnScriptAnalyser::getPureBlockWithoutBegin(const String& begin,const String& end,bool checkEnds) throw(Exception)
{
  const String *l=0;
  List<const String *> *list=new List<const String *>;
  l=getLine();
  if(endOfFile) error(*new String(ERROR_SCRIPT_MUST_FIND),end);
  while(!equal(*l,end)) {

    if(checkEnds) checkNotEnd(*l);

    if(l->size()!=0) list->put(new String(*l));

    if(endOfFile) error(*new String(ERROR_SCRIPT_MUST_FIND),end);
    l=getLine();
  }
  message(*new String("Block extracted"),begin);
  return list;
}

void AddOnScriptAnalyser::getBlockAddClass() throw(Exception)
{
  List<const String *> *comment=0;

  message(*new String("Entering block : add class"));
  ignoreEmptyLines();
  nameClass=getLine();
  message(*new String(KEY_WORD_name),*nameClass);

  const String *s=0,*classTemplate=0;
  bool isTemplate=false;

  s=getString(*new String(KEY_WORD_template));
  if(equal(*s,*new String("no"))) {
    isTemplate=false;
  }
  else {
    isTemplate=true;
    classTemplate=s;
  }

  comment=getPureBlock(*new String(KEY_WORD_comment),*new String(KEY_WORD_end_comment));

  ignoreEmptyLines();
  check(*new String(KEY_WORD_end_add_class));

  //

  LineFileModifier makefile(*new String("Makefile"),markMakefile,*new String("#"));

  makefile.place();
  if(!isTemplate) {
    if(!makefile.after("OBJS=\\")) error(*new String("Cannot find 'OBJS=\\' in Makefile"));
    makefile.insertRaw(*nameClass+*new String(".o\\\n"));
  }
  if(!makefile.after("TOCOMMENT=\\")) error(*new String("Cannot find 'TOCOMMENT=\\' in Makefile"));
  makefile.insertRaw(*nameClass+*new String(".h\\\n"));
  
  makefile.save();

  //

  TextFileModifier hFile(*nameClass+*new String(".h"),mark,true);
  hFile.place();
  List<const String *> l;

  if(isTemplate) {
    l.put(&(*new String("template < class ")+*classTemplate+*new String(" > class ")+*nameClass+*new String(";")));
  }
  else l.put(&(*new String("class ")+*nameClass+*new String(";")));
  l.put(new String(""));
  l.put(&(*new String("#ifndef __")+*nameClass+*new String("_h__")));
  l.put(&(*new String("#define __")+*nameClass+*new String("_h__")));
  l.put(new String(""));

  l.put(new String("/**"));
  comment->place();
  while(comment->walk()) l.put(comment->get());
  l.put(new String("*/"));

  if(isTemplate) l.put(&(*new String("template < class ")+*classTemplate+*new String(" >")));

  l.put(&(*new String("class ")+*nameClass+*new String(" {")));
  l.put(new String("};"));
  l.put(new String(""));
  if(isTemplate) {
    l.put(&(*new String("#include \"")+*nameClass+*new String(".cc\"")));
    l.put(new String(""));
  }
  l.put(new String("#endif"));

  hFile.insert(l);

  hFile.save();

  //

  TextFileModifier ccFile(*nameClass+*new String(".cc"),mark,true);
  ccFile.place();
  if(!isTemplate) ccFile.insert(*new String("#include \"")+*nameClass+*new String(".h\""));

  ccFile.save();

  message(*new String("Leaving block : add class"));
}

void AddOnScriptAnalyser::getBlockAddParameter(HFileModifier& hFile,CCFileModifier& ccFile) throw(Exception)
{
  message(*new String("Entering block : add parameter"));

  const String *l=0;
  const String *protection=0,*type=0,*name=0,*init=0;
  List<const String *> *comment=0;

  ignoreEmptyLines();
  l=getLine();
  if(equal(*l,*new String("public")) || equal(*l,*new String("protected")) || equal(*l,*new String("private"))) {
    protection=l;
    message(*new String("protection"),*protection);
  }
  else {
    //%%    list.walkHeadToQueue();
    error(*new String(ERROR_ADDON_SCRIPT_PROTECTION),-1);
  }

  type=getString(*new String(KEY_WORD_type));
  name=getString(*new String(KEY_WORD_name));
  init=getString(*new String(KEY_WORD_initialised_with));

  comment=getPureBlock(*new String(KEY_WORD_comment),*new String(KEY_WORD_end_comment));

  ignoreEmptyLines();
  check(*new String(KEY_WORD_end_add_parameter));

  hFile.addParameter(*protection,*type,*name,*comment);

  if(*init=="void") {
    //rien
  } else ccFile.addParameter(*name,*init);

  message(*new String("Leaving block : add parameter"));
}

void AddOnScriptAnalyser::getBlockAddFunction(HFileModifier& hFile,CCFileModifier& ccFile) throw(Exception)
{
  message(*new String("Entering block : add function"));

  const String *l=0;
  const String *protection=0,*declaration=0,*isVirtual=0,*pure=0,*classTemplate=0;
  bool isTemplate=false,isPure=false;
  List<const String *> *body=0,*comment=0;

  ignoreEmptyLines();
  l=getLine();
  if(equal(*l,*new String("public")) || equal(*l,*new String("protected")) || equal(*l,*new String("private"))) {
    protection=l;
    message(*new String("protection"),*protection);
  }
  else {
    //%%    list.walkHeadToQueue();
    error(*new String(ERROR_ADDON_SCRIPT_PROTECTION),-1);
  }

  l=getString(*new String(KEY_WORD_virtuality));
  if(equal(*l,*new String("virtual"))) {
    isVirtual=new String("virtual ");
    pure=new String("");
  }
  else if(equal(*l,*new String("pure virtual"))) {
    isVirtual=new String("virtual ");
    pure=new String("=0");
    isPure=true;
  } else if(equal(*l,*new String("no"))) {
    isVirtual=new String("");
    pure=new String("");
  } else {
    error(*new String(ERROR_ADDON_SCRIPT_VIRTUALITY));
  }

  l=getString(*new String(KEY_WORD_template));
  if(equal(*l,*new String("no"))) {
    isTemplate=false;
  }
  else {
    isTemplate=true;
    classTemplate=l;
  }

  declaration=getString(*new String(KEY_WORD_declaration));

  if(!isPure) body=getPureBlock(*new String(KEY_WORD_body),*new String(KEY_WORD_end_body));
  else body=new List<const String *>;
  comment=getPureBlock(*new String(KEY_WORD_comment),*new String(KEY_WORD_end_comment));

  ignoreEmptyLines();
  check(*new String(KEY_WORD_end_add_function));

  hFile.addFunction(*protection,*isVirtual+*declaration+*pure,*comment);
  if(!isPure) {
    if(isTemplate) {
      ccFile.addFunctionTemplate(*classTemplate,*declaration,*body);
    } else {
      ccFile.addFunction(*declaration,*body);
    }
  }

  message(*new String("Leaving block : add function"));
}

void AddOnScriptAnalyser::getBlockAddDefine(HFileModifier& hFile,CCFileModifier& ccFile) throw(Exception)
{
  message(*new String("Entering block : add define"));

  const String *l=0;
  const String *id=0,*value=0;
  int in=0;

  ignoreEmptyLines();
  l=getLine();
  if(equal(*l,*new String("in .h"))) {
    in=0;//%%new String(*new String(".h"));
  }
  else if(equal(*l,*new String("in .cc"))) {
    in=1;//%%new String(*new String(".cc"));
  }
  else {
    //%%    list.walkHeadToQueue();
    error(*new String(ERROR_ADDON_SCRIPT_IN),-1);
  }

  if(in==0) message(*new String("in .h"));
  if(in==1) message(*new String("in .cc"));
  id=getString(*new String(KEY_WORD_id));
  value=getString(*new String(KEY_WORD_value));

  ignoreEmptyLines();
  check(*new String(KEY_WORD_end_add_define));

  if(in==0) hFile.addDefine(*id,*value);
  if(in==1) ccFile.addDefine(*id,*value);

  message(*new String("Leaving block : add define"));
}

void AddOnScriptAnalyser::getBlockAddInclude(HFileModifier& hFile,CCFileModifier& ccFile) throw(Exception)
{
  message(*new String("Entering block : add include"));

  const String *l=0;
  const String *file=0;
  int in=0;

  ignoreEmptyLines();
  l=getLine();
  if(equal(*l,*new String("in .h"))) {
    in=0;//%%new String(*new String(".h"));
  }
  else if(equal(*l,*new String("in .cc"))) {
    in=1;//%%new String(*new String(".cc"));
  }
  else {
    //%%    list.walkHeadToQueue();
    error(*new String(ERROR_ADDON_SCRIPT_IN),-1);
  }

  if(in==0) message(*new String("in .h"));
  if(in==1) message(*new String("in .cc"));
  file=getString(*new String(KEY_WORD_file));

  ignoreEmptyLines();
  check(*new String(KEY_WORD_end_add_include));

  if(in==0) hFile.addInclude(*file);
  if(in==1) ccFile.addInclude(*file);

  message(*new String("Leaving block : add include"));
}

void AddOnScriptAnalyser::getBlockAddInEnum(HFileModifier& hFile) throw(Exception)
{
  message(*new String("Entering block : add in enum"));

  const String *l=0;
  const String *id=0;
  const String *enumName=0;
  List<const String *> *comment=0,*localisation=0;
  LimitedLocalisationPos pos=LIMITED_LOCALISATION_POS_AT_THE_END;

  ignoreEmptyLines();
  enumName=getLine();

  id=getString(*new String(KEY_WORD_id));
  
  ignoreEmptyLines();
  l=getLine();
  if(equal(*l,*new String(KEY_WORD_after))) {
    localisation=getPureBlockWithoutBegin(*new String(KEY_WORD_after),*new String(KEY_WORD_end_after));
    pos=LIMITED_LOCALISATION_POS_AFTER;
  }
  else if(equal(*l,*new String(KEY_WORD_before))) {
    localisation=getPureBlockWithoutBegin(*new String(KEY_WORD_before),*new String(KEY_WORD_end_before));
    pos=LIMITED_LOCALISATION_POS_BEFORE;
  }
  //  else if(equal(*l,*new String("at the beginning")))
  //    pos=POS_AT_THE_BEGINNING;
  else if(equal(*l,*new String(KEY_WORD_at_the_end)))
    pos=LIMITED_LOCALISATION_POS_AT_THE_END;
  else {
    //%%    list.walkHeadToQueue();
    error(*new String(ERROR_ADDON_SCRIPT_LIMITED_POSITION),-1);
  }

  comment=getPureBlock(*new String(KEY_WORD_comment),*new String(KEY_WORD_end_comment));

  ignoreEmptyLines();
  check(*new String(KEY_WORD_end_add_in_enum));

  switch(pos) {
  case LIMITED_LOCALISATION_POS_AFTER:
    hFile.addInEnumAfter(*enumName,*id,*localisation,*comment);
    break;
  case LIMITED_LOCALISATION_POS_BEFORE:
    hFile.addInEnumBefore(*enumName,*id,*localisation,*comment);
    break;
  case LIMITED_LOCALISATION_POS_AT_THE_END:
    hFile.addInEnumAtEnd(*enumName,*id,*comment);
    break;
  }

  message(*new String("Leaving block : add in enum"));
}

void AddOnScriptAnalyser::getBlockAddInheritance(HFileModifier& hFile,CCFileModifier& ccFile) throw(Exception)
{
  message(*new String("Entering block : add inheritance"));

  const String *l=0;
  const String *protection=0,*motherClass=0,*init=0;

  ignoreEmptyLines();
  l=getLine();
  if(equal(*l,*new String("public")) || equal(*l,*new String("protected")) || equal(*l,*new String("private"))) {
    protection=l;
    message(*new String("protection"),*protection);
  }
  else {
    //%%    list.walkHeadToQueue();
    error(*new String(ERROR_ADDON_SCRIPT_PROTECTION),-1);
  }

  motherClass=getString(*new String(KEY_WORD_mother_class));
  init=getString(*new String(KEY_WORD_initialised_with));

  ignoreEmptyLines();
  check(*new String(KEY_WORD_end_add_inheritance));

  hFile.addInheritance(*protection,*motherClass);
  if(*init=="void") {
    //rien
  }
  else ccFile.addInheritance(*motherClass,*init);

  message(*new String("Leaving block : add inheritance"));
}

void AddOnScriptAnalyser::getBlockAddSubclass(HFileModifier& hFile) throw(Exception)
{
  message(*new String("Entering block : add subclass"));

  const String *l=0;
  const String *protection=0;//%%,*declaration=0;
  //%%  List<const String *> *comment=0;
  List<const String *> *wallDeclaration=0;

  ignoreEmptyLines();
  l=getLine();
  if(equal(*l,*new String("public")) || equal(*l,*new String("protected")) || equal(*l,*new String("private"))) {
    protection=l;
    message(*new String("protection"),*protection);
  }
  else {
    //%%    list.walkHeadToQueue();
    error(*new String(ERROR_ADDON_SCRIPT_PROTECTION),-1);
  }

  //%%  declaration=getString(*new String("wall declaration"));

  //%%  comment=getPureBlock(*new String("comment"),*new String("end comment"));
  wallDeclaration=getPureBlock(*new String(KEY_WORD_wall_declaration),*new String(KEY_WORD_end_wall_declaration));

  ignoreEmptyLines();
  check(*new String(KEY_WORD_end_add_subclass));

  hFile.addSubclass(*protection,*wallDeclaration);

  message(*new String("Leaving block : add subclass"));
}

void AddOnScriptAnalyser::getBlockAddRoughly(HFileModifier& hFile,CCFileModifier& ccFile) throw(Exception)
{
  message(*new String("Entering block : add roughly"));

  const String *l=0;
  int in=0;
  List<const String *> *body,*localisation=0;//%%*comment=0;
  LocalisationPos pos=LOCALISATION_POS_AT_THE_END;

  ignoreEmptyLines();
  l=getLine();
  if(equal(*l,*new String("in .h"))) {
    in=0;//%%new String(*new String(".h"));
  }
  else if(equal(*l,*new String("in .cc"))) {
    in=1;//%%new String(*new String(".cc"));
  }
  else {
    //%%    list.walkHeadToQueue();
    error(*new String(ERROR_ADDON_SCRIPT_IN),-1);
  }

  if(in==0) message(*new String("in .h"));
  if(in==1) message(*new String("in .cc"));

  body=getPureBlock(*new String(KEY_WORD_body),*new String(KEY_WORD_end_body));


  ignoreEmptyLines();
  l=getLine();
  if(equal(*l,*new String(KEY_WORD_after))) {
    localisation=getPureBlockWithoutBegin(*new String(KEY_WORD_after),*new String(KEY_WORD_end_after));
    pos=LOCALISATION_POS_AFTER;
  }
  else if(equal(*l,*new String(KEY_WORD_before))) {
    localisation=getPureBlockWithoutBegin(*new String(KEY_WORD_before),*new String(KEY_WORD_end_before));
    pos=LOCALISATION_POS_BEFORE;
  }
  else if(equal(*l,*new String(KEY_WORD_at_the_beginning)))
    pos=LOCALISATION_POS_AT_THE_BEGINNING;
  else if(equal(*l,*new String(KEY_WORD_at_the_end)))
    pos=LOCALISATION_POS_AT_THE_END;
  else {
    //%%    list.walkHeadToQueue();
    error(*new String(ERROR_ADDON_SCRIPT_POSITION),-1);
  }

  //%%  comment=getPureBlock(*new String("comment"),*new String("end comment"));

  ignoreEmptyLines();
  check(*new String(KEY_WORD_end_add_roughly));

  switch(pos) {
  case LOCALISATION_POS_AFTER:
    if(in==0) hFile.addRoughlyAfter(*body,*localisation);
    if(in==1) ccFile.addRoughlyAfter(*body,*localisation);
    break;
  case LOCALISATION_POS_BEFORE:
    if(in==0) hFile.addRoughlyBefore(*body,*localisation);
    if(in==1) ccFile.addRoughlyBefore(*body,*localisation);
    break;
  case LOCALISATION_POS_AT_THE_BEGINNING:
    if(in==0) hFile.addRoughlyAtBeginning(*body);
    if(in==1) ccFile.addRoughlyAtBeginning(*body);
    break;
  case LOCALISATION_POS_AT_THE_END:
    if(in==0) hFile.addRoughlyAtEnd(*body);
    if(in==1) ccFile.addRoughlyAtEnd(*body);
    break;
  }

  message(*new String("Leaving block : add roughly"));
}

void AddOnScriptAnalyser::getBlockCompleteFunction(CCFileModifier& ccFile) throw(Exception)
{
  message(*new String("Entering block : complete function"));

  const String *l=0;
  const String *declaration=0,*classTemplate=0;
  List<const String *> *body=0,*localisation=0;
  LocalisationPos pos=LOCALISATION_POS_AT_THE_END;

  bool isTemplate=false;

  l=getString(*new String(KEY_WORD_template));
  if(equal(*l,*new String("no"))) {
    isTemplate=false;
  }
  else {
    isTemplate=true;
    classTemplate=l;
  }


  declaration=getString(*new String(KEY_WORD_declaration));
  body=getPureBlock(*new String(KEY_WORD_added_body),*new String(KEY_WORD_end_added_body));
  
  ignoreEmptyLines();
  l=getLine();
  if(equal(*l,*new String(KEY_WORD_after))) {
    localisation=getPureBlockWithoutBegin(*new String(KEY_WORD_after),*new String(KEY_WORD_end_after));
    pos=LOCALISATION_POS_AFTER;
  }
  else if(equal(*l,*new String(KEY_WORD_before))) {
    localisation=getPureBlockWithoutBegin(*new String(KEY_WORD_before),*new String(KEY_WORD_end_before));
    pos=LOCALISATION_POS_BEFORE;
  }
  else if(equal(*l,*new String(KEY_WORD_at_the_beginning)))
    pos=LOCALISATION_POS_AT_THE_BEGINNING;
  else if(equal(*l,*new String(KEY_WORD_at_the_end)))
    pos=LOCALISATION_POS_AT_THE_END;
  else {
    //%%    list.walkHeadToQueue();
    error(*new String(ERROR_ADDON_SCRIPT_POSITION),-1);
  }

  //%%  comment=getPureBlock(*new String("comment"),*new String("end comment"));

  ignoreEmptyLines();
  check(*new String(KEY_WORD_end_complete_function));

  if(isTemplate) {
    switch(pos) {
    case LOCALISATION_POS_AFTER:
      ccFile.completeFunctionTemplateAfter(*classTemplate,*declaration,*body,*localisation);
      break;
    case LOCALISATION_POS_BEFORE:
      ccFile.completeFunctionTemplateBefore(*classTemplate,*declaration,*body,*localisation);
      break;
    case LOCALISATION_POS_AT_THE_BEGINNING:
      ccFile.completeFunctionTemplateAtBeginning(*classTemplate,*declaration,*body);
      break;
    case LOCALISATION_POS_AT_THE_END:
      ccFile.completeFunctionTemplateAtEnd(*classTemplate,*declaration,*body);
      break;
    }
  }
  else {
    switch(pos) {
    case LOCALISATION_POS_AFTER:
      ccFile.completeFunctionAfter(*declaration,*body,*localisation);
      break;
    case LOCALISATION_POS_BEFORE:
      ccFile.completeFunctionBefore(*declaration,*body,*localisation);
      break;
    case LOCALISATION_POS_AT_THE_BEGINNING:
      ccFile.completeFunctionAtBeginning(*declaration,*body);
      break;
    case LOCALISATION_POS_AT_THE_END:
      ccFile.completeFunctionAtEnd(*declaration,*body);
      break;
    }
  }

  message(*new String("Leaving block : complete function"));
}

void AddOnScriptAnalyser::getBlockCompleteClass() throw(Exception)
{
  message(*new String("Entering block : complete class"));

  const String *l=0;

  if(endOfFile) error(*new String(ERROR_SCRIPT_MUST_FIND),*new String("className"));

  ignoreEmptyLines();

  nameClass=getLine();
  message(*new String(KEY_WORD_name),*nameClass);

  ignoreEmptyLines();

  if(endOfFile) error(*new String(ERROR_SCRIPT_MUST_FIND),*new String(KEY_WORD_end_complete_class));

  HFileModifier hFile(*nameClass,mark);
  CCFileModifier ccFile(*nameClass,mark);

  l=getLine();
  //  message(*new String("debug",*l);
  while(!equal(*l,*new String(KEY_WORD_end_complete_class))) {

    if(equal(*l,*new String(KEY_WORD_add_parameter))) getBlockAddParameter(hFile,ccFile);

    else if(equal(*l,*new String(KEY_WORD_add_function))) getBlockAddFunction(hFile,ccFile);

    else if(equal(*l,*new String(KEY_WORD_add_define))) getBlockAddDefine(hFile,ccFile);

    else if(equal(*l,*new String(KEY_WORD_add_include))) getBlockAddInclude(hFile,ccFile);
    
    else if(equal(*l,*new String(KEY_WORD_add_in_enum))) getBlockAddInEnum(hFile);

    else if(equal(*l,*new String(KEY_WORD_add_inheritance))) getBlockAddInheritance(hFile,ccFile);

    else if(equal(*l,*new String(KEY_WORD_add_subclass))) getBlockAddSubclass(hFile);
    
    else if(equal(*l,*new String(KEY_WORD_add_roughly))) getBlockAddRoughly(hFile,ccFile);

    else if(equal(*l,*new String(KEY_WORD_complete_function))) getBlockCompleteFunction(ccFile);

    else if(l->size()!=0) error(ERROR_ADDON_SCRIPT_UNKNOWN,*l);

    if(endOfFile) error(*new String(ERROR_SCRIPT_MUST_FIND),*new String(KEY_WORD_end_complete_class));
    l=getLine();
  }

  hFile.save();
  ccFile.save();

  message(*new String("Leaving block : complete class"));
}

void AddOnScriptAnalyser::getBlockCompleteIniFiles() throw(Exception)
{
  message(*new String("Entering block : complete .ini files"));

  const String *l=0;
  List<const String *> fileList;

  if(endOfFile) error(*new String(ERROR_ADDON_SCRIPT_INI_FILE));

  ignoreEmptyLines();

  l=getLine();
  system(*new String("ls -1 --color=no ")+*l+*new String(" > .AddOnScriptAnalyser.trash"));
  File ftrash(".AddOnScriptAnalyser.trash");
  while(!ftrash.eof()) {
    String *s;
    ftrash.read(s);
    fileList.put(s);
  }

  ignoreEmptyLines();

  if(endOfFile) error(*new String(ERROR_SCRIPT_MUST_FIND),*new String(KEY_WORD_end_complete_ini_files));

  l=getLine();
  while(!equal(*l,*new String(KEY_WORD_end_complete_ini_files))) {
    if(equal(*l,*new String(KEY_WORD_add_roughly_section))) {
      message(*new String("Entering block : add roughly section"));

      List< const String * > *body=getPureBlock(*new String(KEY_WORD_body),*new String(KEY_WORD_end_body));
      
      body->place();
      while(body->walk());
      body->put(new String("[]\n"));

      fileList.place();
      while(fileList.walk()) {
	try {
	  message(*new String("Completing file:")+*fileList.get());

	  SectionFileModifier ini(*fileList.get(),markMakefile,*new String("#"));

	  ini.place();
	  while(ini.walk());
	  ini.insert(*body);

	  ini.save();

	} catch(Exception) {
	  messageErr(*new String("Continuing..."));
	}
      }
      
      ignoreEmptyLines();
      check(*new String(KEY_WORD_end_add_roughly_section));
      
      message(*new String("Leaving block : add roughly section"));

    } else if(equal(*l,*new String(KEY_WORD_add_id))) {

      message(*new String("Entering block : add id"));
      
      const String *section=0,*sectionNumber=0,*id=0,*idNumber=0,*value=0;
      int sectionN=-1,idN=-1;
      bool convertOk=true;

      section=getString(KEY_WORD_in_section);
      sectionNumber=getString(KEY_WORD_section_number);
      if(!equal(*sectionNumber,"all")) {
	try {
	  sectionN=sectionNumber->toInt();
	} catch(Exception) {
	  convertOk=false;
	}
      }
      id=getString(KEY_WORD_id);
      idNumber=getString(KEY_WORD_id_number);
      if(!equal(*idNumber,"maximum")) {
	try {
	  idN=idNumber->toInt();
	} catch(Exception) {
	  convertOk=false;
	}
      }
      value=getString(KEY_WORD_value);

      if(!convertOk) error("Cannot convert to int");

      fileList.place();
      while(fileList.walk()) {
	try {
	  message(*new String("Completing file:")+*fileList.get());

	  if(idN<0) {
	    SectionFileModifier ini(*fileList.get(),markMakefile,*new String("#"));

	    List< const String * > body;
	    body.put(&(*id+*new String(":")+*value));

	    if(sectionN>=0) {

	      ini.place();
	      int cpt=0;
	      bool sectionFound=false;
	      while(ini.after(*section+*new String("="))) {
		if(cpt==sectionN) { sectionFound=true; break; }
		cpt++;
	      }
	      if(!sectionFound) messageErr(*new String("Cannot find section:")+*section+*new String("#")+*sectionNumber+*new String(" in file ")+*fileList.get());
	      else if(!ini.after("[]")) messageErr("Cannot find []");
	      else {
		ini.before("[]");
		ini.insert(body);
		ini.save();
	      }
	    }
	    else {
	      ini.place();
	      while(ini.after(*section+*new String("="))) {
		if(!ini.after("[]")) messageErr("Cannot find []");
		else {
		  ini.before("[]");
		  ini.insert(body);
		}
	      }
	      ini.save();
	    }
	  
	  }
	  else {
	    Ini ini(*fileList.get());

	    if(sectionN>=0) {

	      const String *ret=ini.getWithoutException(*section,sectionN,
							*id,idN);
	      if(ret && !(*ret==*value)) {
		messageErr(*new String("Warning while completing file :")+*fileList.get());
		messageErr(*new String("The old value of the id ")+*id+
			   *new String("#")+*idNumber+
			   *new String(" in section ")+*section+
			   *new String("#")+*sectionNumber+
			   *new String(" was ")+*ret);
	      }
	      
	      ini.set(*section,sectionN,
		      *id,idN,
		      *value);
	    } else {
	      ini.setInAllSection(*section,
				  *id,idN,
				  *value);
	    }
	    ini.save();
	  }
	
	} catch(Exception) {
	  messageErr(*new String("Continuing..."));
	}
      }

      
      ignoreEmptyLines();
      check(*new String(KEY_WORD_end_add_id));
      
      message(*new String("Leaving block : add id"));
    }
    else if(l->size()!=0) error(ERROR_ADDON_SCRIPT_UNKNOWN,*l);

    if(endOfFile) error(*new String(ERROR_SCRIPT_MUST_FIND),*new String(KEY_WORD_end_complete_ini_files));
    l=getLine();
  }

  message(*new String("Leaving block : complete .ini files"));
}

void AddOnScriptAnalyser::supprBlockCompleteIniFiles() throw(Exception)
{
  message(*new String("Entering block : complete .ini files"));

  const String *l=0;
  List<const String *> fileList;

  if(endOfFile) {
    messageErr(*new String(ERROR_ADDON_SCRIPT_INI_FILE));
    return;
  }

  ignoreEmptyLines();

  l=getLine();
  system(*new String("ls -1 --color=no ")+*l+*new String(" > .AddOnScriptAnalyser.trash"));
  try {
    File ftrash(".AddOnScriptAnalyser.trash");
    while(!ftrash.eof()) {
      String *s;
      ftrash.read(s);
      fileList.put(s);
    }
  } catch(Exception) {
    messageErr("Continuing...");
    return;
  }

  ignoreEmptyLines();

  if(endOfFile) {
    messageErr(*new String(ERROR_SCRIPT_MUST_FIND)+*new String(":")+*new String(KEY_WORD_end_complete_ini_files));
    return;
  }

  fileList.place();
  while(fileList.walk()) {
    try {
      message(*new String("Cleaning file:")+*fileList.get());
      
      SectionFileModifier ini(*fileList.get(),markMakefile,*new String("#"));
      
      ini.suppress();
      ini.save();
      
    } catch(Exception) {
      messageErr(*new String("Continuing..."));
    }
  }
      
  l=getLine();
  while(!equal(*l,*new String(KEY_WORD_end_complete_ini_files))) {
    if(endOfFile) {
      messageErr(*new String(ERROR_SCRIPT_MUST_FIND)+*new String(KEY_WORD_end_complete_ini_files));
      return;
    }
    l=getLine();
  }
  
  message(*new String("Leaving block : complete .ini files"));
}

bool AddOnScriptAnalyser::checkIfAlready(const String& f) throw()
{
  if(name==0) {
    getPureBlock(KEY_WORD_addon_parameter,KEY_WORD_end_addon_parameter);

    name=getString(*new String(KEY_WORD_addon_name));
    author=getString(*new String(KEY_WORD_author));
  }

  message(*new String("Checking if already added (in file : ")+f+*new String(" )"));
  
  LineFileModifier addonfile(f,markMakefile,*new String("#"));
  
  addonfile.place();
  List<const String *> l;
  l.put(new String(*new String("#addon:")+*name));
  l.put(new String(*new String("#author:")+*author));
  return addonfile.after(l);
}


void AddOnScriptAnalyser::checkNeeded(const String& nameN,const String& authorN) throw(Exception)
{
  message(*new String("Checking if addon needed present in file : .addon (name:")+nameN+*new String(",author:")+authorN+*new String(")"));

  LineFileModifier addonfile(*new String(".addon"),markMakefile,*new String("#"));

  addonfile.place();
  List<const String *> l;
  l.put(&(*new String("#addon:")+nameN));
  l.put(&(*new String("#author:")+authorN));
  bool ret=addonfile.after(l);
  if(!ret) error(*new String("Addon needed not added (name:")+nameN+*new String(",author:")+authorN+*new String(") - It is marked in file : .addon\n -> Add this addon needed before !"));

  //

  addonfile.place();
  addonfile.insert(*new String("#needed addon:")+nameN+*new String(" ")+authorN);
  
  addonfile.save();

  //

  message(*new String("Ok"));
}

void AddOnScriptAnalyser::getBlockNeededAddon() throw(Exception)
{
  message(*new String("Entering block : needed addon"));

  const String *l=0,*nameAddon=0,*authorAddon=0;

  //%%  ignoreEmptyLines();

  for(;;) {
    ignoreEmptyLines();
    if(endOfFile) error(*new String(ERROR_SCRIPT_MUST_FIND),*new String(KEY_WORD_end_needed_addon));
    l=getLine();
    if(equal(*l,*new String(KEY_WORD_end_needed_addon))) break;
    if(equal(*l,*new String(KEY_WORD_addon_name))) {
      ignoreEmptyLines();
      nameAddon=getLine();
      message(*new String(KEY_WORD_addon_name),*nameAddon);
      authorAddon=getString(*new String(KEY_WORD_author));
      checkNeeded(*nameAddon,*authorAddon);
    }
    else if(l->size()!=0) error(ERROR_ADDON_SCRIPT_UNKNOWN,*l);
  }

  message(*new String("Leaving block : add needed addon"));
}

void AddOnScriptAnalyser::includeScript(const String& file,List<const String *> param,bool suppress) throw(Exception)
{
  if(!suppress) message(*new String("Using other script :")+file);
  else message(*new String("Suppressing from other script :")+file);
  AddOnScriptAnalyser f(file,verbose);
  if(!suppress) f.analyse(param,*name,*author);
  else f.suppress(param,*name,*author);
}

void AddOnScriptAnalyser::getBlockIncludeScript(bool suppress) throw(Exception)
{
  message(*new String("Entering block : include script"));

  const String *l=0,*scriptFile=0;
  List<const String *> *param=0;

  //%%  ignoreEmptyLines();

  for(;;) {
    ignoreEmptyLines();
    if(endOfFile) error(*new String(ERROR_SCRIPT_MUST_FIND),*new String(KEY_WORD_end_include_script));
    l=getLine();
    if(equal(*l,*new String(KEY_WORD_end_include_script))) break;
    if(equal(*l,*new String(KEY_WORD_script_file))) {
      ignoreEmptyLines();
      scriptFile=getLine();
      message(*new String(KEY_WORD_script_file),*scriptFile);
      param=getPureBlock(*new String(KEY_WORD_parameters),*new String(KEY_WORD_end_parameters));
      includeScript(*scriptFile,*param,suppress);
    }
    else if(l->size()!=0) error(*new String(ERROR_ADDON_SCRIPT_UNKNOWN)+
				*new String("(Must find '")+
				*new String(KEY_WORD_script_file)+
				*new String("' or '")+
				*new String(KEY_WORD_end_include_script)+
				*new String("')"),
				*l,-1);
  }

  message(*new String("Leaving block : include script"));
}

List<int> *constructListOfInt(const String& s) throw()
{
  List<int> *l=new List<int>;
  List<char> lc;
  int k=0;
  while(k<s.size()) {
    if(s[k]=='.') {
      String *aux=new String(lc);
      lc.place();
      while(lc.walk()) lc.suppr();
      int i;
      try {
	i=aux->toInt();
      } catch(Exception) {
	i=-1;
      }
      delete aux;
      l->put(i);
    } else lc.put(s[k]);
    k++;
  }

  if(!lc.empty()) {
    String *aux=new String(lc);
    int i;
    try {
      i=aux->toInt();
    } catch(Exception) {
      i=-1;
    }
    delete aux;
    l->put(i);
  }
  return l;
}

bool compareVersion(const String& s0,const String& s1) throw()
{
  List<int> *l0=constructListOfInt(s0);
  List<int> *l1=constructListOfInt(s1);
  l0->place();
  l1->place();
  while(l1->walk() && l0->walk()) {
    if(l0->get()<l1->get()) {
      delete l0;
      delete l1;
      return false;
    }
  }
  delete l0;
  delete l1;
  return true;
}

void AddOnScriptAnalyser::getVersionNeeded() throw(Exception)
{
  message(*new String("Checking for version"));
  ignoreEmptyLines();
  const String *versionNeeded=getLine();

  if(!File::exists(".version")) error(*new String("Addon need a version of the program :\n Version needed:")+*versionNeeded+*new String("\nThe file .version does not exists, cannot know the version !"));

  File f(".version");
  String *s;
  f.read(s);
  if(!compareVersion(*s,*versionNeeded))
    error(*new String("Addon need a newer version of the program :\n Version needed:")+*versionNeeded);
  delete s;
}

bool AddOnScriptAnalyser::isIf(const String& s) const throw()
{
  String i(KEY_WORD_if);
  if(s.size()<i.size()+1) return false;
  int k=0;
  while(k<i.size()) {
    if(s[k]!=i[k]) return false;
    k++;
  }
  return true;
  //return (s.size()>=4 && s[0]=='i' && s[1]=='f' && s[2]==' ');
}

void AddOnScriptAnalyser::cleanTheIfs(bool all) throw(Exception)
{
  keepPosition();
  
  List<bool> keepingLine;
  keepingLine.put(true);

  int cpt=0;

  list.place();
  while(list.walk()) {
    String *s=simplifyBlank(*list.get());
    if(isIf(*s)) { //%%%%->size()>=4 && (*s)[0]=='i' && (*s)[1]=='f' && (*s)[2]==' ') {
      
      cpt++;

      bool equ=true;

      if(all || cpt==1) {

      List<char> l0;
      List<char> l1;
      int k=(new String(KEY_WORD_if))->size();
      while((*s)[k]!='=') {
	if(k>=s->size()) error(*new String("Invalid if, must find '=='"));
	l0.put((*s)[k]);
	k++;
      }
      k++;
      if(k>=s->size() || (*s)[k]!='=') error(*new String("Invalid if, must find '=='"));
      k++;
      while(k<s->size()) {
	l1.put((*s)[k]);
	k++;
      }
      delete list.get();
      list.suppr();
      list.put(new String(""));

      //comparer l0 et l1
      l0.place();
      l1.place();
      while(l0.walk()) {
	if(!l1.walk()) { equ=false; break; }
	if(l0.get()!=l1.get()) { equ=false; break; }
      }
      if(l1.walk()) equ=false;

      }
      else equ=true;

      keepingLine.placeCurrentOnHead();
      keepingLine.put(equ);
    }
    else if(equal(*list.get(),*new String(KEY_WORD_end_if))) {
      keepingLine.placeCurrentOnHead();
      keepingLine.suppr();
      if(keepingLine.empty()) error(*new String("Invalid end if"));

      if(all || cpt==1) {

      delete list.get();
      list.suppr();
      list.put(new String(""));

      }

      cpt--;
      if(!all && cpt==0) break;

    }
    else {
      keepingLine.place();
      while(keepingLine.walk())
	if(!keepingLine.get()) {
	  delete list.get();
	  list.suppr();
	  list.put(new String(""));
	  break;
	}
    }
  }

  recallPosition();
}

void AddOnScriptAnalyser::replaceWord(const String& from,const String& to) throw()
{
  if(from.size()==0) return;

  List<char> lfrom;
  for(int k=0;k<from.size();k++) lfrom.put(from[k]);
  List<char> lto;
  for(int k=0;k<to.size();k++) lto.put(to[k]);

  keepPosition();
  
  list.place();
  while(list.walk()) {
    //prendre le String
    const String *s=list.get();
    List<char> l;
    for(int k=0;k<s->size();k++) l.put((*s)[k]);

    list.suppr();
    delete s;
    
    //remplacer dedans
    
    lfrom.place();
    lfrom.walk();

    l.place();
    while(l.walk()) {
      if(l.get()==lfrom.get()) {
	if(!lfrom.walk()) {
	  do {
	    l.suppr();
	  } while(lfrom.walkHeadToQueue());
	  lfrom.place();
	  lfrom.walk();
	  lto.place();
	  while(lto.walk()) l.put(lto.get());
	}
      }
      else {
	lfrom.place();
	lfrom.walk();
      }
    }
    
    //remettre le String
    list.put(new String(l));
  }

  recallPosition();
}


void AddOnScriptAnalyser::replaceParam(List<const String*> param,const String& name,const String& author) throw (Exception)
{
  this->name=&name;
  this->author=&author;

  /*
  List<const String *> *addonParam=getPureBlock("addon parameter","end addon parameter",false);
  */


  ignoreEmptyLines();
  check(KEY_WORD_addon_parameter);

  const String *l=0;
  //  List<const String *> addonParam;
  l=getLine();
  if(endOfFile) error(*new String(ERROR_SCRIPT_MUST_FIND),*new String(KEY_WORD_end_addon_parameter));

  param.place();

  while(!equal(*l,KEY_WORD_end_addon_parameter)) {

    checkNotEnd(*l);

    //    if(l->size()!=0) list->put(new String(*l));
    if(l->size()>0) {
      if(isIf(*l))
	cleanTheIfs(false);
      else {
	if(!param.walk()) error(*new String("Not enough parameters to include addon"));
	message(*new String("Replacing '")+*l+*new String("' with '")+*(param.get())+*new String("'"));
	replaceWord(*l,*(param.get()));
      }
    }

    if(endOfFile) error(*new String(ERROR_SCRIPT_MUST_FIND),*new String(KEY_WORD_end_addon_parameter));
    l=getLine();
  }
  if(param.walk()) error(*new String("Two much parameters to include addon"));
  cleanTheIfs(true);


  /*
  addonParam->place();
  param.place();
  while(param.walk()) {
    for(;;) {
      if(!addonParam->walk()) error(*new String("Wrong number of parameters to include addon"));
      if(addonParam->get()->size()>0) {
	if(!isIf(*(addonParam->get()))) break;
	cleanTheIfs(false);
      }
    }

    message(*new String("Replacing '")+*(addonParam->get())+*new String("' with '")+*(param.get())+*new String("'"));
    replaceWord(*(addonParam->get()),*(param.get()));
  }
  if(addonParam->walk()) error(*new String("Wrong number of parameters to include addon"));

  cleanTheIfs(true);
  */

  getString(*new String(KEY_WORD_addon_name));
  getString(*new String(KEY_WORD_author));
}


void AddOnScriptAnalyser::analyse(List<const String*>& param,const String& name,const String& author) throw(Exception)
{
  replaceParam(param,name,author);

  if(!File::exists(".addon")) {
    File f(".addon",File::WRITE_ONLY);
  }

  run();
}

void AddOnScriptAnalyser::analyse() throw(Exception)
{
  List<const String *> *addonParam=getPureBlock(KEY_WORD_addon_parameter,KEY_WORD_end_addon_parameter);

  if(!addonParam->empty()) error(*new String("This addon need some parameters, it only can be included by another addon"));

  name=getString(*new String(KEY_WORD_addon_name));
  author=getString(*new String(KEY_WORD_author));


  if(!File::exists(".addon")) {
    File f(".addon",File::WRITE_ONLY);
  }
  if(checkIfAlready(*new String(".addon")))
    error("Addon already added - It is marked in file : .addon");
  else message(*new String("Ok"));

  run();

  completeAddonFile();
}


void AddOnScriptAnalyser::completeAddonFile() throw(Exception)
{
  LineFileModifier addonfile(*new String(".addon"),markMakefile,*new String("#"));

  addonfile.place();
  addonfile.insert(*new String("#This line is useful to check the addon"));
  
  addonfile.save();
}

//protected
void AddOnScriptAnalyser::run() throw(Exception)
{
  const String *l=0;
  message(*new String("Analyse begun"));

  mark.put(new String("/*"));
  mark.put(&(*new String("addon:")+*name));
  mark.put(&(*new String("author:")+*author));
  mark.put(new String("*/"));

  markMakefile.put(&(*new String("#addon:")+*name));
  markMakefile.put(&(*new String("#author:")+*author));

  ignoreEmptyLines();
  if(endOfFile) error(*new String(ERROR_SCRIPT_MUST_FIND),*new String(KEY_WORD_needed_addon));
  l=getLine();
  if(!equal(*l,*new String(KEY_WORD_needed_addon))) {
    //%%    list.walkHeadToQueue();
    error(*new String(ERROR_SCRIPT_MUST_FIND),*new String(KEY_WORD_needed_addon),-1);
  }
  else {
    getBlockNeededAddon();
    ignoreEmptyLines();
    if(!endOfFile) l=getLine();
  }

  /*%%%%%%
  if(!equal(*l,*new String("include script"))) {
    error(*new String(ERROR_SCRIPT_MUST_FIND),*new String("include script"),-1);
  }
  else {
    getBlockIncludeScript();
    ignoreEmptyLines();
    if(!endOfFile) l=getLine();
  }
  */

  if(!endOfFile)
    for(;;) {
      if(equal(*l,*new String(KEY_WORD_include_script))) getBlockIncludeScript();
      else if(equal(*l,*new String(KEY_WORD_complete_class))) getBlockCompleteClass();
      else if(equal(*l,*new String(KEY_WORD_add_class))) getBlockAddClass();
      else if(equal(*l,*new String(KEY_WORD_complete_ini_files))) getBlockCompleteIniFiles();
      else if(equal(*l,*new String(KEY_WORD_version_needed))) getVersionNeeded();
      else if(l->size()!=0) error(ERROR_ADDON_SCRIPT_UNKNOWN,*l);
      ignoreEmptyLines();
      if(endOfFile) break;
      l=getLine();
      if(endOfFile) break;
    }

  message(*new String("Analyse ended"));
}

void AddOnScriptAnalyser::warningNeeded() throw()
{
  message(*new String("Checking if there are addons needing the suppress addon in file : .addon"));

  LineFileModifier addonfile(*new String(".addon"),markMakefile,*new String("#"));

  addonfile.place();
  List<const String *> l;
  l.put(&(*new String("#needed addon:")+*name+*new String(" ")+*author));

  while(addonfile.after(l)) {

    const String *authorLine,*nameLine;

    if(!addonfile.walkInvers()) messageErr("Found dependance, but cannot find addon author, not enough line");
    if(!addonfile.walkInvers()) messageErr("Found dependance, but cannot find addon author, not enough line");
    authorLine=addonfile.current();
    if(!addonfile.walkInvers()) messageErr("Found dependance, but cannot find addon name, not enough line");
    nameLine=addonfile.current();

    if(authorLine && nameLine) {
      int size=(new String("#author:"))->size();
      if(authorLine->size()<=size)
	messageErr("Found dependance, but cannot find addon author, invalid line");
      else {
	List<char> authorList;
	for(int k=0;k<authorLine->size()-1-size;k++)
	  authorList.put((*authorLine)[k+size]);

	size=(new String("#addon:"))->size();
	if(nameLine->size()<=size)
	  messageErr("Found dependance, but cannot find addon name, invalid line");
	else {
	  List<char> nameList;
	  for(int k=0;k<nameLine->size()-1-size;k++)
	    nameList.put((*nameLine)[k+size]);

	  messageErr(*new String("WARNING:an included addon need the addon removed ! You should also remove it :\n")+*new String(nameList)+*new String(";")+*new String(authorList));
	  
	}
      }
    }

    addonfile.walk();
    addonfile.walk();
    addonfile.walk();
  }

  //

  message(*new String("Ok"));
}

void AddOnScriptAnalyser::suppress(List<const String *>& param,const String& name,const String& author) throw(Exception)
{
  replaceParam(param,name,author);
  unrun();
}

void AddOnScriptAnalyser::suppress() throw(Exception)
{
  List<const String *> *addonParam=getPureBlock(KEY_WORD_addon_parameter,KEY_WORD_end_addon_parameter);

  if(!addonParam->empty()) error(*new String("This addon need some parameters, it only can be suppressed by another addon"));

  name=getString(*new String(KEY_WORD_addon_name));
  author=getString(*new String(KEY_WORD_author));

  warningNeeded();

  unrun();
}

void AddOnScriptAnalyser::unrun() throw(Exception)
{
  const String *l;
  message(*new String("Suppress begun"));

  if(!File::exists(".addon")) {
    File f(".addon",File::WRITE_ONLY);
  }

  mark.put(new String("/*"));
  mark.put(&(*new String("addon:")+*name));
  mark.put(&(*new String("author:")+*author));
  mark.put(new String("*/"));

  markMakefile.put(&(*new String("#addon:")+*name));
  markMakefile.put(&(*new String("#author:")+*author));


  while(!endOfFile) {
    ignoreEmptyLines();
    if(endOfFile) break;
    l=getLine();
    if(equal(*l,*new String(KEY_WORD_needed_addon))) {
      while(!endOfFile) {
	l=getLine();
	if(equal(*l,*new String(KEY_WORD_end_needed_addon))) break;
      }
    }
    else if(equal(*l,*new String(KEY_WORD_include_script))) {
      getBlockIncludeScript(true);
    }
    else if(equal(*l,*new String(KEY_WORD_complete_class))) {

      //COMPLETE
      message(*new String("Entering block : complete class"));

      //%%      const String *l=0;
      ignoreEmptyLines();


      if(endOfFile) {
	error(*new String(ERROR_SCRIPT_MUST_FIND),*new String("className"),0,false);
      }
      else {

	nameClass=getLine();
	message(*new String(KEY_WORD_name),*nameClass);
	
	ignoreEmptyLines();
	
	if(endOfFile) {
	  error(*new String(ERROR_SCRIPT_MUST_FIND),*new String(KEY_WORD_end_complete_class),0,false);
	}
	else {

	  if(File::exists(*nameClass+*new String(".h")))
	    try {
	      message(*new String("Suppressing in ")+*nameClass+*new String(".h"));
	      LineFileModifier hFile(*nameClass+*new String(".h"),mark,*new String("//"));
	      hFile.suppress();
	      hFile.save();
	    } catch(Exception) {
	      messageErr(*new String("Continuing..."));
	    }

	  if(File::exists(*nameClass+*new String(".cc")))
	    try {
	      message(*new String("Suppressing in ")+*nameClass+*new String(".cc"));
	      LineFileModifier ccFile(*nameClass+*new String(".cc"),mark,*new String("//"));
	      ccFile.suppress();
	      ccFile.save();
	    } catch(Exception) {
	      messageErr(*new String("Continuing..."));
	    }
	  
	  l=getLine();
	  while(!equal(*l,*new String(KEY_WORD_end_complete_class))) {
	    if(endOfFile) {
	      error(*new String(ERROR_SCRIPT_MUST_FIND),*new String(KEY_WORD_end_complete_class),0,false);
	      break;
	    }
	    l=getLine();
	  }
	}
      }
      message(*new String("Leaving block : complete class"));
    }
    else if(equal(*l,*new String(KEY_WORD_add_class))) {

      //ADD

      message(*new String("Entering block : add class"));

      ignoreEmptyLines();
	
      if(endOfFile) {
	error(*new String(ERROR_SCRIPT_MUST_FIND),*new String("className"),0,false);
      }
      else {
	nameClass=getLine();
	message(*new String(KEY_WORD_name),*nameClass);
	
	const String *s=0;
	bool isTemplate=true;
	
	try {
	  s=getString(*new String(KEY_WORD_template));
	  if(equal(*s,*new String("no"))) {
	    isTemplate=false;
	  }
	  else {
	    isTemplate=true;
	  }
	} catch(Exception) {
	  messageErr(*new String("Continuing..."));
	}

	const String *l=getLine();
	while(!equal(*l,*new String(KEY_WORD_end_add_class))) {
	  if(endOfFile) {
	    error(*new String(ERROR_SCRIPT_MUST_FIND),*new String(KEY_WORD_end_add_class),0,false);
	    break;
	  }
	  l=getLine();
	}
	
	//SUPPRESS IN MAKEFILE
	
	message(*new String("Cleaning Makefile"));
	
	try {
	  LineFileModifier makefile(*new String("Makefile"),markMakefile,*new String("#"));
	  
	  message(*new String("Suppressing comment blocs"));
	  
	  makefile.place();
	  makefile.suppress();
	  
	  if(!isTemplate) {
	    message(*new String("Suppressing class in OBJS"));
	    
	    bool conti=true;
	    while(conti) {
	      makefile.place();
	      if(!makefile.after("OBJS=\\")) conti=false;
	      //%% error(*new String("Cannot find 'OBJS=\\' in Makefile"),0,false);
	      else if(!makefile.after(*nameClass+*new String(".o\\"))) conti=false;
	      //%%		error(*new String("Cannot find ")+*nameClass+*new String(".o\\")+*new String(" in Makefile"),0,false);
	      else makefile.suppressLine();
	    }
	  }
	  
	  message(*new String("Suppressing class in TOCOMMENT"));
	  
	  bool conti=true;
	  while(conti) {
	    makefile.place();
	    if(!makefile.after("TOCOMMENT=\\")) conti=false;
	    //%% error(*new String("Cannot find 'TOCOMMENT=\\' in Makefile"),0,false);
	    else if(!makefile.after(*nameClass+*new String(".h\\"))) conti=false;
	    //%%	    error(*new String("Cannot find ")+*nameClass+*new String(".h\\")+*new String(" in Makefile"),0,false);
	    else makefile.suppressLine();
	  }

	  makefile.save();
	} catch(Exception) {
	  messageErr(*new String("Continuing..."));
	}
/*
	//

	String *sn=0;

	try {
	  message(*new String("Renaming file ")+*nameClass+*new String(".h"));//%% in .h.bak"));
	  sn=File::renameCheck(*nameClass+*new String(".h"),*new String(".bak"));
	  //%% error(*new String("CANNOT RENAME FILE ")+*nameClass+*new String(".h"),0,false);
	  message(*new String("New file name:")+*sn);
	} catch(Exception) {
	  messageErr(*new String("Continuing..."));
	}

	try {
	  message(*new String("Renaming file ")+*nameClass+*new String(".cc"));//%%.bak"));
	  sn=File::renameCheck(*nameClass+*new String(".cc"),*new String(".bak"));
	  //%% error(*new String("CANNOT RENAME FILE ")+*nameClass+*new String(".cc"),0,false);
	  message(*new String("New file name:")+*sn);
	} catch(Exception) {
	  messageErr(*new String("Continuing..."));
	}
*/

	message(*new String("Leaving block : add class"));
      }
    }

    else if(equal(*l,*new String(KEY_WORD_complete_ini_files))) {
      supprBlockCompleteIniFiles();
    }
    //%%    else if(l->size()!=0) error(ERROR_ADDON_SCRIPT_UNKNOWN,0,false);
  }

  message(*new String("Cleaning file : .addon"));
	
  try {
    LineFileModifier addonfile(*new String(".addon"),markMakefile,*new String("#"));
	  
    message(*new String("Suppressing comment blocs"));
	  
    addonfile.place();
    addonfile.suppress();

    addonfile.save();
  } catch(Exception) {
    messageErr(*new String("Continuing..."));
  }

	  
  message(*new String("Suppress ended"));

}



void AddOnScriptAnalyser::listAddons() throw()
{
  if(!File::exists(".addon")) return;

  List<String *> addonList;
  File addonfile(".addon");
  while(!addonfile.eof()) {
    String *read;
    addonfile.read(read);

    String addonWord("#addon:");
    bool idem=true;
    int k=0;
    while(idem && k<addonWord.size()) {
      if(k>=read->size()) idem=false;
      else if((*read)[k]!=addonWord[k]) idem=false;
      else k++;
    }
    if(idem) {
      List<char> titleAddon;
      while(k<read->size()) {
	titleAddon.put((*read)[k]);
	k++;
      }
      addonList.place();
      idem=false;
      String *toAdd=new String(titleAddon);
      while(!idem && addonList.walk()) {
	if((*addonList.get())==*toAdd) idem=true;
      }
      if(!idem) addonList.put(toAdd);
    }

    delete read;
  }
  
  if(addonList.empty()) {
  }
  else {
    addonList.place();
    while(addonList.walk()) {
      cout<<(const char *)(*addonList.get())<<"\n";
      cout.flush();
      delete addonList.get();
    }
  }
}
