/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



#include <iostream.h>
#include "Anim.h"
#include "File.h"
#include "error.h"
#include "GifIn.h"

//Now the file name can be like: /Sprite/agif,-20,10
//to decal the anim vertically and horizontally !

#define NB_OPTION_ON_FILE_NAME 4

String *extractFromNameFile(const String& nameFile,int& jDecal,int& iDecal,bool& on,bool& to,bool& under,char listSeparator) throw()
{
  List<char> l[NB_OPTION_ON_FILE_NAME];
  int cur=0;
  int k=0;
  while(k<nameFile.size()) {
    if(nameFile[k]==listSeparator) {
      cur++;
    }
    else if(cur<NB_OPTION_ON_FILE_NAME) l[cur].put(nameFile[k]);
    k++;
  }
  if(l[1].empty()) jDecal=0;
  else {
    String *aux=new String(l[1]);
    try {
      jDecal=aux->toInt();
    } catch(Exception) { jDecal=0; }
    delete aux;
  }

  if(l[2].empty()) iDecal=0;
  else {
    String *aux=new String(l[2]);
    try {
      iDecal=aux->toInt();
    } catch(Exception) { iDecal=0; }
    delete aux;
  }

  if(!l[3].empty()) {
    String *aux=new String(l[3]);
    String sOn("on");
    String sTo("to");
    String sUnder("under");
    if(*aux==sOn) on=true;
    if(*aux==sTo) to=true;
    if(*aux==sUnder) under=true;
    delete aux;
  }

  return new String(l[0]);
}

Anim::Anim(const String& nameFile,int slow,char listSeparator) throw (Exception)
  :Pageable(slow),color(toPixel(0,0,0,0)),colored(false),on(false),to(false),under(false),pal(0),
   deleter(true),uppest(false)
{
  bool _on=false,_to=false,_under=false;
  decal.j=decal.i=0;
  String *nameF=extractFromNameFile(nameFile,decal.j,decal.i,_on,_to,_under,listSeparator);
  GifIn gif(*nameF);

  gif.place();
  while(gif.walk()) list.put(gif.get());

  list.place();
  list.walk();
  size.j=list.get()->nbLine();
  size.i=list.get()->nbColumn();
  move(0,0);

  delete nameF;

  if(_on) setOn();
  if(_to) setTo();
  if(_under) setUnder();
}

Anim::Anim(Anim& a) throw()
  :Pageable(a.getSlow()),list(a.list),pos(a.pos.j,a.pos.i),decal(a.decal.j,a.decal.i),
  color(a.color),colored(a.colored),
   on(a.on),to(a.to),under(a.under),pal(a.pal),
   deleter(false),uppest(a.uppest)
{
}

Anim::~Anim() throw ()
{
  if(deleter) {
    list.place();
    while(list.walk()) delete list.get();
  }
}

int Anim::virtualWalk() throw()
{
  int aux=list.walk();
  if(aux==0) {
    unactivate();
    size.j=size.i=0;
  }
  return aux;
}

void Anim::setUppest() throw()
{
  uppest=true;
}

void Anim::setTo() throw()
{
  to=true;
  on=false;
  under=false;

  list.place();
  while(list.walk()) list.get()->noTakeLuz();
  list.place();
  list.walk();
}

void Anim::setOn() throw()
{
  on=true;
  to=false;
  under=false;

  list.place();
  while(list.walk()) list.get()->noTakeLuz();
  list.place();
  list.walk();
}

void Anim::setUnder() throw()
{
  under=true;
  on=false;
  to=false;
}

void Anim::setPalette(const PixelPalette& palette) throw()
{
  pal=&palette;
}

void Anim::setColor(Pixel col) throw()
{
  colored=true;
  color=col;
}

void Anim::restoreColor() throw()
{
  colored=false;
}

void Anim::paint(WindowPipe& windowPipe) throw()
{
  if(colored) 
    windowPipe.putColorTo(color,*list.get(),pos.j+decal.j,pos.i+decal.i);
  else 
    if(pal) windowPipe.putWithPalette(*list.get(),pos.j+decal.j,pos.i+decal.i,*pal);
    else if(to) windowPipe.putTo(*list.get(),pos.j+decal.j,pos.i+decal.i,uppest);//->paintTo(windowPipe,pos.j,pos.i);
    else if(on) windowPipe.putOn(*list.get(),pos.j+decal.j,pos.i+decal.i);
    else if(under) windowPipe.putUnder(*list.get(),pos.j+decal.j,pos.i+decal.i);
    else windowPipe.put(*list.get(),pos.j+decal.j,pos.i+decal.i);
}

void Anim::reinit() throw()
{
  //level osf
  list.place();
  list.walk();
  activate();
  size.j=list.get()->nbLine();
  size.i=list.get()->nbColumn();
}

void Anim::move(int j,int i) throw()
{
  pos.j=j;
  pos.i=i;
}

void Anim::getPos(int& j,int& i) const throw()
{
  j=pos.j;
  i=pos.i;
}

/*
void Anim::changeColor(List< Pixel > colorFrom,List< Pixel > colorTo) throw()
{
  list.place();
  while(list.walk()) list.get()->changeColor(colorFrom,colorTo);
  list.place();
  list.walk();
}
*/

void Anim::getSize(int& j,int& i) const throw()
{
  j=size.j;
  i=size.i;
}

void Anim::revert() throw()
{
  List<Sprite *> l=list;
  list.place();
  while(list.walk()) list.suppr();
  l.place();
  while(l.walk()) {
    list.place();
    list.put(l.get());
  }
}
