/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



class Background;

#ifndef __Background_h__
#define __Background_h__

#include "Window.h"
#include "WindowPipe.h"
#include "GraphicMode.h"
#include "String.h"
#include "Exception.h"
#include "Tji.h"
#include "PixelMatrix.h"
#include "UnderMaskMatrix.h"
#include "GifIn.h"

// ouvre un fichier et cree le 'WindowPipe' correspondant

/**
   A background groups a matrix of pixels and a WindowPipe from it to GraphicMode. So, it is easy to paint different animations on this background, through its WindowPipe.
   @see WindowPipe
   @see PixelMatrix
   @see UnderMaskMatrix
   @author David FAUTHOUX
 */
class Background {
 protected:
  ///
  PixelMatrix *data;
  ///
  UnderMaskMatrix *underMask;
  ///
  Window *winData;
  ///
  Window *winScreen;
  ///
  WindowPipe *windowPipe;
  ///
  Tji size;
  ///Flatten the gif, creating the UnderMaskMatrix and the shadow
  void aplatir(GifIn& gif,int jDecalShadow,int iDecalShadow) throw();
 public:
  ///
  Background(const String *nameFile,
	     GraphicMode& graphicMode,
	     int jDecalShadow,int iDecalShadow) throw (Exception);
  ///
  virtual ~Background() throw();

  ///Paint the matrix of the ground on the WindowPipe. Attention : do not refresh.
  virtual void paint() throw();

  ///
  WindowPipe *getWindowPipe() throw();

  ///
  int nbLine() const throw();
  ///
  int nbColumn() const throw();
};


#endif
