/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



class Bonhomme;

#ifndef __Bonhomme_h__
#define __Bonhomme_h__

#include "List.h"
#include "Ini.h"
#include "Bonhomme20.h"
#include "Bonhomme60.h"
#include "Sounder.h"
#include "Exception.h"
#include "StripContainer.h"
#include "TextPainter.h"
#include "Pixel.h"

// Cette classe ne sert qu'a regrouper le 'Bonhomme20 et le 'Bonhomme60'
// En effet, un bonhomme est decoupe en deux :
// Bonhomme20 : dans le jeu (le nom vient de la definition initiale
//              du bonhomme20 par des sprite de 20x20)
// Bonhomme60 : dans le page de choix du bonhomme (le nom vient de la
//              definition initiale du bonhomme60 par des sprite de 60x60)

// Cette classe gere egalement les equipes,
// gestion associee au travail de 'BonhommeContainer'.

/**
   This class groups the two parts of a playing character : the Bonhomme60 (when choosing the character) and the Bonhomme20 (playing in the game).
   @see Bonhomme20
   @see Bonhomme60
   @see BonhommeContainer
   @see PageBonhomme
   @see PageGame
   @author David FAUTHOUX
 */
class Bonhomme {
 protected:

  ///
  Bonhomme20 *bonhomme20;
  ///
  Bonhomme60 *bonhomme60;

  ///Id marked in .ini file
  const String *const id;

  ///
  Pixel color;
  /**
     The grey pixels of all the anims of the Bonhomme (Bonhomme20+Bonhomme60) are colored with the Bonhomme color (passing trough the palette).
     @see Anim
   */
  PixelPalette pixelPalette;

  ///Contains all the playing characters (and himself)
  List< Bonhomme * > playingAll; //contient aussi soi-meme
  ///Contains all the playing characters in his team (and himself)
  List< Bonhomme * > playingTeam;  //contient aussi soi-meme
  ///Contains all the playing characters not in his team (not himself)
  List< Bonhomme * > playingExceptTeam;

  ///
  int virtualWalk() throw();

 public:
  ///
  Bonhomme(Ini& ini,int numeroSection,
	   Sounder& sounder,
	   TextPainter& textPainter,
	   StripContainer& stripContainer)
    throw(Exception);

  ///
  Pixel getColor() const throw() { return color; }
  ///
  const String *getId() const throw();

  ///
  virtual ~Bonhomme() throw();

  ///
  Bonhomme20 *getBonhomme20() throw();
  ///
  Bonhomme60 *getBonhomme60() throw();

  ///
  List< Bonhomme * > *getPlayingAll() throw();//renvoie aussi lui

  ///
  List< Bonhomme * > *getPlayingTeam() throw();//renvoie aussi lui
  ///
  List< Bonhomme * > *getPlayingExceptTeam() throw();

  ///Called by BonhommeContainer
  void addPlaying(Bonhomme& bonhomme,bool inTeam) throw();

  ///Called by BonhommeContainer
  void supprAllPlayingAdded() throw();


  ///
  void setColor(Pixel color) throw();

};

#endif
