/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



class Bonhomme20;

#ifndef __Bonhomme20_h__
#define __Bonhomme20_h__

#include "ArtificialIntelligenceContainer.h"
#include "MoreMultiAnim.h"
#include "Ini.h"
#include "CounterContainer.h"
#include "Leaver.h"
#include "Boue.h"
#include "Trou.h"
#include "Sounder.h"
#include "Exception.h"
#include "CaseId.h"
#include "Bonhomme.h"
#include "Grille.h"
#include "Tji.h"
#include "StripContainer.h"
#include "TextPainter.h"
#include "Pixel.h"

/**
   The playing character in game. Inheritating from MoreMultiAnim, it can contains a lot of different anims, included very easily (example : an animation ON the Bonhomme20 when invincible).
   A remark : the name comes from the first animation create for a Bonhomme wich measures 20x20 pixels !
   @see Bonhomme
   @see PageGame
   @author David FAUTHOUX
 */
class Bonhomme20:public MoreMultiAnim,public CounterContainer {
 public:
  ///Moving way
  enum DirectionId {
    ///
    DIR_UP,
    ///
    DIR_DOWN,
    ///
    DIR_LEFT,
    ///
    DIR_RIGHT,
    ///
    NO_DIR
  };

  ///State of the Bonhomme20
  enum IdEtat {
    ///Normal
    ETAT_NORMAL,
    ///Burning
    ETAT_CRAME,
    ///Falling in ghost
    ETAT_BRISE,
    ///From burned to normal
    ETAT_REVIT,
    ///Burned
    ETAT_GRIS,
    ///Ghost
    ETAT_FANTOME,
    ///Resurrecting (live again !)
    ETAT_RESURRECT,
    ///Chicken
    ETAT_CHICKEN,
    ///Chicken to ghost
    ETAT_CHICKEN_BRISE,
    ///Teleporting
    ETAT_TELEPORT
  };

 protected:
  ///Reference
  Ini *const iniRef;
  ///Reference
  Bonhomme *const bonhommeRef;
  ///Reference
  Grille *grilleRef;
  ///Reference
  Sounder *const sounderRef;
  
  ///
  ArtificialIntelligenceContainer *artificialIntelligenceContainer;

  ///Sound reference, to be used with Sounder
  int soundCrameId;
  ///
  int soundBriseId;
  ///
  int soundChickenDeadId;
  ///
  int soundUpDownId;
  ///
  int soundGetOptionId;
  ///
  int soundWonId;


  ///Id for the time counters, to be used with Counter
  enum IdCount {
    ///
    COUNT_INVISIBLE,
    ///
    COUNT_INVINCIBLE,
    ///
    COUNT_TELEPORT,
    ///
    COUNT_TRAVERSE_MUR,
    ///
    COUNT_SURPUISSANT,
    ///
    COUNT_WIN95,
    ///
    COUNT_FEU_CUL,
    ///
    COUNT_SPEED,
    ///
    COUNT_CHICKEN
  



};

  ///Id for different puts
  enum IdFire {
    ///Normal bomb
    FIRE_NORMAL,
    ///Dynamite
    FIRE_DYNAMITE,
    ///Mine
    FIRE_MINE,
    ///Wall
    FIRE_MUR
  };

  ///State
  IdEtat etat;
  ///
  IdFire specialFire;
  ///A the beginning (catch in Ini)
  int initNbBombe;
  ///A the beginning (catch in Ini)
  int initPuissance;
  ///Number of bomb put simultaneously
  int nbBombe;
  ///Powerful of bomb
  int puissance;
  ///
  int nbTeleport;

  ///Normal speed moving
  int depNormal;
  ///Chicken speed moving
  int depChicken;
  ///Teleport speed moving
  int depTeleport;
  ///Win95 slowing
  int ralentissementWin95;
  ///Current speed moving
  int dep;


  ///Time used with Counter and IdCount : burned
  int tempsGris;
  ///Time chicken
  int tempsChicken;
  ///Time "flam leaved"
  int tempsFeuCul;
  ///Time quick moving
  int tempsSpeed;
  ///Time invincible
  int tempsInvincible;
  ///Time invisible
  int tempsInvisible;
  ///Time very powerful bombs
  int tempsSurpuissant;
  ///Time win95
  int tempsWin95;
  ///Time "pass through the walls"
  int tempsTraverseMur;
  ///Time "love hart"
  int tempsCoeur;
  ///Time teleporting
  int tempsTeleport;
  ///To count how many times the player has to press keys before wake up
  int cptTempsGris;
  

  ///Boolean state
  bool invincible;
  ///
  bool invisible;
  ///
  bool surpuissant;
  ///
  bool traverseMur;
  ///
  bool superBombe;
  ///
  bool  speed;


  ///Moving way
  DirectionId movingDir;
  ///
  bool firing0;
  ///
  bool firing1;

  ///Amount of puts
  int nbPose;

  ///Id for the anims, to be used with MultiAnim
  int animCenter;
  ///
  int animUp;
  ///
  int animDown;
  ///
  int animLeft;
  ///
  int animRight;
  ///
  int animCrame;
  ///
  int animGris;
  ///
  int animRevit;
  ///
  int animBrise;
  ///
  int animResurrect;
  ///
  int animFantomeCenter;
  ///
  int animFantomeUp;
  ///
  int animFantomeDown;
  ///
  int animFantomeLeft;
  ///
  int animFantomeRight;
  ///
  int animChickenCenter;
  ///
  int animChickenUp;
  ///
  int animChickenDown;
  ///
  int animChickenLeft;
  ///
  int animChickenRight;
  ///
  int animChickenBrise;
  ///
  int animTeleport;
  ///
  int animInvincible;
  ///
  int animSurpuissant;
  ///
  int animWin95;
  ///
  int animTraverseMur;
  ///
  int animTraceBoueUp;
  ///
  int animTraceBoueDown;
  ///
  int animTraceBoueLeft;
  ///
  int animTraceBoueRight;
  
  ///Multiplier (square size)
  Tji mult;

  ///Used for moving
  struct Tlimit {
    ///
    Tji pos;
    ///
    Tji size;
  };
  ///
  Tlimit limit;

  ///Used by Grille to call or not action() of Case
  Tji sizeAction;
  ///Used for moving
  struct Tpix {
    ///
    Tji pos[7];
    ///
    Tji cpt[7];
    ///
    Tji init[7];
    ///
    Tji realPos;
    ///
    Tji decalPaint;
  };
  ///Used for moving and painting
  Tpix pix;
  

  ///Up or down the walls
  int niv;

  ///Used for dynamite
  bool blowUpDynamite;

  ///
  int nbBombeSaveSurpuissant;

  //
  
  ///Used to leave animations behind the Bonhomme
  List< Leaver* > leaverList;

  ///Mud
  Boue *boue;
  ///Hole when you are killed and irritated !
  Trou *trou;

  ///Used for help (passing strips)
  StripContainer *const stripContainerRef;
  ///
  const String *stripUpWall;
  ///
  const String *stripDownWall;
  ///
  const String *stripCrame;
  ///
  const String *stripBrise;
  ///
  const String *stripPicoteDuPainDur;

  ///Used to paint something...
  TextPainter *const textPainterRef;

  //

  ///Called by Walkable
  int virtualWalk() throw();

  ///Used for moving
  void pixMoveUp() throw();
  ///
  void pixMoveDown() throw();
  ///
  void pixMoveLeft() throw();
  ///
  void pixMoveRight() throw();

  ///Used for moving
  void privateMoveUp() throw();
  ///
  void privateMoveDown() throw();
  ///
  void privateMoveLeft() throw();
  ///
  void privateMoveRight() throw();

  ///
  void privateFire0() throw();
  ///
  void privateFire1() throw();

  ///Initialisation
  void reinitAllValues() throw();

  ///To add a new time counter, automatically runned (at the end, CounterContainer calls endOfCount(...)
  void beginOfCount(int value) throw();
  ///Called by CounterContainer
  void endOfCount(int value) throw();

  ///To add easyly a Leaver
  void addLeaver(CaseId::Id c) throw();
  ///
  void supprLeaver(CaseId::Id c) throw();

  ///For chicken
  void ecraserChicken() throw();
  ///
  void testerEcrasementChicken() throw();

 public:
  ///
  Bonhomme20(Ini& ini,int numeroSection,Bonhomme& bonhomme,
	     Sounder& sounder,
	     TextPainter& textPainter,
	     StripContainer& stripContainer,
	     const PixelPalette& pixelPalette) throw(Exception);
  ///
  virtual ~Bonhomme20() throw();

  ///
  void paint(WindowPipe& windowPipe) throw();

  ///
  void init(int j,int i,int jMult,int iMult,Grille& grille) throw(Exception);
  //pour grille

  ///
  void changeArtificialIntelligence() throw();
  ///returns 0 if no AI
  const String *getArtificialIntelligenceId() throw();

  ///
  void move(int j,int i) throw();

  ///
  void getSize(int& j,int& i) const throw();//size du sprite

  ///
  void setLimit(int j,int i,int jSize,int iSize) throw();

  ///
  void moveUp() throw();
  ///
  void moveDown() throw();
  ///
  void moveLeft() throw();
  ///
  void moveRight() throw();
  ///
  void doNotMove() throw();
  ///
  void fire0() throw();
  ///
  void fire1() throw();
  ///
  void doNotFire0() throw();
  ///
  void doNotFire1() throw();

  ///
  DirectionId getDirection() const throw();

  ///
  void decNbPose() throw();

  //

  ///
  void incNbBombe() throw();
  ///
  void incPuissance() throw();
  ///
  void incNbTeleport() throw();

  ///
  void setTraverseMur() throw();
  ///
  void setSurpuissant() throw();

  ///
  void setFireDynamite() throw();
  ///
  void setFireMine() throw();
  ///
  void setFirePoseMur() throw();

  ///
  void setChicken() throw();
  ///
  void setFeuCul() throw();
  ///
  void setSpeed() throw();
  ///
  void setInvisible() throw();
  ///
  void setInvincible() throw();
  ///
  void setSuperBombe() throw();//bombe qui traverse les murs
  ///
  void setWin95() throw();
  
  ///Says true if was down
  bool setUpNiv() throw();//renvoie true si pas deja en niv==1
  ///Says true if was up
  bool setDownNiv() throw();

  ///
  void cramer() throw();
  ///
  void cramerIfVivant() throw();
  ///Says true if has moved
  bool setEtatGrisIfMove() throw();//renvoie true si a bouge
  ///Says true if was dead
  bool resurrect() throw();//renvoie true si avant, etait mort

  /**
     The Bonhomme has two groups of test pixels : here, the tolerant (usually the hardly reachable) - for flams
     @see Grille
  */
  void getPosTolerantAction(int& niv,int j[4],int i[4]) const throw();
  /**
     The Bonhomme has two groups of test pixels : here, the non tolerant (usually the easily reachable) - for options
     @see Grille
  */
  void getPosAction(int& niv,int j[4],int i[4]) const throw();

  //
  ///
  void reinit() const throw();//rien 

  //
  ///Enables to take option ?
  bool capableToTakeOption(bool playSound=true) throw();

  //
  ///
  int getPuissance() const throw();
  ///
  bool isSuperBombe() const throw();

  //
  ///
  bool haveToBlowUpDynamite() const throw();

  //
  ///
  bool isChicken() const throw();
  ///
  IdEtat getEtat() const throw();

  //
  ///Reference to the container
  Bonhomme *getBonhomme() throw();
  ///
  const Bonhomme *getBonhomme() const throw();

  //
  ///
  int getTempsCoeur() const throw();

  //
  ///
  Trou *getTrou() throw();

  //
  ///Set mud
  void setBoueColleeAuxPieds() throw();

  //
  ///
  void playSoundWon() throw();

  ///
  Pixel getNormalColor() const throw();
  ///
  void printStrip(const String& s) throw();
  ///
  void printStrip(PreStrip::Id id) throw();

  ///
  void paintText(WindowPipe& windowPipe,const String& text) throw();

  ///
  virtual void paintNight(WindowPipe& windowPipe) throw();
};
#endif
