/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



#include "Bonhomme60.h"
#include "Grille.h"
#include "Anim.h"
#include "LoopAnim.h"

#define DEFAULT_ANIM_SLOW 0
#define DEFAULT_POS_ANIM_DEPART_J 0
#define DEFAULT_POS_ANIM_DEPART_I 0
#define DEFAULT_POS_ANIM_ARRIVEE_J 0
#define DEFAULT_POS_ANIM_ARRIVEE_I 0
#define DEFAULT_DECAL_ANIM_MORE_J 0
#define DEFAULT_DECAL_ANIM_MORE_I 0
#define DEFAULT_SLOW_ANIM_MORE 0

Bonhomme60::Bonhomme60(Ini& ini,int numeroSection,
		       Bonhomme& bonhomme,
		       TextPainter& textPainter,
		       Sounder& sounder,
		       const PixelPalette& pixelPalette) throw(Exception)
  :bonhommeRef(&bonhomme),
   sounderRef(&sounder),
   textPainterRef(&textPainter),
   moreActivated(false),
   animMore(0),loopAnimMore(0),
   ecra(false)
{
  int slowChoice=ini.getWithDefault("[Bonhomme]",numeroSection,"anim choice slow",0,DEFAULT_ANIM_SLOW);

  animPrincipale=add(*new LoopAnim(*ini.get("[Bonhomme]",numeroSection,
					    "anim principale",0),slowChoice,ini.getListSeparator()));
  animDepart=add(*new Anim(*ini.get("[Bonhomme]",numeroSection,
				    "anim depart",0),slowChoice,ini.getListSeparator()));
  animEcrasement=add(*new Anim(*ini.get("[Bonhomme]",numeroSection,
					"anim ecrasement",0),slowChoice,ini.getListSeparator()));
  animArrivee=add(*new LoopAnim(*ini.get("[Bonhomme]",numeroSection,
					 "anim arrivee",0),slowChoice,ini.getListSeparator()));
  
  posAnimDepart.j=ini.getWithDefault("[Bonhomme]",numeroSection,
				     "pos anim depart j",0,
				     DEFAULT_POS_ANIM_DEPART_J);
  posAnimDepart.i=ini.getWithDefault("[Bonhomme]",numeroSection,
				     "pos anim depart i",0,
				     DEFAULT_POS_ANIM_DEPART_I);
  posAnimArrivee.j=ini.getWithDefault("[Bonhomme]",numeroSection,
				     "pos anim arrivee j",0,
				     DEFAULT_POS_ANIM_ARRIVEE_J);
  posAnimArrivee.i=ini.getWithDefault("[Bonhomme]",numeroSection,
				     "pos anim arrivee i",0,
				     DEFAULT_POS_ANIM_ARRIVEE_I);

  //

  List< const String * > *listMore=ini.getListWithoutException("[Bonhomme]",numeroSection,
							       "anim more");
  if(listMore) {
    int cpt=0;
    listMore->place();
    while(listMore->walk()) {
      Anim *a=new Anim(*listMore->get(),
		       ini.getWithDefault("[Bonhomme]",numeroSection,
					  "slow anim more",cpt,
					  DEFAULT_SLOW_ANIM_MORE),
		       ini.getListSeparator());
      a->move(posAnimDepart.j+ini.getWithDefault("[Bonhomme]",numeroSection,
						 "decal anim more j",cpt,
						 DEFAULT_DECAL_ANIM_MORE_J),
	      posAnimDepart.i+ini.getWithDefault("[Bonhomme]",numeroSection,
						 "decal anim more i",cpt,
						 DEFAULT_DECAL_ANIM_MORE_I));
      a->setPalette(pixelPalette);
      animMore.add(*a);
      allMore.put(a);
      cpt++;
    }
  }

  listMore=ini.getListWithoutException("[Bonhomme]",numeroSection,
					"loop anim more");
  if(listMore) {
    int cpt=0;
    listMore->place();
    while(listMore->walk()) {
      LoopAnim *a=new LoopAnim(*listMore->get(),
			       ini.getWithDefault("[Bonhomme]",numeroSection,
						  "slow loop anim more",cpt,
						  DEFAULT_SLOW_ANIM_MORE),
			       ini.getListSeparator());
      a->move(posAnimDepart.j+ini.getWithDefault("[Bonhomme]",numeroSection,
						 "decal loop anim more j",cpt,
						 DEFAULT_DECAL_ANIM_MORE_J),
	      posAnimDepart.i+ini.getWithDefault("[Bonhomme]",numeroSection,
						 "decal loop anim more i",cpt,
						 DEFAULT_DECAL_ANIM_MORE_I));
      a->setPalette(pixelPalette);
      loopAnimMore.add(*a);
      allMore.put(a);
      cpt++;
    }
  }

  //

  int channelReserved=sounderRef->reserveChannel();

  soundDepartId=sounderRef->add(*ini.getList("[Bonhomme]",numeroSection,
					     "sound depart",
					     "[Bonhomme]",numeroSection),
				channelReserved);
  soundEcrasementId=sounderRef->add(*ini.getList("[Bonhomme]",numeroSection,
						 "sound ecrasement",
						 "[Bonhomme]",numeroSection),
				    channelReserved);

  //
  setPalette(pixelPalette);

  move(posAnimDepart.j,posAnimDepart.i);
  unactivateMore();
}

Bonhomme60::~Bonhomme60() throw()
{
}

void Bonhomme60::move(int j,int i) throw()
{
  MultiAnim::move(j,i);
}

void Bonhomme60::reinit() throw()
{
  ecra=false;
  move(posAnimDepart.j,posAnimDepart.i);
  on(animPrincipale);
  if(moreActivated) activateMore();
  else unactivateMore();
}

void Bonhomme60::ecraser() throw()
{
  ecra=true;
  on(animDepart);
  move(posAnimDepart.j,posAnimDepart.i);
  sounderRef->play(soundDepartId);
  loopAnimMore.unactivate();
}

int Bonhomme60::virtualWalk() throw()
{
  if(!MultiAnim::virtualWalk()) {
    if(getCurrent()==animDepart) {
      on(animEcrasement);
      move(posAnimArrivee.j,posAnimArrivee.i);
      sounderRef->play(soundEcrasementId);
    }
    else if(getCurrent()==animEcrasement) on(animArrivee);
  }
  if(animMore.isActive()) animMore.walk();
  if(loopAnimMore.isActive()) loopAnimMore.walk();
  return 1;
}

bool Bonhomme60::isEcrase() const throw()
{
  return ecra;
}


void Bonhomme60::activateMore() throw()
{
  animMore.activate();
  animMore.reinit();
  loopAnimMore.activate();
  loopAnimMore.reinit();
  moreActivated=true;
}

void Bonhomme60::unactivateMore() throw()
{
  loopAnimMore.unactivate();
  moreActivated=false;
}

void Bonhomme60::paint(WindowPipe& windowPipe) throw()
{
  MultiAnim::paint(windowPipe);
  if(animMore.isActive()) animMore.paint(windowPipe);
  if(loopAnimMore.isActive()) loopAnimMore.paint(windowPipe);

  if(moreActivated) {
    const String *idAI=getBonhomme()->getBonhomme20()->getArtificialIntelligenceId();
    if(idAI) paintText(windowPipe,*idAI);
    else paintText(windowPipe,*(getBonhomme()->getId()));
  }
}

//

Bonhomme *Bonhomme60::getBonhomme() throw()
{
  return bonhommeRef;
}

const Bonhomme *Bonhomme60::getBonhomme() const throw()
{
  return bonhommeRef;
}

Pixel Bonhomme60::getNormalColor() const throw()
{
  return getBonhomme()->getColor();
}


void Bonhomme60::paintText(WindowPipe& windowPipe,const String& text) throw()
{
  textPainterRef->paint(windowPipe,
			pos.j-textPainterRef->nbLine(),
			pos.i,
			text,
			getNormalColor());
}
