/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



class BonhommeContainer;

#ifndef __BonhommeContainer_h__
#define __BonhommeContainer_h__

#include "Bonhomme.h"
#include "Player.h"
#include "Keyable.h"
#include "Score.h"
//#include "FlashPalette.h"
#include "List.h"
#include "Sounder.h"
#include "Exception.h"
#include "StripContainer.h"
#include "TextPainter.h"

// Ce container cree et gere des 'Bonhomme' (passage des actions (cf
// PageableContainer pour le passage d'actions), gestion d'equipe 
// en association avec la classe 'Bonhomme').

/**
   Manage the containing an relationship between Bonhomme. Teams, actions passing, key passing, painting...
   @see Bonhomme
   @author David FAUTHOUX
 */
class BonhommeContainer:public Keyable {
 protected:

  ///
  List< Bonhomme * > listBonhomme;
  ///Bonhommes not chosen
  List< Bonhomme * > listBonhommeAlone;
  ///
  List< Player * > listPlayer;

  ///
  List< Bonhomme * > listBonhommePlaying;
  ///
  List< Player * > listPlayerPlaying;
  
  ///Totally amount of players
  int nbPlayer;
  ///
  int maxNbPlayer;

  ///
  Ini *const iniRef;

  ///
  bool teamSet;
  ///
  int amountPlayerInTeam;

  //

  ///
  Bonhomme *findBonhomme(const String& b) throw();
  ///Verify the Bonhomme of the Player
  void checkBonhomme(Player& p) throw();

  ///
  void setBonhommePlaying() throw();
  ///
  void setPlayingAdded() throw();

  //%%  void alignLeftToRight() throw();
  ///Walking in the Bonhomme list
  Bonhomme *findRight(Bonhomme *b) throw();
  ///Walking in the Bonhomme list
  Bonhomme *findLeft(Bonhomme *b) throw();
  
 public:
  ///
  BonhommeContainer(Ini& ini,Sounder& sounder,
		    TextPainter& textPainter,
		    StripContainer& stripContainer)
    throw(Exception);
  ///
  virtual ~BonhommeContainer() throw();

  ///
  void virtual incNbPlayer() throw();
  ///
  void virtual decNbPlayer() throw();
  ///
  int virtual getNbPlayer() const throw();
  ///
  int virtual getMaxNbPlayer() const throw();

  ///Set or unset team
  void setOrUnsetTeam() throw();
  ///
  bool isTeamSet() const throw();

  ///
  List< Bonhomme * > *getAllBonhomme() throw();
  ///
  List< Bonhomme * > *getBonhommePlaying() throw();

  ///
  List< Player * > *getAllPlayer() throw();
  ///
  List< Player * > *getPlayerPlaying() throw();

  //%%  void testChangeBonhomme() throw();

  //
  ///
  bool eachPlayerHasBonhomme() throw();//cf PageBonhomme

  //
  ///Passed the action to the Bonhomme according to laws (playing ? choosing ?...)
  void keyAction(const Key& key) throw();

};

#endif
