/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



#include "CCFileModifier.h"
#include "util.h"

CCFileModifier::CCFileModifier(const String& className,
			       List<const String *>& markList) throw(Exception)
  :TextFileModifier(className+*new String(".cc"),markList),
   className(&className)
{
}

CCFileModifier::~CCFileModifier() throw()
{
}


bool CCFileModifier::addInheritance(const String& motherClass,const String& init) throw()
{
  place();
  if(!after(*className+*new String("::")+*className)) {
    message(*new String("Cannot find '")+*className+*new String("::")+*className+*new String("'"));
    return false;
  }
  limit();
  if(!after("{")) {
    message("Cannot find '{' of constructor");
    unlimit();
    return false;
  }
  before("{");
  jumpBlank();
  limit();
  if(before(":")) {

    after(":");
    for(;;) {
      //All class must have a first capital letter !!
      if(!isCapital((*nextWord())[0])) break;
      if(!after(",")) {
	insert(*new String(",")+motherClass+*new String("(")+init+*new String(")"));
	unlimit();
	return true;
      }
    }
    insert(motherClass+*new String("(")+init+*new String("),"));
    unlimit();
    return true;
  }

  after("{");//bloque par limit
  insert(*new String(":")+motherClass+*new String("(")+init+*new String(")"));
  unlimit();
  return true;
}


bool CCFileModifier::placeAtEnd() throw()
{
  place();
  while(walk());
  return true;
}

bool CCFileModifier::placeAfterSharps() throw() //diese #
{
  place();
  for(;;) {
    if(*nextWord()=="#define") {
      after("#define");
      goEndLine();
    }
    else if(*nextWord()=="#include") {
      after("#include");
      goEndLine();
    }
    else break;
  }

  return true;
}

bool CCFileModifier::addParameter(const String& name,const String& init) throw()
{
  place();
  if(!after(*className+*new String("::")+*className)) {
    message(*new String("Cannot find '")+*className+*new String("::")+*className+*new String("'"));
    return false;
  }
  limit();
  if(!after("{")) {
    message("Cannot find '{' of constructor");
    unlimit();
    return false;
  }
  limit();
  before("{");
  jumpBlank();
  keepPosition();
  if(before(":")) {
    recallPosition();
    insert(*new String(",")+name+*new String("(")+init+*new String(")"));
    unlimit();
    return true;
  }
  recallPosition();
  insert(*new String(":")+name+*new String("(")+init+*new String(")"));
  unlimit();
  return true;


  /*
  if(placeAtEnd()) {
    List<const String *> l;
    l.put(&(protection+*new String(":")));
    l.put(new String("/ **"));
    comment.place();
    while(comment.walk()) l.put(comment.get());
    l.put(new String("* /"));
    l.put(&(type+*new String(" ")+name+*new String(";")));
    insert(l);
    return true;
  }
  return false;
  */
}

String *CCFileModifier::simplifyAndCompleteDeclaration(const String& declaration,const String& completion) throw()
{
  String *d=simplifyBlank(declaration);
  List<char> dl;
  int i=0;
  while(i<d->size() && (*d)[i]!='(') i++;
  if(i>=d->size()) {
    message("Cannot find '(' of declaration");
    delete d;
    return 0;
  }
  i--;
  while(i>=0 && (*d)[i]!=' ' && !isSeparator((*d)[i])) i--;

  for(int k=0;k<=i;k++) dl.put((*d)[k]);
  i++;
  for(int k=0;k<className->size();k++) dl.put((*className)[k]);

  for(int k=0;k<completion.size();k++) dl.put(completion[k]);
  
  dl.put(':');
  dl.put(':');
  while(i<d->size()) {
    if((*d)[i]=='=') {
      while(i<d->size() && (*d)[i]!=',' && (*d)[i]!=')') i++;
      if(i>=d->size()) {
	message("Cannot find ')' of declaration");
	delete d;
	return 0;
      }
    }
    dl.put((*d)[i]);
    i++;
  }
  delete d;
  return new String(dl);
}

bool CCFileModifier::addFunction(const String& declaration,List< const String * >& body) throw()
{
  if(!placeAtEnd()) return false;
  String *d=simplifyAndCompleteDeclaration(declaration,*new String(""));
  if(!d) return false;
  List<const String *> l;
  l.put(d);
  l.put(new String("{"));
  body.place();
  while(body.walk()) l.put(body.get());
  l.put(new String("}"));
  insert(l);
  return true;

  /*

  if(!after(*d)) {
    message("Cannot find "+*d);
    delete d;
    return false;
  }
  limit();
  if(!after("{")) {
    message("Cannot find '{' of "+*d);
    delete d;
    return false;
  }
  if(before("::")) {
    message("Cannot find '{' of "+*new String(dl));
    return false;
  }
  after("{");
  //on est sur d'etre apres la { de la function !
  keepPosition();
  findCloseBrace();
  */
  /*
[...]


  if(after(placeAtEnd()) {
    List<const String *> l;
    l.put(&(protection+*new String(":")));
    l.put(new String("/ **"));
    comment.place();
    while(comment.walk()) l.put(comment.get());
    l.put(new String("* /"));
    l.put(&(declaration+*new String(";")));
    insert(l);
    return true;
  }
  return false;
*/
}

bool CCFileModifier::addFunctionTemplate(const String& classTemplate,const String& declaration,List< const String * >& body) throw()
{
  if(!placeAtEnd()) return false;
  String *d=simplifyAndCompleteDeclaration(declaration,*new String("<")+classTemplate+*new String(">"));
  if(!d) return false;
  List<const String *> l;
  l.put(&(*new String("template < class ")+classTemplate+*new String(" >")));
  l.put(d);
  l.put(new String("{"));
  body.place();
  while(body.walk()) l.put(body.get());
  l.put(new String("}"));
  insert(l);
  return true;
}

bool CCFileModifier::auxCompleteFunctionAfter(const String& declaration,List<const String *>& body,List<const String *>& afterLines,const String& completion) throw()
{
  String *d=simplifyAndCompleteDeclaration(declaration,completion);
  if(!d) return false;

  place();
  if(!after(*d)) {
    message(*new String("Cannot find ")+*d);
    delete d;
    return false;
  }
  if(!after("{")) {
    message(*new String("Cannot find '{' of ")+*d);
    delete d;
    return false;
  }
  keepPosition();
  if(!findCloseBrace()) {
    message(*new String("Cannot find '}' of ")+*d);
    delete d;
    return false;
  }
  limit();
  recallPosition();
  if(!after(afterLines)) {
    message(*new String("Cannot find lines in ")+*d);
    delete d;
    return false;
  }
  insert(body);
  delete d;
  return true;
}

bool CCFileModifier::auxCompleteFunctionBefore(const String& declaration,List<const String *>& body,List<const String *>& beforeLines,const String& completion) throw()
{
  String *d=simplifyAndCompleteDeclaration(declaration,completion);
  if(!d) return false;

  place();
  if(!after(*d)) {
    message(*new String("Cannot find ")+*d);
    delete d;
    return false;
  }
  if(!after("{")) {
    message(*new String("Cannot find '{' of ")+*d);
    delete d;
    return false;
  }
  limit();
  if(!findCloseBrace()) {
    message(*new String("Cannot find '}' of ")+*d);
    delete d;
    return false;
  }
  if(!before(beforeLines)) {
    message(*new String("Cannot find lines in ")+*d);
    delete d;
    return false;
  }
  insert(body);
  delete d;
  return true;
}

bool CCFileModifier::auxCompleteFunctionAtBeginning(const String& declaration,List<const String *>& body,const String& completion) throw()
{
  String *d=simplifyAndCompleteDeclaration(declaration,completion);
  if(!d) return false;

  place();
  if(!after(*d)) {
    message(*new String("Cannot find ")+*d);
    delete d;
    return false;
  }
  if(!after("{")) {
    message(*new String("Cannot find '{' of ")+*d);
    delete d;
    return false;
  }
  insert(body);
  delete d;
  return true;
}

bool CCFileModifier::auxCompleteFunctionAtEnd(const String& declaration,List<const String *>& body,const String& completion) throw()
{
  String *d=simplifyAndCompleteDeclaration(declaration,completion);
  if(!d) return false;

  place();
  if(!after(*d)) {
    message(*new String("Cannot find ")+*d);
    delete d;
    return false;
  }
  if(!after("{")) {
    message(*new String("Cannot find '{' of ")+*d);
    delete d;
    return false;
  }
  limit();
  if(!findCloseBrace()) {
    message(*new String("Cannot find '}' of ")+*d);
    delete d;
    return false;
  }
  before("}");
  jumpBlank();
  insert(body);
  delete d;
  return true;
}



bool CCFileModifier::addDefine(const String& id,const String& value) throw()
{
  if(placeAfterSharps()) {
    insert(*new String("#define ")+id+*new String(" ")+value);
    return true;
  }
  return false;
}

bool CCFileModifier::addInclude(const String& file) throw()
{
  if(placeAfterSharps()) {
    insert(*new String("#include \"")+file+*new String("\""));
    return true;
  }
  return false;
}

/*
bool CCFileModifier::findEnum(const String& enumName) throw()
{
  place();
  do {
    if(!after("enum")) {
      message("Cannot find 'enum'");
      return false;
    }
    if(!after(*nextWord())) {
      message("Cannot find enum name");
      return false;
    }
  } while(*nextWord()==";");

  if(!after("{")) {
    message("Cannot find '{'");
    return false;
  }

  return true;
}

bool CCFileModifier::addInEnumAfter(const String& enumName,const String& id,List<const String *>& afterLines,List<const String *>& comment) throw()
{
  if(findEnum(enumName) && after(afterLines)) {
    List<const String *> l;
    l.put(new String("/ **"));
    comment.place();
    while(comment.walk()) l.put(comment.get());
    l.put(new String("* /"));
    l.put(&(id+*new String(",")));
    insert(l);
    return true;
  }
  return false;
}

bool CCFileModifier::addInEnumBefore(const String& enumName,const String& id,List<const String *>& beforeLines,List<const String *>& comment) throw()
{
  if(findEnum(enumName) && after(beforeLines) && before(beforeLines)) {
    List<const String *> l;
    l.put(new String("/ **"));
    comment.place();
    while(comment.walk()) l.put(comment.get());
    l.put(new String("* /"));
    l.put(&(id+*new String(",")));
    insert(l);
    return true;
  }
  return false;
}

bool CCFileModifier::addInEnumAtEnd(const String& enumName,const String& id,List<const String *>& comment) throw()
{
  if(findEnum(enumName) && after("};") && before("};")) {
    jumpBlank();
    List<const String *> l;
    l.put(new String("/ **"));
    comment.place();
    while(comment.walk()) l.put(comment.get());
    l.put(new String("* /"));
    l.put(&(*new String(",")+id));
    insert(l);
    return true;
  }
  return false;
}
*/

void CCFileModifier::message(const String& m) throw()
{
  cerr<<CC_FILE_MODIFIER_ERROR<<":"<<(const char*)m<<"\n";cerr.flush();
}

/*
bool CCFileModifier::addSubclass(const String& protection,List< const String * >& wallDeclaration) throw()
{
  if(placeAtEnd()) {
    List<const String *> l;
    l.put(&(protection+*new String(":")));
    int cpt=0;
    wallDeclaration.place();
    while(wallDeclaration.walk()) cpt++;
    wallDeclaration.place();
    for(int k=0;k<cpt-1;k++) {
      wallDeclaration.walk();
      l.put(wallDeclaration.get());
    }
    wallDeclaration.walk();
    l.put(&(*wallDeclaration.get()+*new String(";")));
    insert(l);
    return true;
  }
  return false;
}
*/

bool CCFileModifier::addRoughlyAfter(List< const String * >& body,List< const String * >& afterLines) throw()
{
  place();
  if(after(afterLines)) {
    insert(body);
    return true;
  }
  return false;
}

bool CCFileModifier::addRoughlyBefore(List< const String * >& body,List< const String * >& beforeLines) throw()
{
  place();
  if(after(beforeLines) && before(beforeLines)) {
    insert(body);
    return true;
  }
  return false;
}

bool CCFileModifier::addRoughlyAtBeginning(List< const String * >& body) throw()
{
  place();
  insert(body);
  return true;
}

bool CCFileModifier::addRoughlyAtEnd(List< const String * >& body) throw()
{
  place();
  while(walkIgnoring());
  insert(body);
  return true;
}


bool CCFileModifier::completeFunctionAfter(const String& declaration,List<const String *>& body,List<const String *>& afterLines) throw()
{
  return auxCompleteFunctionAfter(declaration,body,afterLines,*new String(""));
}

bool CCFileModifier::completeFunctionBefore(const String& declaration,List<const String *>& body,List<const String *>& beforeLines) throw()
{
  return auxCompleteFunctionBefore(declaration,body,beforeLines,*new String(""));
}

bool CCFileModifier::completeFunctionAtBeginning(const String& declaration,List<const String *>& body) throw()
{
  return auxCompleteFunctionAtBeginning(declaration,body,*new String(""));
}

bool CCFileModifier::completeFunctionAtEnd(const String& declaration,List<const String *>& body) throw()
{
  return auxCompleteFunctionAtEnd(declaration,body,*new String(""));
}

bool CCFileModifier::completeFunctionTemplateAfter(const String& classTemplate,const String& declaration,List<const String *>& body,List<const String *>& afterLines) throw()
{
  return auxCompleteFunctionAfter(declaration,body,afterLines,*new String("<")+classTemplate+*new String(">"));
}

bool CCFileModifier::completeFunctionTemplateBefore(const String& classTemplate,const String& declaration,List<const String *>& body,List<const String *>& beforeLines) throw()
{
  return auxCompleteFunctionBefore(declaration,body,beforeLines,*new String("<")+classTemplate+*new String(">"));
}

bool CCFileModifier::completeFunctionTemplateAtBeginning(const String& classTemplate,const String& declaration,List<const String *>& body) throw()
{
  return auxCompleteFunctionAtBeginning(declaration,body,*new String("<")+classTemplate+*new String(">"));
}

bool CCFileModifier::completeFunctionTemplateAtEnd(const String& classTemplate,const String& declaration,List<const String *>& body) throw()
{
  return auxCompleteFunctionAtEnd(declaration,body,*new String("<")+classTemplate+*new String(">"));
}

