/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



class Case;

//pour rajouter une option:
// 1) rajouter l'Id dans CaseGestion
// 1bis) rajouter le #include dans CaseGestion
// 2) rajouter la classe derivee dans le Makefile
// 3) rajouter le code pour new dans CaseGestion
// 4) rajouter le chemin du fichier anim dans le .ini

#ifndef __Case_h__
#define __Case_h__

#include "MultiAnim.h"
#include "Sounder.h"
#include "Grille.h"
#include "Bonhomme20.h"
#include "CaseId.h"

// La classe mere 'Case' demande a ses filles de definir des methodes 
// qui specifie la case : traversable, stackable...
// Ces methodes ont un sens assez global pour pouvoir deriver 'Case' selon
// les besoins.

// Exemples : une flamme est 'restricted',
// une option est 'stackable', un mur n'est pas 'stackable'.

/**
   Main class to define the properties of a square in the game array.
   - Case is a french word for a square in an array.
   This class define a default square of the Grille (in english : Wire or Array) in a game.
   The goal is : when you want to define a new specification of a square, you must derive the methods 'traversable', 'blowable', etc.
   For example, a MurIndestructible (in english : indestructible wall) is not 'traversable'.
   @author David FAUTHOUX
   @see Grille
   @see Bonhomme20
   @see CaseId
 */
class Case:public MultiAnim {
 private:
  ///Forbidden to copy like that (function declared but not defined)
  Case(Case& c) throw();
 protected:
  /**
     Define the level position of the Case in the array of the game (managed by a Grille) - in french, level is 'niveau' -
  */
  int niv;
  /**
     Define the vertical position of the Case in the array of the game (managed by a Grille).
  */
  int jc;
  /**
     Define the horizontal position of the Case in the array of the game (managed by a Grille).
  */
  int ic;
 public:
  /**
     The constructor does nothing.
  */
  Case() throw();
  /**
     The destructor does nothing.
  */
  virtual ~Case() throw();
  
  /**
     Sets the position of the Case in the Grille, but not interacts with the Grille (only sets the positions - niv,jc,ic members).
     @param niv the level (normally, the Grille manages two levels, so 0 <= niv < 2)
     @param j the vertical position in the array of the game (managed by a Grille)
     @param i the horizontal position in the array
     @param jMult this integer defines the vertical size of a square in the screen (normally 20), it is necessary in order to print the Case on the screen
     @param iMult this integer defines the horizontal size of a square in the screen (normally 20), it is necessary in order to print the Case on the screen
  */
  virtual void move(int niv,int j,int i,int jMult,int iMult) throw();

  /**
     Returns the positions of the Case :
     @param niv level in the Grille
     @param j vertical position in the array of the game (managed by a Grille)
     @param i horizontal position in the array of the game (managed by a Grille)
   */
  virtual void getPos(int& niv,int& j,int& i) const throw();

  /**
     Used to define the properties of the Case :
     'traversable' with force==false means : a Bonhomme can pass through the Case without any problem.
     'traversable' with force==true means : a Bonhomme can pass through the Case forcing (with special option for example).
     @param force enable to give two senses for this function (the weak sense and the hard sense)
     @return true
   */
  bool virtual traversable(bool force) throw();
  //force==true si traverseMur

  /**
     Function called when the Bonhomme 'bonhomme' passes on the Case (the positions used are the 'action' positions of the Bonhomme - normally, the most external)
     @param bonhomme the Bonhomme who walks on this Case and perform the action
   */
  virtual void action(Bonhomme20& bonhomme) throw();

  /**
     Function called when the Bonhomme 'bonhomme' passes on the Case (the positions used are the 'tolerantAction' positions of the Bonhomme - normally, the most internal)
     @param bonhomme the Bonhomme who walks on this Case and perform the action
   */
  virtual void tolerantAction(Bonhomme20& bonhomme) throw();

  /**
     Used to say to the manager (usually Grille) if this Case has to be removed.
   */
  bool virtual toBeBlownUp() const throw();
  //true qd finBombe et walk==0

  /**
     Another property of the Case : 'blowable' means 'can blow up' (a bomb for example).
     @return false
   */
  bool virtual blowable() throw(); //explose qd true, rien qd false

  /**
     A function wich must be derived to define CaseBombe for example : defines special properties of the Case.
   */
  bool virtual getTraverseMur() const throw(); //sert pour bombe,finBombe...

  /**
     A function wich must be derived to define CaseBombe for example : defines special properties of the Case.
   */
  int virtual getPuissance() const throw(); //idem

  /**
     Used when the Case is on another in the Grille (it hides the other). Used when a wall is broken, to activate the options under it.
     @return false
   */
  bool virtual haveToActivateUnder() throw();

  /**
     Used to unactivate an option under a wall for example.
     @return false
   */
  bool virtual haveToUnactivateUnder() throw();


  /**
     Another property of the Case : 'restricted' means 'only another restricted Case can be put on it' (a flam for example).
     @return false
   */
  bool virtual restricted() const throw();
  //on ne peut poser qu'un restricted dessus


  /**
     Another property of the Case : 'stackable' means 'only a restricted Case can be put on it' (a wall is not stackable for example, a flam is restricted).
     @return true
   */
  bool virtual stackable() const throw();
  //si oui: on peut tout poser,si non: que restricted

  /**
     Another property of the Case : 'high' means 'occupies entirely the level in the Grille' (a wall for example).
     @return false
   */
  bool virtual high() const throw();//haut ou pas (cf bonhomme20)

  /**
     Another property of the Case : 'fallable' means 'when there is nothing under it, the Case falls' (a bomb for example, not a flam).
     @return true
   */
  bool virtual fallable() const throw();

  /**
     Another property of the Case : 'paintedUp' means 'painted on the others Case and on the Bonhommes'
     @return false
   */
  bool virtual paintedUp() throw();

};



#endif

