/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



class CaseMurDestructible;

#ifndef __CaseMurDestructible_h__
#define __CaseMurDestructible_h__

#include "CaseMur.h"

/**
   The destructible wall Case.
   It breaks down on a call to traversable(true) or blowable()
   @author David FAUTHOUX
   @see Sounder
   @see Bonhomme20
   @see Anim
 */
class CaseMurDestructible:public CaseMur {
 protected:
  ///
  const int aniNormalId;
  ///
  const int   aniBriseId;
  ///
  const int   aniTasId;
  ///
  const int   aniEnvoleId;
  ///
  int step;//0:Normal, 1:Brise...
 public:
  /**
     Defines entirely the Case : the animations used.
     @param aniNormal the animation while doing nothing
     @param aniBrise the animation when breaking down
     @param aniTas the animation while doing nothing and being broken
     @param aniEnvole the animation when the dust fly away
   */
  CaseMurDestructible(Anim *aniNormal,Anim *aniBrise,
		      Anim *aniTas,Anim *aniEnvole) throw();

  ///
  ~CaseMurDestructible() throw() {}

  /**
     If force==true, the wall breaks down.
   */
  bool virtual traversable(bool force) throw();

  /**
     The Case has to walk in order to manage the passage from the different animations.
   */
  int virtual virtualWalk() throw();
  //(0=>tas=>poussiere=>envole) + ::virtualWalk

  /**
     Breaks the wall.
   */
  bool virtual blowable() throw(); //step=0 => on(Brise)

  /**
     Ups the bonhomme.
   */
  virtual void action(Bonhomme20& bonhomme) throw();//nivUp

  /**
     Only when broken.
   */
  bool stackable() const throw();

  /**
     Only when not broken.
   */
  bool high() const throw();
};

#endif

