/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



class CaseMurRefermable;

#ifndef __CaseMurRefermable_h__
#define __CaseMurRefermable_h__

#include "CaseMur.h"

/**
   The open/close wall Case.
   It opens on a call to traversable(true) or blowable() and closes itself.
   @author David FAUTHOUX
   @see Sounder
   @see Bonhomme20
   @see Anim
 */
class CaseMurRefermable:public CaseMur {
 protected:
  ///
  const int aniNormalId;
  ///
  const int aniOpeningId;
  ///
  const int aniClosingId;
  ///
  int step;//0:Normal, 1:Opening...
 public:
  /**
     Defines entirely the Case : the animations used.
     @param aniNormal the animation while doing nothing
     @param aniOpening the animation while opening
     @param aniClosing the animation while closing
   */
  CaseMurRefermable(Anim *aniNormal,Anim *aniOpening,Anim *aniClosing) throw();

  ///
  ~CaseMurRefermable() throw() {}

  /**
     If force==true, the wall opens.
   */
  bool virtual traversable(bool force) throw();

  /**
     The Case has to walk in order to manage the passage from the different animations.
   */
  int virtual virtualWalk() throw();
  //(0=>Closing ou Normal) + ::virtualWalk

  /**
     Opens the wall.
   */
  bool virtual blowable() throw(); //step=0 => on(Opening)

  /**
     Ups the bonhomme.
   */
  virtual void action(Bonhomme20& bonhomme) throw();//nivUp

  /**
     Only when broken.
   */
  bool stackable() const throw();

  /**
     Only when not broken.
   */
  bool high() const throw();
};

#endif

