/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



#include "Coeur.h"
#include "Anim.h"
#include "LoopAnim.h"

#define DEFAULT_SLOW_COEUR 10

#define DEFAULT_POS_J 0
#define DEFAULT_POS_I 0


int Coeur::virtualWalk() throw()
{
  if(bonhommeRef) {
    int j,i;
    bonhommeRef->getPos(j,i);
    if(pos.j<j) pos.j++;
    if(pos.j>j) pos.j--;
    if(pos.i<i) pos.i++;
    if(pos.i>i) pos.i--;
    move(pos.j,pos.i);
  }
  inc();
  if(!MultiAnim::virtualWalk()) {
    bonhommeRef=0;
    unactivate();
  }
  return 1;
}

Coeur::Coeur(Ini& ini) throw(Exception)
  :bonhommeRef(0),
   haveToBeMoved(false),
   animBegin(MultiAnim::add(*new LoopAnim(*ini.get("[Woman]",0,
						  "loop anim coeur",0),0,
					  ini.getListSeparator()))),
   animEnd(MultiAnim::add(*new Anim(*ini.get("[Woman]",0,
					     "anim coeur destroy",0),0,
				    ini.getListSeparator())))
{
  setOn();

  setSlow(ini.getWithDefault("[Woman]",0,"slow coeur",0,DEFAULT_SLOW_COEUR));
  initPos.j=ini.getWithDefault("[Woman]",0,"pos coeur j",0,DEFAULT_POS_J);
  initPos.i=ini.getWithDefault("[Woman]",0,"pos coeur i",0,DEFAULT_POS_I);
  unactivate();
}

void Coeur::setBonhomme(const Bonhomme20 *b) throw()
{
  if(!bonhommeRef && b) haveToBeMoved=true;
  bonhommeRef=b;
  on(animBegin);
  if(bonhommeRef) {
    CounterContainer::add(bonhommeRef->getTempsCoeur(),0);
    activate();
  } else unactivate();
}

bool Coeur::isOn(const Bonhomme20& b) const throw()
{
  if(&b!=bonhommeRef) return false;
  int bj,bi;
  b.getPos(bj,bi);
  int bsj,bsi;
  b.getSize(bsj,bsi);
  int sj,si;
  getSize(sj,si);
  int pj=pos.j+sj/2;
  int pi=pos.i+si/2;
  if(pj>=bj && pj<bj+bsj && pi>=bi && pi<=bi+bsi) return true;
  if(pos.j>=bj && pos.j<bj+bsj && pos.i>=bi && pos.i<=bi+bsi) return true;
  if(pos.j+sj>=bj && pos.j+sj<bj+bsj
     && pos.i+si>=bi && pos.i+si<=bi+bsi) return true;
  return false;
}

void Coeur::endOfCount(int value) throw()
{
  on(animEnd);
}

void Coeur::paint(WindowPipe& windowPipe) throw()
{
  if(haveToBeMoved) {
    haveToBeMoved=false;
    int j,i,sj,si;
    windowPipe.getFrom(j,i,sj,si);
    move(initPos.j+j,initPos.i+i);
  }
  MultiAnim::paint(windowPipe);
}
