/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



class CounterContainer;

#ifndef __CounterContainer_h__
#define __CounterContainer_h__

#include "Countable.h"
#include "Counter.h"
#include "LoopCounter.h"
#include "List.h"

// 'CounterContainer' gere une liste de 'Counter'
// Cette classe doit etre derivee : la classe fille definie 'endOfCount'
// qui est appelee a chaque fin de compteur, avec la valeur du compteur (value)
// Il est donc interessant de donner a chaque compteur des valeurs differentes
// pour n'ecrire qu'une seule fonction 'endOfCount' qui gere plein de trucs.

/**
   Manages a list of Counter. When a Counter is finished, it calls endOfCount() with the value of the Counter : so you must inherite this class and define endOfCount() wich switch the value. (example Bonhomme20-endOfCount())
   @see Counter
   @author David FAUTHOUX
 */
class CounterContainer:public Countable {
 protected:
  ///
  List< Counter* > list;
  ///
  virtual void endOfCount(int value) throw()=0; //protected
  ///
  void supprIfPresent(int value) throw();
 public:
  ///
  CounterContainer() throw();
  ///
  virtual ~CounterContainer() throw();
  ///
  void virtual add(int t,int value) throw();
  ///When the Counter is finished, it is reactivated
  void virtual addLoop(int t,int value) throw();
  ///
  void virtual reset() throw();
  ///Call it to run this class (it is not a thread)
  void virtual inc() throw();
};



#endif
