/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



/*
  Code of my friend Ulrich ROISSARD - 1999
 */

#include "Exploser.h"
#include "MatriceTrans.h"

Exploser::Exploser(int width,int height,int deltaX,int deltaY) throw()
{
  nbTriangles=2*(height/deltaY)*(width/deltaX);
  triangle=new Triangle3D[nbTriangles];
  
  translationOrig=new Vector[nbTriangles];
  translation=new Vector[nbTriangles];
  rotation=new Vector[nbTriangles];
  
  int p=0;
  for (int y=0;y+deltaY<=height;y+=deltaY)
    for (int x=0;x+deltaX<=width;x+=deltaX) {
      triangle[p][0]=Vector(-deltaX/2.0,-deltaY/2.0,0);
      triangle[p][0].setS(x);
      triangle[p][0].setT(y);
      
      triangle[p][1]=Vector(deltaX/2.0,-deltaY/2.0,0);
      triangle[p][1].setS(x+deltaX);
      triangle[p][1].setT(y);
      
      triangle[p][2]=Vector(deltaX/2.0,deltaY/2.0,0);
      triangle[p][2].setS(x+deltaX);
      triangle[p][2].setT(y+deltaY);
      
      translationOrig[p][0]=x+deltaX/2.0-width/2;
      translationOrig[p][1]=y+deltaY/2.0-height/2;
      translationOrig[p][2]=0;
      
      p++;
      
      triangle[p][0]=Vector(-deltaX/2.0,-deltaY/2.0,0);
      triangle[p][0].setS(x);
      triangle[p][0].setT(y);
      
      triangle[p][1]=Vector(deltaX/2.0,deltaY/2.0,0);
      triangle[p][1].setS(x+deltaX);
      triangle[p][1].setT(y+deltaY);
      
      triangle[p][2]=Vector(-deltaX/2.0,deltaY/2.0,0);
      triangle[p][2].setS(x);
      triangle[p][2].setT(y+deltaY);

      translationOrig[p][0]=x+deltaX/2.0-width/2;
      translationOrig[p][1]=y+deltaY/2.0-height/2;
      translationOrig[p][2]=0;
      
      p++;
    }
}

int Exploser::getNbTriangles() const throw()
{
  return nbTriangles;
}

void Exploser::setTranslation(int i,const Vector &v) throw()
{
  translation[i]=v;
}

void Exploser::setRotation(int i,const Vector &v) throw()
{
  rotation[i]=v;
}

Triangle3D Exploser::getTriangle(int i) const throw()
{
  MatriceTrans mat;
  Triangle3D tmp;
  
  mat.rotate(rotation[i][0],rotation[i][1],rotation[i][2]);
  mat.translate(translationOrig[i][0],translationOrig[i][1],translationOrig[i][2]);
  mat.translate(translation[i][0],translation[i][1],translation[i][2]);
  
  for (int k=0;k<3;k++) {
    tmp[k]=mat*triangle[i][k];
    tmp[k].setS(triangle[i][k].getS());
    tmp[k].setT(triangle[i][k].getT());
  }
  
  return tmp;
}

Vector Exploser::getOrigin(int i) const throw()
{
  return translationOrig[i];
}

