/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



#ifndef __File_h__
#define __File_h__

#include <stdio.h>

#include "String.h"
#include "Exception.h"

///
class File {
 public:
  ///
  enum OpenAutorisation {
    ///
    READ_ONLY,
    ///
    WRITE_ONLY
  };
  ///
  enum SeekCode {
    ///
    FROM_BEGIN,
    ///
    FROM_END,
    ///
    FROM_CURRENT
  };
 private:
  ///
  OpenAutorisation autorisation;
  ///
  int size;
  ///
  int pos;
  ///
  FILE *f;
  ///
  const char *name;

  ///
  void calculerSize() throw(Exception);

 public:
  ///
  void open(const char *name,OpenAutorisation autorisation)
    throw (Exception);
  ///
  void close() throw ();
  ///
  int getSize() const throw ();

  ///
  void write(const char *buffer,int size) throw (Exception);
  ///Writes while str[i]!=0
  void write(const char *str) throw (Exception); //jusqu'a un 0
  ///
  void write(char c) throw (Exception);
  ///
  void write(short c) throw (Exception);
  ///
  void write(int c) throw (Exception);
  ///
  void read(char *buffer,int size) throw (Exception);
  ///Read while read!=0 (str must be allocated)
  void read(char *& str) throw (Exception);//jusqu'a un 0
  ///
  void read(char& c) throw (Exception);
  ///
  void read(short& c) throw (Exception);
  ///
  void read(int& c) throw (Exception);

  ///s construct by this function, but must be deleted by caller, this function stop when has found a \n or the end of the file
  void read(String *& s) throw (Exception);

  ///
  void seek(int offset,SeekCode code) throw (Exception);

  ///
  File(const char *name,OpenAutorisation autorisation=READ_ONLY)
    throw (Exception);
  ///
  File(const String& name,OpenAutorisation autorisation=READ_ONLY)
    throw (Exception);
  ///
  ~File() throw ();

  ///
  bool eof() const throw ();

  ///
  static bool exists(const String& nameFile) throw();

  ///Renames the file, but do not remove any file. Prefers concatenate toAdd anoter time. Returns the new name.
  static String *renameCheck(const String& nameFile,const String& toAdd) throw(Exception);
};

#endif


