/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



#include "Font.h"
#include "GifIn.h"

#define NB_CHAR 256

#define DEFAULT_FONT_COLOR toPixel(maxGreyValue,maxGreyValue,maxGreyValue,0)
#define DEFAULT_BACKGROUND_COLOR toPixel(0,0,0,0)


Font::Font(Ini& ini) throw(Exception)
  :alphabet(NB_CHAR),
   translation(NB_CHAR),
   fontColor(ini.getWithDefaultColor("[Strip]",0,
				     "default font color",0,
				     DEFAULT_FONT_COLOR)),
   backgroundColor(ini.getWithDefaultColor("[Strip]",0,
					   "default background color",0,
					   DEFAULT_BACKGROUND_COLOR))
{

  const String *nameFile=ini.get("[Strip]",0,"font",0);
  GifIn gif(*nameFile);

  gif.place();
  gif.walk();
  PixelMatrix *a=gif.get()->getMask();

  size.j=a->nbLine();
  size.i=a->nbColumn()/alphabet.size();
  if(size.i==0) throw Exception(ERROR_INVALID_FILE,*nameFile);

  for(int k=0;k<alphabet.size();k++) {
    alphabet[k]=new Sprite(size.j,size.i);
    for(int j=0;j<size.j;j++)
      for(int i=0;i<size.i;i++)
	if((*a)[j][i+k*size.i]==PIXEL_MASK_FULL) 
	  alphabet[k]->set(fontColor,j,i);
    	else
	  alphabet[k]->set(backgroundColor,j,i);
  }

  //initialise translation
  for(int k=0;k<translation.size();k++) {
    translation[k]=k;
  }
  for(int k=0;k<translation.size();k++) {
    char aux[2]={k,0};
    const String *s=ini.getWithoutException("[Font translation]",0,aux,0);
    try {
      if(s) translation[k]=s->toInt();
    } catch(Exception) {
      translation[k]=k;
    }
  }

}

void Font::setColor(Pixel oldFontColor) throw()
{
  for(int k=0;k<alphabet.size();k++)
    for(int j=0;j<size.j;j++)
      for(int i=0;i<size.i;i++)
	if((*(alphabet[k]->getData()))[j][i]==oldFontColor)
	  alphabet[k]->set(fontColor,j,i);
  	else
	  alphabet[k]->set(backgroundColor,j,i);
}

void Font::copyOther(const Font& font) throw()
{
  for(int k=0;k<alphabet.size();k++)
    alphabet[k]=new Sprite(*(font.alphabet[k]));
  for(int k=0;k<translation.size();k++)
    translation[k]=font.translation[k];
}

Font::Font(const Font& font,Pixel fontColor,Pixel backgroundColor) throw()
  :alphabet(NB_CHAR),
   translation(NB_CHAR),
   size(font.size),fontColor(fontColor),backgroundColor(backgroundColor)
{
  copyOther(font);
  setColor(font.fontColor);
}

Font::Font(const Font& font,Pixel fontColor) throw()
  :alphabet(NB_CHAR),
   translation(NB_CHAR),
   size(font.size),fontColor(fontColor),backgroundColor(font.backgroundColor)
{
  copyOther(font);
  setColor(font.fontColor);
}

Font::Font(const Font& font) throw()
  :alphabet(NB_CHAR),
   translation(NB_CHAR),
   size(font.size),
   fontColor(font.fontColor),backgroundColor(font.backgroundColor)
{
  copyOther(font);
}

Font::~Font() throw()
{
  for(int k=0;k<alphabet.size();k++) {
    delete alphabet[k];
  }
}

const Sprite *Font::get(char c) const throw()
{
  return alphabet[translation[(unsigned char)c]];
}

int Font::nbLine() const throw()
{
  return size.j;
}

int Font::nbColumn() const throw()
{
  return size.i;
}

Pixel Font::getFontColor() const throw()
{
  return fontColor;
}

Pixel Font::getBackgroundColor() const throw()
{
  return backgroundColor;
}



