/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



class Ini;

#ifndef __Ini_h__
#define __Ini_h__

#include "ValueString.h"
#include "SectionAssoc.h"
#include "IniFileAnalyser.h"
#include "Pixel.h"

// 'Ini' : Cette grosse classe ouvre un fichier et l'analyse
// pour creer une liste associant une '[section]'
// a une liste d'association id/valeur.

// Elle est utilise par enormement de classes du jeu.


// Remarque : l'appel --withDefault cree l'id s'il n'est pas present
// (et la section si besoin).



/**
   Initialisation class. Used by a lot of classes. It reads a text file with structure [Section]= ... id:value
   Very useful to change the game without recompile it !!
   @author David FAUTHOUX
 */
class Ini:public IniFileAnalyser {
 protected:
  ///
  List< SectionAssoc * > section;

  ///
  ValueString *currentValueString;
  ///
  const String *tmpId;

  ///
  List< ValueString * > *currentListValueString;
  ///
  bool continueValue;

  ///
  char listSeparator;

  ///
  virtual void putSection(const String& s) throw(Exception);
  ///
  virtual void putId(const String& s) throw(Exception);
  ///
  virtual void putValue(const String& s) throw(Exception);
  ///
  virtual void putComment(const String& s) throw(Exception);

  ///
  virtual void initForSave() throw(Exception);
  ///
  virtual const String *getSection() throw(Exception);
  ///
  virtual bool isSectionUsed() throw();
  ///
  virtual bool walkSection() throw(Exception);
  ///
  virtual const String *getId() throw(Exception);
  ///
  virtual bool isIdUsed() throw();
  ///
  virtual bool walkId() throw(Exception);
  ///
  virtual const String *getValue() throw(Exception);
  ///
  virtual bool walkValue() throw(Exception);

  ///
  virtual const String *getComment() throw(Exception);

  ///
  const String *auxGet(const String& s,int numero,
		       const String& id,int idNumero,bool ex)
    throw (Exception);

 public:

  ///
  Ini(const String& name) throw (Exception);
  ///
  ~Ini() throw ();

  ///
  bool isPresentSection(const String& s,int numero) throw ();

  ///Throw Exception if not found
  const String *get(const String& s,int numero,
		    const String& id,int idNumero=0) throw (Exception);
  //numero de la section qd plusieurs
  ///Returns 0 if not found
  const String *getWithoutException(const String& s,int numero,
				    const String& id,int idNumero=0)
    throw (Exception);


  ///Set the id to stringDefault value and returns it if not found
  const String *getWithDefault(const String& s,int numero,
			 const String& id,int idNumero,
			 const String& stringDefault) throw(Exception);
  ///idem with int
  int getWithDefault(const String& s,int numero,
		     const String& id,int idNumero,
		     int intDefault) throw(Exception);
  /**
     Sets withStar if star(*) found, used for more complicated int
     @see Grille
  */
  int getWithDefault(const String& s,int numero,
		     const String& id,int idNumero,
		     int intDefault,bool& withStar) throw(Exception);


  ///
  void set(const String& s,int numero,
	   const String& id,int idNumero,const String& val) throw (Exception);
  //numero de la section qd plusieurs

  ///Add an id after all the identical ids
  void setWithMaximumNumber(const String& s,int numero,
			    const String& id,const String& val) throw (Exception);

  ///
  void setInAllSection(const String& s,
		       const String& id,int idNumero,const String& val) throw (Exception);

  //

  ///Returns all the values with the same id, if not found in section s, search in section defaultSection. If s not found, or, if list in it empty, searchs in defaultSection, throws Exception if not found, or list in it empty.
  List< const String* > *getList(const String& s,int numero,
			 const String& id,
			 const String& defaultSection,int numeroD=0)
    throw(Exception);

  ///See getLis, but returns 0 if defaultSection not found, or list empty
  List< const String* > *Ini::getListWithoutException(const String& s,int numero,
						    const String& id)
    throw(Exception);


  ///See getWithDefault, but with Pixel (list of 3 int - example : 120,252,0)
  Pixel getWithDefaultColor(const String& s,int numero,
			    const String& id,int idNumero,
			    Pixel pixelDefault) throw(Exception);

  ///See getWithDefault, but with KEY (int) for read directly key name (NaN) (F.)
  int getWithDefaultKey(const String& s,int numero,
			    const String& id,int idNumero,
			    int keyDefault) throw(Exception);

  ///Return an int list (form 12,2,0,5,1, etc)
  List< int > *getWithDefaultIntList(const String& s,int numero,
				     const String& id,int idNumero,
				     const String& intListDefault) throw(Exception);

  ///Return an int list (form "SPACE,A,F1,etc")
  List< int > *getWithDefaultKeyList(const String& s,int numero,
				     const String& id,int idNumero,
				     const String& intListDefault) throw(Exception);


  ///Usually ","
  char getListSeparator() const throw() { return listSeparator; }
};

#endif



