/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



#include <iostream.h>
#include <time.h>
#include <stdlib.h>
#include "Main.h"
#include "display.h"
#include "util.h"
#include "Key.h"

//2 seconds
#define LAP_TIME_INIT 2

#define DEFAULT_KEY_HALT SCROLLOCK
//#define DEFAULT_KEY_VBL 99

#define DEFAULT_TIME_WAIT 5000

#define DEFAULT_MUSIK_VOLUME 100

#define DEFAULT_SCREEN_BLOW 10

#define ERROR_FATAL "Bomb o bomb cannot continue, sorry... but maybe you can put it right...\n\n"

//"linux"
#define DEFAULT_MESSAGE_LINUX "linux"
//"help"
#define DEFAULT_MESSAGE_HELP "help"

#define DEFAULT_MESSAGE_HELP_OFF "The help is not active, you can switch it on by typing 'help'"
#define DEFAULT_MESSAGE_HELP_ON "The help is active, you can switch it off by typing 'help'"

#define DEFAULT_DECAL_SHADOW -10
//

#define DEFAULT_SLOW 15

#define DEFAULT_MESSAGE_TIME "time"
#define DEFAULT_TIME_PRINT_BEGIN "You have played "
#define DEFAULT_TIME_PRINT_END " minutes"



Main::Main(const char *nameFileIni,bool willClean) throw()
  :Walkable(0),
   clean(willClean),
   incCptClock(0),
   nextCptClock(0),
   halted(false),//vbl(false),
   keyer(0),
   //%%   signaler(0),
   pageChanger(0),graphicMode(0),
   palette(0),sounder(0),ini(0),
   nameFile(nameFileIni),
   initClockTime(0),
   timeMessageBegin(""),
   timeMessageEnd("")
{
  setSlow(DEFAULT_SLOW);

  try {

    cout<<"Loading...\n";cout.flush();

    srand(time(0));
    ini=new Ini(*new String(nameFileIni));

    //
    message(*ini->getWithDefaultKeyList("[Message]",0,"linux",0,DEFAULT_MESSAGE_LINUX), MESSAGE_LINUX);
    message(*ini->getWithDefaultKeyList("[Message]",0,"help",0,DEFAULT_MESSAGE_HELP), MESSAGE_HELP);
    message(*ini->getWithDefaultKeyList("[Message]",0,"time",0,DEFAULT_MESSAGE_TIME),MESSAGE_TIME);

    linuxMessage=ini->getListWithoutException("[Message]",0,"linux print");

    timeMessageBegin=*(ini->getWithDefault("[Message]",0,"time print begin",0,DEFAULT_TIME_PRINT_BEGIN));
    timeMessageEnd=*(ini->getWithDefault("[Message]",0,"time print end",0,DEFAULT_TIME_PRINT_END));

    keyHalt=ini->getWithDefaultKey("[Global]",0,"key halt",0,DEFAULT_KEY_HALT);
    //    keyVbl=ini->getWithDefault("[Global]",0,"key vbl",0,DEFAULT_KEY_VBL);

    sounder=new Sounder;

    cout<<"Loading Musiks...";cout.flush();
    soundMusik=sounder->add(*(ini->getListWithoutException("[Sound]",0,
							   "musik")),false);

    cout<<"ok\n";cout.flush();
    sounder->setVolume(soundMusik,
		       ini->getWithDefault("[Sound]",0,"musik volume",0,
					  DEFAULT_MUSIK_VOLUME));

    add(sounder->play(soundMusik),0);

    timeWaitMusik=ini->getWithDefault("[Sound]",0,"silence between musik",0,
				     DEFAULT_TIME_WAIT);

    graphicMode=new GraphicMode(ini->getWithDefault("[Palette]",0,"decal shadow",
						    0,DEFAULT_DECAL_SHADOW));

    palette=new FlashPalette(*ini,*graphicMode);
    //    palette->load(*ini->get("[Global]",0,"palette",0));

    keyer=new Keyer();

    //%%    signaler=new Signaler();


    blower=new Blower(*ini,graphicMode->nbLine(),graphicMode->nbColumn());

    //si l'on veut, on peut changer la redirection de la sortie
    graphicMode->redirect(*blower);

    // on pourrait rajouter la ligne :
    //  graphicMode->redirect(un autre Redirector);
    // qui sauvegarderait l'ecran par exemple

    //
    List< const String *> *runningMessage=ini->getListWithoutException("[Running]",0,"message");
    if(runningMessage && !runningMessage->empty()) {
      /*      int cpt=0;
	      runningMessage->place();
	      while(runningMessage->walk()) cpt++;
	      int r=rand()%cpt;*/
      int r=ini->getWithDefault("[Running]",0,"amount runs",0,0);
      runningMessage->place();
      runningMessage->walk();
      while(r>0) {
	r--;
	if(!runningMessage->walk()) {
	  runningMessage->place();
	  runningMessage->walk();
	  //plus one, not to print again the first tip
	  if(!runningMessage->walk()) {
	    runningMessage->place();
	    runningMessage->walk();
	  }
	}
      }
      cout<<(const char*)(*runningMessage->get());
      cout<<"\n";cout.flush();
    }

    //

    textPainter=new TextPainter(*ini);
    stripContainer=new StripContainer(*textPainter,*ini,*graphicMode);

    const String *strHelpOn=ini->getWithDefault("[Strip]",0,"message help on",0,
					*new String(DEFAULT_MESSAGE_HELP_ON));
    const String *strHelpOff=ini->getWithDefault("[Strip]",0,"message help off",0,
					*new String(DEFAULT_MESSAGE_HELP_OFF));

    if(!ini->getWithDefault("[Strip]",0,"activate",0,1)) {
      cout<<(const char*)*strHelpOff<<"\n";cout.flush();
      stripContainer->unactivate();
    }
    else {
      cout<<(const char*)*strHelpOn<<"\n";cout.flush();
    }


    //

    pageChanger=new PageChanger(*this,*ini,*graphicMode,*palette,
				*sounder,*keyer,
				*blower,
				*textPainter,*stripContainer,
				clean);



    //    keyer->keyboardOut();
    graphicMode->on();
    //    keyer->keyboardIn();
    keyer->add(*pageChanger);
    keyer->add(*this);

    cout<<"Loaded.\n";cout.flush();

  }
  catch(Exception) {
    cout<<ERROR_FATAL;cout.flush();
    halted=true;
  }
  nextCptClock=clock()+CLOCKS_PER_SEC*LAP_TIME_INIT;

  initClockTime=clock();
}

Main::~Main() throw()
{
  try {

    if(keyer) delete keyer;
    if(graphicMode) delete graphicMode;

    //%%    delete signaler;
    if(pageChanger) delete pageChanger;
    if(stripContainer) delete stripContainer;
    if(textPainter) delete textPainter;
    if(blower) delete blower;
    if(palette) delete palette;
    if(sounder) delete sounder;

    if(ini) {

      if(stripContainer)
	if(stripContainer->isActive()) ini->set("[Strip]",0,"activate",0,1);
	else ini->set("[Strip]",0,"activate",0,0);

      List<const String *> *lString=
	ini->getListWithoutException("[Sound]",0,"musik from");
      if(!lString->empty()) {
	cout<<"Musik from :\n";
	lString->place();
	while(lString->walk()) cout<<"\t"<<(const char*)(*(lString->get()))<<"\n";
	cout.flush();
      }

      lString=ini->getListWithoutException("[Graphics]",0,"graphic from");
      if(!lString->empty()) {
	cout<<"Graphic from :\n";
	lString->place();
	while(lString->walk()) cout<<"\t"<<(const char*)(*(lString->get()))<<"\n";
	cout.flush();
      }

      cout<<"Coded by :\n";
      lString=ini->getList("[Global]",0,"coded by","[Global]");
      lString->place();
      while(lString->walk()) cout<<"\t"<<(const char*)(*(lString->get()))<<"\n";
      cout.flush();

      int r=ini->getWithDefault("[Running]",0,"amount runs",0,0);
      ini->set("[Running]",0,"amount runs",0,r+1);


      if(!halted) {
	if(clean)
	  ini->saveCleaned();
	else
	  ini->save();
	cout<<nameFile<<" auto-modified and saved.\n";
      } else cout<<"Halted.\n";
      cout<<"Bye...\n";cout.flush();
//      cout<<"...maybe [alt-f7] to return to X...\n";cout.flush();

      delete ini;

      cout<<"\n";cout.flush();
    }
  }
  catch(Exception) {
    cout<<ERROR_FATAL;cout.flush();
  }
}

int Main::virtualWalk() throw()
{
  if(halted) return 0;

  if(clock()<nextCptClock) return 1;
  nextCptClock+=incCptClock;
  if(nextCptClock<clock()) nextCptClock=clock();

  try {
    int t=1;
    //    if(vbl) waitRetrace();

    //    if(vbl) setPalette(0,rgbMax(),0,0);
    t=pageChanger->walk();
    if(stripContainer->isActive()) stripContainer->walk();
    //    if(vbl) setPalette(0,0,rgbMax(),0);
    if(clock()<nextCptClock) {
      pageChanger->paint();
      if(stripContainer->isActive()) stripContainer->paint();
      //    if(vbl) setPalette(0,0,0,rgbMax());
      palette->walk();
      palette->refresh();

      graphicMode->refresh();
      //    if(vbl) setPalette(0,0,0,0);
    }

    keyer->walk();
    inc();

    if(halted) return 0;
    return t;
  }
  catch(Exception) {
    cout<<ERROR_FATAL;cout.flush();
    halted=true;
    return 0;
  }
}

void Main::keyAction(const Key& key) throw()
{
  if(key.getCode()==keyHalt) halted=true;
  /*%%%%%
  if(key.getCode()==keyVbl)
    if(key.isUp()) vbl=false;
    else vbl=true;
  */
  work(key);
}

void Main::endOfCount(int value) throw()
{
  if(halted) return;
  switch(value) {
  case 0:
    add(timeWaitMusik,1);
    break;
  case 1:
    add(sounder->play(soundMusik),0);
    break;
  default:
    break;
  }
}

/*%%
void Main::signalAction(int sig) throw()
{
  cout<<"signal got\n";
  cout.flush();
}
*/

void Main::action(int id) throw()
{
  switch(id) {
  case MESSAGE_TIME:
    if(!stripContainer->isActive()) stripContainer->activate();
    stripContainer->add(timeMessageBegin+*new String((clock()-initClockTime)/(CLOCKS_PER_SEC*60))+timeMessageEnd);
    break;
  case MESSAGE_HELP:
    if(stripContainer->isActive()) stripContainer->unactivate();
    else stripContainer->activate();
    break;
  case MESSAGE_LINUX:
    if(linuxMessage && !linuxMessage->empty()) {
      if(!stripContainer->isActive()) stripContainer->activate();
      stripContainer->add(*(linuxMessage->get()));
      if(!linuxMessage->walk()) {
	linuxMessage->place();
	linuxMessage->walk();
      }
    }
    break;
  }
}

void Main::setSlow(int slow) throw()
{
  if(slow==0) slow=10;
  incCptClock=(slow*CLOCKS_PER_SEC)/1000;
}

