/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



#include "MultiAnim.h"
#include "error.h"

MultiAnim::MultiAnim() throw()
  :Pageable(0),tab(0),length(0),activeAnim(0),
   color(toPixel(0,0,0,0)),colored(false),_on(false),to(false),under(false),
   pal(0),
   currentAnim(0),
   uppest(false)
{
  move(0,0);
}

MultiAnim::~MultiAnim() throw()
{
  for(int i=0;i<length;i++) delete tab[i];
  delete tab;
}

void MultiAnim::setOn() throw()
{
  _on=true;
  under=false;
  to=false;
  for(int i=0;i<length;i++) tab[i]->setOn();
}

void MultiAnim::setUppest() throw()
{
  uppest=true;
  for(int i=0;i<length;i++) tab[i]->setUppest();
}

void MultiAnim::setTo() throw()
{
  _on=false;
  under=false;
  to=true;
  for(int i=0;i<length;i++) tab[i]->setTo();
}

void MultiAnim::setUnder() throw()
{
  _on=false;
  under=true;
  to=false;
  for(int i=0;i<length;i++) tab[i]->setUnder();
}

void MultiAnim::setPalette(const PixelPalette& palette) throw()
{
  pal=&palette;
  for(int i=0;i<length;i++) tab[i]->setPalette(palette);
}


void MultiAnim::paint(WindowPipe& windowPipe) throw()
{
  if(activeAnim)
    if(activeAnim->isActive()) activeAnim->paint(windowPipe);
}

void MultiAnim::paintNight(WindowPipe& windowPipe) throw()
{
  if(activeAnim)
    if(activeAnim->isActive()) activeAnim->paintNight(windowPipe);
}

void MultiAnim::setColor(Pixel col) throw()
{
  color=col;
  colored=true;
  for(int i=0;i<length;i++)
    tab[i]->setColor(color);
}

void MultiAnim::restoreColor() throw()
{
  colored=false;
  for(int i=0;i<length;i++)
    tab[i]->restoreColor();
}

void MultiAnim::reinit() throw()
{
  if(length>0) activeAnim=tab[0];
  for(int i=0;i<length;i++)
    tab[i]->reinit();
}

int MultiAnim::virtualWalk() throw()
{
  if(activeAnim) return activeAnim->walk();
  else return 1;
}

void MultiAnim::keyAction(const Key& key) throw()
{
  if(activeAnim) activeAnim->keyAction(key);
}

void MultiAnim::move(int j,int i) throw()
{
  pos.j=j;
  pos.i=i;
  for(int i=0;i<length;i++)
    tab[i]->move(pos.j,pos.i);
}

void MultiAnim::getPos(int& j,int& i) const throw()
{
  j=pos.j;
  i=pos.i;
}


int MultiAnim::add(Anim& anim) throw()
{
  Anim **aux=new Anim*[length+1];
  for(int i=0;i<length;i++) aux[i]=tab[i];
  aux[length]=&anim;

  anim.move(pos.j,pos.i);
  if(colored) anim.setColor(color);
  if(_on) anim.setOn();
  if(uppest) anim.setUppest();
  if(to) anim.setTo();
  if(under) anim.setUnder();
  if(pal) anim.setPalette(*pal);

  delete tab;
  tab=aux;
  length++;
  if(length==1) on(length-1);
  return length-1;
}

void MultiAnim::on(int id) throw (Exception)
{
  if(id<0 || id>=length) throw Exception(ERROR_MULTIANIM_INVALID_ID);
  currentAnim=id;
  activeAnim=tab[id];
  activeAnim->reinit();
}

Anim *MultiAnim::getAnim(int id) throw (Exception)
{
  if(id<0 || id>=length) throw Exception(ERROR_MULTIANIM_INVALID_ID);
  return tab[id];
}

/*
void MultiAnim::changeColor(List< Pixel >& colorFrom,List< Pixel >& colorTo) throw ()
{
  for(int i=0;i<length;i++)
    tab[i]->changeColor(colorFrom,colorTo);
}
*/

int MultiAnim::getCurrent() throw()
{
  return currentAnim;
}

void MultiAnim::getSize(int& j,int& i) const throw()
{
  if(!activeAnim) {
    j=i=0;
    return;
  }
  activeAnim->getSize(j,i);
}
