/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



class PageChanger;

#ifndef __PageChanger_h__
#define __PageChanger_h__

#include "Pageable.h"
#include "Page.h"
#include "GraphicMode.h"
#include "FlashPalette.h"
#include "List.h"
#include "BonhommeContainer.h"
#include "Sounder.h"
#include "Keyer.h"
#include "Blower.h"
#include "Tji.h"
#include "StripContainer.h"
#include "TextPainter.h"
#include "PageChangerCouple.h"
#include "PageChangerDescChanging.h"

/**
   Manages all the Pages : loading and interactions (load classes according to the id of the Page in .ini files).
   Uses the [Changing Page] section in .ini files :
   Law : there is an elastic hanging the destination position of the Page and its initial position
   @author David FAUTHOUX
 */
class PageChanger:public Keyable,public Walkable {
 protected:


  ///
  struct AuxStruct {
    //%%  public:
    ///
    PageChangerCouple *couple;
    ///
    int cpt;
    ///
    //%%    AuxStruct(Couple *c,int i) throw();
  };


  ///
  List< PageChangerCouple * > listCouple;
  ///
  PageChangerCouple *currentCouple;
  ///
  List< PageChangerDescChanging * > listChanging;
  ///
  int nbChanging;

  ///
  Page *newPage;
  ///
  Page *oldPage;
  ///
  bool changing;
  ///
  PageChangerDescChanging *currentChanging;
  ///
  Tji newVit;
  ///
  Tji oldVit;

  ///
  bool finished;

  ///
  GraphicMode *const graphicModeRef;
  ///
  FlashPalette *const paletteRef;
  ///
  Ini *const iniRef;
  ///
  Sounder *const sounderRef;

  ///
  BonhommeContainer *bonhommeContainer;
  ///
  Grille *grille;
  ///
  int soundBlowUpMotId;

  ///
  Blower *redirectorRef;

  ///
  StripContainer *stripContainerRef;


  ///
  bool elastiqueEtSol(Page& page,int jTo,int iTo,int& jVit,int& iVit,bool haveToSetTotalyGone) throw();


  ///
  void constructCouple(Walkable& allTheGame,Keyer& keyer,bool clean) throw();
  ///
  void getChangingValues() throw();
  ///
  void testAllChange(Page& newPage) throw();
 private:
  ///
  void changeOn(int key) throw();

 public:

  ///
  PageChanger(Walkable& allTheGame,Ini& _ini,GraphicMode& graphicMode,
	      FlashPalette& palette,Sounder& sounder,Keyer& keyer,
	      Blower& redirector,
	      TextPainter& textPainter,
	      StripContainer& stripContainer,
	      bool clean)
    throw();

  ///
  virtual ~PageChanger() throw();

  ///
  void virtual keyAction(const Key& key) throw();
  ///
  void virtual paint() throw();
  ///
  int virtual virtualWalk() throw();


};

#endif
