/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



#include "PageChoiceFond.h"
#include "error.h"
#include "Key.h"
#include "util.h"

#define DEFAULT_KEY_ACCEPT RETURN
#define DEFAULT_KEY_LEFT LEFT
#define DEFAULT_KEY_RIGHT RIGHT
//%%%%#define DEFAULT_CURRENT_FOND 0

#define DEFAULT_ID "default"
#define DEFAULT_POS_ID_J 0
#define DEFAULT_POS_ID_I 0
#define DEFAULT_COLOR_ID toPixel(0,0,0,0)


PageChoiceFond::PageChoiceFond(Ini& ini,int numeroSection,
			       GraphicMode& graphicMode,
			       Sounder& sounder,int soundBlowMotId,
			       StripContainer& stripContainer)
  throw()
  :Page(ini,numeroSection,graphicMode,sounder,soundBlowMotId,stripContainer),
   pageGameRef(0),
   iniRef(&ini),numeroSectionRef(numeroSection)
{
  bool found=false;
  const String *k;
  int cptK=0;
  while(!found &&
	(k=ini.getWithoutException("[Page]",numeroSection,"change to",cptK))) {
    if(*k=="game") {
      keyAccept=ini.getWithDefaultKey("[Page]",numeroSection,"when key",cptK,
				   DEFAULT_KEY_ACCEPT);
      found=true;
    }
    cptK++;
  }
  if(!found) {
    ini.set("[Page]",numeroSection,"change to",cptK,"game");
    ini.set("[Page]",numeroSection,"when key",cptK,getKeyForName(DEFAULT_KEY_ACCEPT));
    keyAccept=DEFAULT_KEY_ACCEPT;
  }

  keyLeft=ini.getWithDefaultKey("[Page]",numeroSection,"key left",0,DEFAULT_KEY_LEFT);
  keyRight=ini.getWithDefaultKey("[Page]",numeroSection,"key right",0,DEFAULT_KEY_RIGHT);

  const String *currentFondId=ini.getWithDefault("[Page]",numeroSection,"current fond",0,DEFAULT_ID);
  currentFond=0;

  nbFond=0;
  int cpt=0;
  while(ini.isPresentSection("[Fond]",cpt)) {
    const String *idFondSection=ini.getWithDefault("[Fond]",cpt,"id",0,DEFAULT_ID);
    if(*idFondSection==*currentFondId) currentFond=cpt;
    Mot *m=new Mot(ini,
		   *idFondSection,
		   ini.getWithDefault("[Fond]",cpt,"pos id j",0,
				      DEFAULT_POS_ID_J),
		   ini.getWithDefault("[Fond]",cpt,"pos id i",0,
				      DEFAULT_POS_ID_I),
		   ini.getWithDefaultColor("[Fond]",cpt,"color id begin",0,
					   DEFAULT_COLOR_ID),
		   ini.getWithDefaultColor("[Fond]",cpt,"color id end",0,
					   DEFAULT_COLOR_ID),
		   0,*sounderRef,soundBlowUpId);
    m->unactivate();
    listMotFond.put(m);
    listSoundFond.put(sounderRef->add(*ini.getList("[Fond]",cpt,
    						   "sound","[Fond]",cpt)));
    add(*m);
    cpt++;
    nbFond++;
  }
  if(listMotFond.empty()) throw Exception(ERROR_NO_VALID_FOND);

  //%%%%%  if(currentFond<0 || currentFond>=nbFond) currentFond=0;

  listMotFond.place();
  while(listMotFond.walk()) listMotFond.get()->unactivate();

  listMotFond.place();
  listMotFond.walk();
  listSoundFond.place();
  listSoundFond.walk();
  cpt=0;
  while(cpt<currentFond) {
    listMotFond.walk();
    listSoundFond.walk();
    cpt++;
  }
  listMotFond.get()->activate();
}

void PageChoiceFond::setPageGame(PageGame& pageGame,bool loadAll) throw()
{
  pageGameRef=&pageGame;

  if(loadAll) {
    for(int cpt=0;cpt<nbFond;cpt++) {
      try {
	pageGameRef->setNumeroFond(cpt);
      } catch(Exception) {}
    }
  }

  //%%%%%  int oldCurrentFond=currentFond;
  int cpt=0;
  bool ok=false;
  while(!ok) {
    try {
      pageGameRef->setNumeroFond(currentFond);

      listMotFond.place();
      while(listMotFond.walk()) listMotFond.get()->unactivate();

      listMotFond.place();
      listMotFond.walk();
      listSoundFond.place();
      listSoundFond.walk();
      int cpt=0;
      while(cpt<currentFond) {
	listMotFond.walk();
	listSoundFond.walk();
	cpt++;
      }
      listMotFond.get()->activate();
      ok=true;
    } catch(Exception) {
      currentFond++;
      cpt++;
    }
    //C EST ICI !!
    if(currentFond==nbFond) {
      currentFond=0;
    }
    if(cpt==nbFond) {
      //      pageGameRef=0;
      //      return;
      throw Exception(ERROR_NO_VALID_FOND);
    }
  }
}


PageChoiceFond::~PageChoiceFond() throw()
{
  //%%%%  iniRef->set("[Page]",numeroSectionRef,"current fond",0,currentFond);
  iniRef->set("[Page]",numeroSectionRef,"current fond",0,*((const String *)*listMotFond.get()));
}

void PageChoiceFond::keyAction(const Key& key) throw()
{
  Page::keyAction(key);
  if(key.isUp()) return;
  if(key.getCode()==keyLeft) {
    currentFond++;
    listMotFond.get()->unactivate();
    listSoundFond.walk();
    if(!listMotFond.walk()) {
      listMotFond.place();
      listMotFond.walk();
      listSoundFond.place();
      listSoundFond.walk();
      currentFond=0;
    }
    listMotFond.get()->activate();
    sounderRef->play(listSoundFond.get());
 }
  if(key.getCode()==keyRight) {
    currentFond--;
    listMotFond.get()->unactivate();
    listSoundFond.walkHeadToQueue();
    if(!listMotFond.walkHeadToQueue()) {
      listMotFond.placeCurrentOnHead();
      listSoundFond.placeCurrentOnHead();
      currentFond=nbFond-1;
    }
    listMotFond.get()->activate();
    sounderRef->play(listSoundFond.get());
  }
  if(isTotalyGone() && key.getCode()==keyAccept) {
    if(pageGameRef) pageGameRef->setNumeroFond(currentFond);
  }
}
