/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



#include <limits.h>
#include <stdlib.h>
#include "PageGame.h"
#include "Key.h"

#define DEFAULT_KEY_END RETURN

#define DEFAULT_MOT_END "gagne!"
#define DEFAULT_POS_MOT_END_J 0
#define DEFAULT_POS_MOT_END_I 0
#define DEFAULT_COLOR_MOT_END toPixel(0,0,0,0)

#define DEFAULT_DECAL_SHADOW_J 15
#define DEFAULT_DECAL_SHADOW_I 15

PageGame::PageGame(Ini& ini,int numeroSection,GraphicMode& graphicMode,
		   BonhommeContainer& bonhommeContainer,
		   Grille& grille,Sounder& sounder,int soundBlowMotId,
		   StripContainer& stripContainer,
		   Blower& blower)
  throw(Exception)
  :Page(ini,numeroSection,graphicMode,sounder,soundBlowMotId,stripContainer),
   Trembleur(&blower),
   bonhommeContainerRef(&bonhommeContainer),
   grilleRef(&grille),
   graphicModeRef(&graphicMode),
   iniRef(&ini),numeroFond(0),
   finished(false),
   dead(false),
   keyWhenDead(ini.getWithDefaultKey("[Page]",numeroSection,"key when end",0,DEFAULT_KEY_END)),
   nbPartie(ini.getWithDefault("[Global]",0,"nombre de parties jouees",0,0)),
   motDead(new Mot(ini,
		   *ini.getWithDefault("[Page]",numeroSection,"mot end",0,DEFAULT_MOT_END),
		   ini.getWithDefault("[Page]",numeroSection,"pos mot end j",0,DEFAULT_POS_MOT_END_J),
		   ini.getWithDefault("[Page]",numeroSection,"pos mot end i",0,DEFAULT_POS_MOT_END_I),
		   ini.getWithDefaultColor("[Page]",numeroSection,"color mot end begin",0,DEFAULT_COLOR_MOT_END),
		   ini.getWithDefaultColor("[Page]",numeroSection,"color mot end end",0,DEFAULT_COLOR_MOT_END)))
{
  motDead->unactivate();
  toScroll.j=toScroll.i=realScroll.j=realScroll.i=0;
  wSize.j=wSize.i=decal.j=decal.i=0;
}


void PageGame::setNumeroFond(int num) throw(Exception)
{
  setBackground(*new Background(iniRef->
				getWithoutException("[Fond]",num,
						    "background",0),
				*graphicModeRef,
				iniRef->getWithDefault("[Page]",num,
						       "decal shadow j",0,DEFAULT_DECAL_SHADOW_J),
				iniRef->getWithDefault("[Page]",num,
						       "decal shadow i",0,DEFAULT_DECAL_SHADOW_I)));

  WindowPipe *w=background->getWindowPipe();
  int jOsf,iOsf;
  w->getTo(jOsf,iOsf,wSize.j,wSize.i);
  decal.j=-wSize.j/2;
  decal.i=-wSize.i/2;
  toScroll.j=toScroll.i=realScroll.j=realScroll.i=0;
  
  grilleRef->create(*iniRef,num,background->nbLine(),background->nbColumn());
    
  finished=dead=false;
  motDead->unactivate();
}

void PageGame::reinit() throw()
{
  finished=false;
  Trembleur::reinit();
  Page::reinit();
}


int PageGame::virtualWalk() throw()
{
  List<Bonhomme*> *l=bonhommeContainerRef->getBonhommePlaying();

  int j=0,i=0,cpt=0;
  int jMinLimit=INT_MAX;
  int jMaxLimit=0;
  int iMinLimit=INT_MAX;
  int iMaxLimit=0;
  l->place();
  while(l->walk()) {
    int jAux,iAux;
    int jSize,iSize;
    l->get()->getBonhomme20()->getPos(jAux,iAux);
    l->get()->getBonhomme20()->getSize(jSize,iSize);
    if(jAux<jMinLimit) jMinLimit=jAux;
    if(jAux+jSize>jMaxLimit) jMaxLimit=jAux+jSize;
    if(iAux<iMinLimit) iMinLimit=iAux;
    if(iAux+iSize>iMaxLimit) iMaxLimit=iAux+iSize;
    j+=jAux;
    i+=iAux;
    cpt++;
  }
  if(cpt) {
    j/=cpt;
    i/=cpt;
    toScroll.j=j+decal.j;
    toScroll.i=i+decal.i;
    if(realScroll.j<toScroll.j &&
       realScroll.j<jMinLimit && 
       realScroll.j+wSize.j<background->nbLine()) realScroll.j++;
    else if(realScroll.j>toScroll.j &&
	    realScroll.j+wSize.j>jMaxLimit &&
	    realScroll.j>0) realScroll.j--;
    if(realScroll.i<toScroll.i &&
       realScroll.i<iMinLimit &&
       realScroll.i+wSize.i<background->nbColumn()) realScroll.i++;
    else if(realScroll.i>toScroll.i &&
	    realScroll.i+wSize.i>iMaxLimit &&
	    realScroll.i>0) realScroll.i--;

    int tj,ti;
    get(tj,ti);
    tj+=realScroll.j;
    ti+=realScroll.i;
    if(tj+wSize.j>background->nbLine()) tj=background->nbLine()-wSize.j;
    if(tj<0) tj=0;
    if(ti+wSize.i>background->nbColumn()) ti=background->nbColumn()-wSize.i;
    if(ti<0) ti=0;
    background->getWindowPipe()
      ->moveFrom(tj,
		 ti);
  }

  int sj,si;
  background->getWindowPipe()->getFrom(j,i,sj,si);
  l->place();
  while(l->walk()) {
    Bonhomme20 *b=l->get()->getBonhomme20();
    b->setLimit(realScroll.j,realScroll.i,sj,si);
    b->walk();
  }

  grilleRef->walk();

  Trembleur::virtualWalk();

  if(motDead->isActive()) motDead->walk();

  Page::virtualWalk();
  testDead();
  if(finished) return 0;

  return 1;
}

PageGame::~PageGame() throw()
{
  delete motDead;
  iniRef->set("[Global]",0,"nombre de parties jouees",0,nbPartie);
}


void PageGame::paint() throw()
{
  WindowPipe *w=background->getWindowPipe();

  if(grilleRef->isInNight()) w->clearTo();
  else Page::paint();

  grilleRef->paint(*w);

  if(motDead->isActive()) motDead->paint(*w);

  List<Bonhomme*> *l=bonhommeContainerRef->getBonhommePlaying();
  l->place();
  while(l->walk()) {
    Trou *t=l->get()->getBonhomme20()->getTrou();
    if(t) t->paint(*w);
  }
}

void PageGame::keyAction(const Key& key) throw()
{
  if(dead && key.isDown() && key.getCode()==keyWhenDead) finished=true;
  List<Player*> *l=bonhommeContainerRef->getPlayerPlaying();
  l->place();
  while(l->walk())
    l->get()->keyAction(key);

  grilleRef->keyAction(key);

  Page::keyAction(key);
}

void PageGame::testDead() throw()
{
  if(dead || finished) return;

  if(bonhommeContainerRef->getNbPlayer()==1) return;

  List<Player*> *l=bonhommeContainerRef->getPlayerPlaying();

  l->place();
  while(l->walk()) {
    List<Bonhomme*> *b=l->get()->getBonhomme()->getPlayingExceptTeam();
    b->place();
    bool end=true;
    while(b->walk())
      if(b->get()->getBonhomme20()->getEtat()!=Bonhomme20::ETAT_FANTOME) {
	end=false;
	break;
      }
    if(end) {
      dead=true;
      motDead->activate();
      List<Bonhomme*> *c=l->get()->getBonhomme()->getPlayingTeam();
      c->place();
      int cpt=0;
      while(c->walk())
	if(c->get()->getBonhomme20()->getEtat()!=Bonhomme20::ETAT_FANTOME)
	  cpt++;
      for(int k=0;k<cpt;k++) l->get()->incScore();
      l->get()->getBonhomme()->getBonhomme20()->playSoundWon();
      //NON return true (car on inc le score de tous ceux qui gagnent)
    }
  }

  if(dead) nbPartie++;
}




