/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



#include "PageOption.h"
#include "LoopAnim.h"
#include "Key.h"

//#define MULT_SPEED 10000

#define DEFAULT_KEY_NB_PLAYER_PLUS KP_PLUS
#define DEFAULT_KEY_NB_PLAYER_MOINS KP_MINUS
#define DEFAULT_KEY_TEAM T
#define DEFAULT_KEY_SPEED TAB

#define DEFAULT_MOT_NB_PLAYER_1 "  player"
#define DEFAULT_MOT_NB_PLAYER_1_POS_J 0
#define DEFAULT_MOT_NB_PLAYER_1_POS_I 0
#define DEFAULT_MOT_NB_PLAYER_1_COLOR toPixel(0,0,0,0)

#define DEFAULT_MOT_NB_PLAYER_PLUS "  players"
#define DEFAULT_MOT_NB_PLAYER_PLUS_POS_J 0
#define DEFAULT_MOT_NB_PLAYER_PLUS_POS_I 0
#define DEFAULT_MOT_NB_PLAYER_PLUS_COLOR toPixel(0,0,0,0)

#define DEFAULT_MOT_SOLO "solo"
#define DEFAULT_POS_MOT_SOLO_J 0
#define DEFAULT_POS_MOT_SOLO_I 0
#define DEFAULT_MOT_SOLO_COLOR toPixel(0,0,0,0)

#define DEFAULT_MOT_TEAM "team"
#define DEFAULT_POS_MOT_TEAM_J 0
#define DEFAULT_POS_MOT_TEAM_I 0
#define DEFAULT_MOT_TEAM_COLOR toPixel(0,0,0,0)

#define DEFAULT_SLOW_ANIMATION 0
#define DEFAULT_POS_SPEED_J 0
#define DEFAULT_POS_SPEED_I 0
#define DEFAULT_SPEED 0


PageOption::PageOption(Ini& ini,int numeroSection,
		       GraphicMode& graphicMode,
		       BonhommeContainer& bonhommeContainer,
		       Walkable& allTheGameToSlow,
		       Sounder& sounder,int soundBlowMotId,
		       StripContainer& stripContainer) throw(Exception)
  :Page(ini,numeroSection,graphicMode,sounder,soundBlowMotId,stripContainer),
   bonhommeContainerRef(&bonhommeContainer),
   allTheGameRef(&allTheGameToSlow),
   iniRef(&ini),numeroSectionRef(numeroSection),
   keyNbPlayerPlus(ini.getWithDefaultKey("[Page]",numeroSection,"key nombre de Player ++",0,DEFAULT_KEY_NB_PLAYER_PLUS)),
   keyNbPlayerMoins(ini.getWithDefaultKey("[Page]",numeroSection,"key nombre de Player --",0,DEFAULT_KEY_NB_PLAYER_MOINS)),
   keyTeam(ini.getWithDefaultKey("[Page]",numeroSection,"key team",0,DEFAULT_KEY_TEAM)),
   keySpeed(ini.getWithDefaultKey("[Page]",numeroSection,"key speed",0,DEFAULT_KEY_SPEED))
{
  struct {
    const String *m;
    struct { int j,i; } pos;
    struct { Pixel beg,end; } color;
  } m1,mP;


  m1.m=ini.getWithDefault("[Page]",numeroSection,
			  "mot nombre de Player 1",0,
			  DEFAULT_MOT_NB_PLAYER_1);
  m1.pos.j=ini.getWithDefault("[Page]",numeroSection,
			      "pos mot nombre de Player 1 j",0,
			      DEFAULT_MOT_NB_PLAYER_1_POS_J);
  m1.pos.i=ini.getWithDefault("[Page]",numeroSection,
			      "pos mot nombre de Player 1 i",0,
			      DEFAULT_MOT_NB_PLAYER_1_POS_I);
  m1.color.beg=ini.getWithDefaultColor("[Page]",numeroSection,
				       "nombre de Player 1 color begin",0,
				       DEFAULT_MOT_NB_PLAYER_1_COLOR);
  m1.color.end=ini.getWithDefaultColor("[Page]",numeroSection,
				       "nombre de Player 1 color end",0,
				       DEFAULT_MOT_NB_PLAYER_1_COLOR);

  mP.m=ini.getWithDefault("[Page]",numeroSection,
			  "mot nombre de Player plusieurs",0,
			  DEFAULT_MOT_NB_PLAYER_PLUS);
  mP.pos.j=
    ini.getWithDefault("[Page]",numeroSection,
		       "pos mot nombre de Player plusieurs j",0,
		       DEFAULT_MOT_NB_PLAYER_PLUS_POS_J);
  mP.pos.i=
    ini.getWithDefault("[Page]",numeroSection,
		       "pos mot nombre de Player plusieurs i",0,
		       DEFAULT_MOT_NB_PLAYER_PLUS_POS_I);
  mP.color.beg=
    ini.getWithDefaultColor("[Page]",numeroSection,
			    "nombre de Player plusieurs color begin",0,
			    DEFAULT_MOT_NB_PLAYER_PLUS_COLOR);
  mP.color.end=
    ini.getWithDefaultColor("[Page]",numeroSection,
			    "nombre de Player plusieurs color end",0,
			    DEFAULT_MOT_NB_PLAYER_PLUS_COLOR);
  
  int max=bonhommeContainer.getMaxNbPlayer();
  if(max>0) {
    motNb=new Mot*[max];
    motNb[0]=new Mot(ini,
		     *new String(0+1),
		     m1.pos.j,
		     m1.pos.i,
		     m1.color.beg,
		     m1.color.end);
    motNb[0]->unactivate();
    add(*motNb[0]);
    for(int i=1;i<max;i++) {
      motNb[i]=new Mot(ini,
		       *new String(i+1),
		       mP.pos.j,
		       mP.pos.i,
		       mP.color.beg,
		       mP.color.end);
      motNb[i]->unactivate();
      add(*motNb[i]);
    }
    
    motNbPlayer1=new Mot(ini,*(m1.m),
			 m1.pos.j,
			 m1.pos.i,
			 m1.color.beg,
			 m1.color.end);
    motNbPlayerPlusieurs=new Mot(ini,*(mP.m),
				 mP.pos.j,
				 mP.pos.i,
				 mP.color.beg,
				 mP.color.end);
    motNbPlayer1->unactivate();
    motNbPlayerPlusieurs->unactivate();
    add(*motNbPlayer1);
    add(*motNbPlayerPlusieurs);

    int n=bonhommeContainer.getNbPlayer();
    oldNb=n;
    if(n>1) {
      motNb[n-1]->activate();
      motNbPlayerPlusieurs->activate();
    }
    else {
      motNb[0]->activate();
      motNbPlayer1->activate();
    }
  }

  //
  motSolo=new Mot(ini,
		  *ini.getWithDefault("[Page]",numeroSection,"mot solo",0,
				      DEFAULT_MOT_SOLO),
		  ini.getWithDefault("[Page]",numeroSection,"pos mot solo j",0,
				     DEFAULT_POS_MOT_SOLO_J),
		  ini.getWithDefault("[Page]",numeroSection,"pos mot solo i",0,
				     DEFAULT_POS_MOT_SOLO_I),
		  ini.getWithDefaultColor("[Page]",numeroSection,"solo color begin",
					  0,DEFAULT_MOT_SOLO_COLOR),
		  ini.getWithDefaultColor("[Page]",numeroSection,"solo color end",
					  0,DEFAULT_MOT_SOLO_COLOR));
  motTeam=new Mot(ini,
		  *ini.getWithDefault("[Page]",numeroSection,"mot team",0,
				      DEFAULT_MOT_TEAM),
		  ini.getWithDefault("[Page]",numeroSection,"pos mot team j",0,
				     DEFAULT_POS_MOT_TEAM_J),
		  ini.getWithDefault("[Page]",numeroSection,"pos mot team i",0,
				     DEFAULT_POS_MOT_TEAM_I),
		  ini.getWithDefaultColor("[Page]",numeroSection,"team color begin",
					  0,DEFAULT_MOT_TEAM_COLOR),
		  ini.getWithDefaultColor("[Page]",numeroSection,"team color end",
					  0,DEFAULT_MOT_TEAM_COLOR));

  if(bonhommeContainer.isTeamSet()) motSolo->unactivate();
  else motTeam->unactivate();

  add(*motSolo);
  add(*motTeam);
  
  //
  
  animSpeed=new MultiAnim();
  add(*animSpeed);

  const String *m;
  int cpt=0;
  while((m=ini.getWithoutException("[Page]",numeroSection,
				   "loop anim speed",cpt))) {
    LoopAnim *a=new LoopAnim(*m,
			     ini.getWithDefault("[Slow]",0,"Animation",0,
						DEFAULT_SLOW_ANIMATION),
			 ini.getListSeparator());
    a->setOn();
    animSpeed->add(*a);
    listSpeed.put(ini.getWithDefault("[Page]",numeroSection,
				     "speed",cpt,
				     DEFAULT_SPEED));
    cpt++;
  }
  animSpeed->move(ini.getWithDefault("[Page]",numeroSection,
				     "pos loop anim speed j",0,
				     DEFAULT_POS_SPEED_J),
		  ini.getWithDefault("[Page]",numeroSection,
				     "pos loop anim speed i",0,
				     DEFAULT_POS_SPEED_I));
  onWhatSpeed=ini.getWithDefault("[Page]",numeroSection,"current speed",0,0);

  if(!listSpeed.empty()) {
    listSpeed.place();
    listSpeed.walk();
    for(int k=0;k<onWhatSpeed;k++) {
      if(!listSpeed.walk()) {
	onWhatSpeed=0;
	listSpeed.place();
	listSpeed.walk();
	break;
      }
    }
    animSpeed->on(onWhatSpeed);
    allTheGameRef->setSlow(listSpeed.get());
  }

}

PageOption::~PageOption() throw()
{
  iniRef->set("[Page]",numeroSectionRef,"current speed",0,onWhatSpeed);
}

void PageOption::setSpeed() throw()
{
  if(!listSpeed.empty()) {
    onWhatSpeed++;
    if(!listSpeed.walk()) { listSpeed.place();listSpeed.walk();onWhatSpeed=0; }
    animSpeed->on(onWhatSpeed);
    allTheGameRef->setSlow(listSpeed.get());
  }
}

void PageOption::setMotNbPlayer() throw()
{
  if(oldNb==0) {
    //rien
  } else if(oldNb>1) {
    motNb[oldNb-1]->unactivate();
    motNbPlayerPlusieurs->unactivate();
  } else {
    motNb[0]->unactivate();
    motNbPlayer1->unactivate();
  }
  int n=bonhommeContainerRef->getNbPlayer();
  if(n==0) {
    //rien
  } else if(n>1) {
    motNb[n-1]->activate();
    motNbPlayerPlusieurs->activate();
  } else {
    motNb[0]->activate();
    motNbPlayer1->activate();
  }
  oldNb=n;
}

void PageOption::keyAction(const Key& key) throw()
{
  if(key.isDown()) {
    if(key.getCode()==keyNbPlayerPlus) {
      bonhommeContainerRef->incNbPlayer();
      setMotNbPlayer();
    }
    if(key.getCode()==keyNbPlayerMoins) {
      bonhommeContainerRef->decNbPlayer();
      setMotNbPlayer();
    }
    
    if(key.getCode()==keyTeam) {
      bonhommeContainerRef->setOrUnsetTeam();
      if(bonhommeContainerRef->isTeamSet()) {
	motSolo->unactivate();
	motTeam->activate();
      } else {
	motSolo->activate();
	motTeam->unactivate();
      }
    }
    
    if(key.getCode()==keySpeed) {
      setSpeed();
    }
  }

  Page::keyAction(key);
}


void PageOption::reinit() throw()
{
  Page::reinit();
  animSpeed->on(onWhatSpeed);
}
