/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



class PageableContainer;

#ifndef __PageableContainer_h__
#define __PageableContainer_h__

#include "Pageable.h"
#include "List.h"
#include "Exception.h"

// Cette classe est tres interessante :

// c'est un container (regroupeur) de 'Pageable',
// il leur passe les actions (par exemple walk, paint) selon ces lois
// (qu'on peut deriver)

// IMPORTANT : cette classe est elle-meme 'Pageable',
// ce qui permet de creer une hierarchie de containers et donc,
// une hierarchie de transmissions d'actions ;
// Exemple : une 'Page' est 'PageableContainer' et elle contient des 'Mot'
// MAIS, elle ne passe pas l'action 'keyAction' tout le temps !
// uniquement quand elle est totalement tombee.
// 'Mot' est egalement un 'PageableContainer'...

// Ainsi, on pourrait creer un container de 'Page'
// qui ne passe les actions que quand la 'Page' est active
// (par exemple, definir le 'PageChanger' derivant de 'PageableContainer')

// En clair, c'est comme des valises a points d'entrees contenant des valises
// (dont la definition est identique)
// qui contiennent elles-memes des valises ...


/**
   This class is very very interesting !!
   It is a container : it contains Pageable, so it manages all the contained automatically. To a call of PageableContainer.paint for example, the PageableContainer call paint of all the contained. Idem with keyAction,reinit,walk...
   The interesting thing now : a PageableContainer is itself Pageable !!
   So a PageableContainer can contains PageableContainer : it is a hierarchy !
   Example : CharacterMot is Pageable (a char in color). Mot is PageableContainer : it contains CharacterMot to make a word. And Mot is Pageable, so it can be put in a Painter (it is a PageableContainer). So the Painter can contains Mot, but can contains also Anim, LoopAnim, MultiAnim, etc... and all you can imagine !!!
   <u><b>This structure programming constructs trees of hierarchical objects. Through the trees passes messages like key evenement, or paint request.<\b><\u>
   If you want more explanations, e-mail me : <mailto:david.fauthoux@free.fr>
   @author David FAUTHOUX
 */
class PageableContainer:public Pageable {
 protected:
  ///
  List< Pageable * > list;
  ///
  List< Pageable * > listWithoutDelete;

  ///
  int virtual virtualWalk() throw();
 public:
  ///
  PageableContainer(int slow) throw();

  ///
  virtual ~PageableContainer() throw();

  ///Be careful, the pageable is deleted here, if you don't want it, use addWithoutDelete
  void virtual add(Pageable& pageable) throw();
  //ATTENTION ! delete des Pageable fait par le destructor de PageableContainer

  ///The pageable must be deleted by the caller
  void virtual addWithoutDelete(Pageable& pageable) throw();


  ///
  void virtual suppr(Pageable *pageable) throw(Exception);
  ///
  void virtual supprAll() throw();
  
  ///
  void virtual paint(WindowPipe& windowPipe) throw();
  ///
  void virtual reinit() throw();
  ///
  void virtual keyAction(const Key& key) throw();

};


#endif
