/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



class PixelMatrix;

#ifndef __PixelMatrix_h__
#define __PixelMatrix_h__

#include "PixelArray.h"
#include "UnderMaskMatrix.h"

/**
   A dynamical matrix, more powerful than [ ] [ ].
   @see Sprite
   @see WindowPipe
   @see Window
   @see GraphicMode
   @author David FAUTHOUX
 */
class PixelMatrix {
 protected:
  ///
  Array< PixelArray * > a;
  ///
  Pixel *p;
 public:
  ///
  PixelMatrix(int nbLine_j,int nbColumn_i) throw ();
  ///
  virtual ~PixelMatrix() throw ();

  ///
  PixelArray& operator[](int k) throw ();
  ///
  const PixelArray& operator[](int k) const throw ();
  ///Vertical
  int nbLine() const throw();
  ///Horizontal
  int nbColumn() const throw();

  ///
  void putSprite(const PixelMatrix& data,
		 const PixelMatrix& mask,
		 int jPosFrom,int iPosFrom,
		 int jPosTo,int iPosTo,
		 int jSizeFrom,int iSizeFrom) throw ();

  ///
  void putSpriteColor(Pixel color,
		      const PixelMatrix& data,
		      int jPosFrom,int iPosFrom,
		      int jPosTo,int iPosTo,
		      int jSizeFrom,int iSizeFrom) throw ();

  ///
  void putSpriteUnderMask(const PixelMatrix& data,
			  const PixelMatrix& mask,
			  int jPosFrom,int iPosFrom,
			  int jPosTo,int iPosTo,
			  int jSizeFrom,int iSizeFrom,
			  const UnderMaskMatrix& underMask,
			  int jPosUnderMask,int iPosUnderMask) throw ();


  ///
  void putSpriteUnderMaskPalette(const PixelMatrix& data,
				 const PixelMatrix& mask,
				 int jPosFrom,int iPosFrom,
				 int jPosTo,int iPosTo,
				 int jSizeFrom,int iSizeFrom,
				 const UnderMaskMatrix& underMask,
				 int jPosUnderMask,int iPosUnderMask,
				 const PixelPalette& pal) throw ();


  ///
  void copy(const PixelMatrix& data,
	    int jPosFrom,int iPosFrom,
	    int jPosTo,int iPosTo,
	    int jSizeFrom,int iSizeFrom) throw ();

  ///
  operator const Pixel*() const throw () { return p; }

  ///
  operator Pixel*() throw () { return p; }

  ///
  PixelMatrix& operator=(Pixel pixel) throw();

};


#endif
