/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



#include <iostream.h>
#include "ScriptAnalyser.h"
#include "File.h"
#include "util.h"

#define COMMENT_MARKER '#'

ScriptAnalyser::ScriptAnalyser(const String &nameFile) throw(Exception)
  :nameOfFile(new String(nameFile)),endOfFile(false),currentPosition(0)
{
  String *s;
  List<char> l;
  File f(nameFile);
  while(!f.eof()) {
    f.read(s);
    l.place();
    while(l.walk()) l.suppr();
    for(int i=0;i<s->size();i++)
      if((*s)[i]==COMMENT_MARKER) break;
      else l.put((*s)[i]);
    delete s;

    //enlever les ' ' au debut et a la fin
    l.place();
    while(l.walk() && isBlank(l.get())) l.suppr();
    if(!l.empty()) {
      l.placeCurrentOnHead();
      while(isBlank(l.get())) {
	l.suppr();
      }
    }
    //

    list.put(new String(l));
  }
  list.place();
  if(!list.walk()) endOfFile=true;
  currentPosition++;
}

ScriptAnalyser::~ScriptAnalyser() throw()
{
  list.place();
  while(list.walk()) delete list.get();
}

void ScriptAnalyser::ignoreEmptyLines() throw()
{
  if(endOfFile) return;
  while(equal(*list.get(),"")) {
    currentPosition++;
    if(!list.walk()) {
      endOfFile=true;
      return;
    }
  }
}

bool ScriptAnalyser::equal(const String& s0,const String& s1) throw()
{
  String *r0=simplifyBlank(s0);
  String *r1=simplifyBlank(s1);
    
  bool ret=(*r0==*r1);
  delete r0;
  delete r1;

  return ret;
}

void ScriptAnalyser::check(const String& s) throw(Exception)
{
  if(!endOfFile) {
    if(equal(*list.get(),s)) {
      currentPosition++;
      if(!list.walk()) endOfFile=true;
      return;
    }
  }
  error(ERROR_SCRIPT_MUST_FIND,s);
}

void ScriptAnalyser::error(const String& e,const String& s,int decal,bool throwException) throw(Exception)
{
  keepPosition();
  int cpt=1;
  while(list.walkHeadToQueue()) cpt++;
  recallPosition();
  if(throwException) throw Exception(*nameOfFile+*new String(":")+*new String(ERROR_SCRIPT_AT_LINE)+*new String(cpt+decal),e+*new String(ERROR_SCRIPT_SEPARATOR)+s);
  else {
    cerr<<(const char*)(*nameOfFile+*new String(":")+*new String(ERROR_SCRIPT_AT_LINE)+*new String(cpt+decal)+*new String(":")+e+*new String(ERROR_SCRIPT_SEPARATOR)+s)<<"\n";
    cerr.flush();
  }
}

void ScriptAnalyser::error(const String& e,int decal,bool throwException) throw(Exception)
{
  keepPosition();
  int cpt=1;
  while(list.walkHeadToQueue()) cpt++;
  recallPosition();
  if(throwException) throw Exception(*nameOfFile+*new String(":")+*new String(ERROR_SCRIPT_AT_LINE)+*new String(cpt+decal),e);
  else {
    cerr<<(const char*)(*nameOfFile+*new String(":")+*new String(ERROR_SCRIPT_AT_LINE)+*new String(cpt+decal)+*new String(":")+e)<<"\n";
    cerr.flush();
  }
}

const String *ScriptAnalyser::getLine() throw(Exception)
{
  if(endOfFile) throw Exception(*nameOfFile+*new String(":")+*new String(ERROR_SCRIPT_END_OF_FILE));
  const String *s=list.get();
  currentPosition++;
  if(!list.walk()) endOfFile=true;
  return s;
}

void ScriptAnalyser::keepPosition() throw()
{
  position.put(currentPosition);
}

void ScriptAnalyser::recallPosition() throw(Exception)
{
  int cpt=position.get();
  position.suppr();
  list.place();
  currentPosition=0;
  while(currentPosition<cpt) {
    list.walk();
    currentPosition++;
  }
}
