/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



class ScriptAnalyser;

#ifndef __ScriptAnalyser_h__
#define __ScriptAnalyser_h__

#include "String.h"
#include "List.h"

#define ERROR_SCRIPT_MUST_FIND "Must find"
#define ERROR_SCRIPT_AT_LINE "Script error at line "
#define ERROR_SCRIPT_SEPARATOR ":"
#define ERROR_SCRIPT_END_OF_FILE "Script error : end of file ! must find something !"

/**
   Can read script : give a lot of useful functions to read a script text file.
   Used by AddOnScriptAnalyser.
   @see AddOnScriptAnalyser
   @author David FAUTHOUX
*/
class ScriptAnalyser {
 protected:
  ///file name
  const String *nameOfFile;
  ///
  List< const String * > list;
  ///
  bool endOfFile;
  ///
  List< int > position;
  ///
  int currentPosition;

  ///
  virtual void ignoreEmptyLines() throw();
  ///
  virtual void check(const String& s) throw(Exception);
  ///
  virtual void error(const String& e,const String& s,int decal=0,bool throwException=true) throw(Exception);
  ///
  virtual void error(const String& e,int decal=0,bool throwException=true) throw(Exception);
  ///
  virtual const String *getLine() throw(Exception);

 public:
  ///Call analyse() after construction
  ScriptAnalyser(const String &nameFile) throw(Exception);
  ///
  virtual ~ScriptAnalyser() throw();

  ///
  virtual bool equal(const String& s0,const String& s1) throw();

  ///Call it after construction
  virtual void analyse() throw(Exception)=0; 

  ///Saves the current position, can save many positions (list)
  virtual void keepPosition() throw();
  ///Places the current position on the saved position, do not recall if not kept !
  virtual void recallPosition() throw(Exception);

};


#endif
