/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



#include "SectionAssoc.h"

SectionAssoc::SectionAssoc(const String& s,const String& comment) throw()
  :title(s),com(comment),isUsed(false)
{
}

SectionAssoc::~SectionAssoc() throw()
{
  l.place();
  while(l.walk()) delete l.get();
}

const String *SectionAssoc::getTitle() const throw()
{
  return &title;
}

const String *SectionAssoc::getComment() const throw()
{
  return &com;
}

void SectionAssoc::setComment(const String& comment) throw()
{
  com=comment;
}

List<ValueString*> *SectionAssoc::getAll() throw()
{
  return &l;
}

void SectionAssoc::add(ValueString& v) throw()
{
  //%%%%  isUsed=true;
  l.placeCurrentOnHead();
  l.put(&v);
}

void SectionAssoc::set(const String& id,int cpt,const String& val) throw()
{
  isUsed=true;
  int k=0;
  l.place();
  while(l.walk())
    if(*(l.get()->getId())==id) {
      if(k==cpt) {
	l.get()->setVal(val);
	l.get()->used=true;
	return;
      }
      k++;
    }
  while(k<cpt) {
    ValueString *v=new ValueString(id,*new String("???"),*new String(""));
    v->used=true;
    add(*v);
    k++;
  }
  ValueString *v=new ValueString(id,val,*new String(""));
  v->used=true;
  add(*v);
}

const String *SectionAssoc::get(const String& id,int cpt) throw(Exception)
{
  const String *ret=getWithoutException(id,cpt);
  if(!ret)
    throw Exception(ERROR_ID_NOT_FOUND,title,id);
  return ret;
}

const String *SectionAssoc::getWithoutException(const String& id,int cpt) throw()
{
  isUsed=true;
  int k=0;
  l.place();
  while(l.walk())
    if(*(l.get()->getId())==id) {
      if(k==cpt) {
	l.get()->used=true;
	return l.get()->getVal();
      }
      k++;
    }
  return 0;
}

const String *SectionAssoc::getWithDefault(const String& id,int cpt,const String& defaut) throw()
{
  isUsed=true;
  int k=0;
  l.place();
  while(l.walk())
    if(*(l.get()->getId())==id) {
      if(k==cpt) {
	l.get()->used=true;
	return l.get()->getVal();
      }
      k++;
    }
  set(id,cpt,defaut);
  return get(id,cpt);
}

List<const String*> *SectionAssoc::getList(const String& id) throw()
{
  isUsed=true;
  List<const String *> *ret=new List<const String*>;
  l.place();
  while(l.walk()) {
    if(*(l.get()->getId())==id) {
      ret->placeCurrentOnHead();
      l.get()->used=true;
      ret->put(l.get()->getVal());
    }
  }
  return ret;
}

List<const String*> *SectionAssoc::
getListWithDefaultSection(const String& id,SectionAssoc& other) throw(Exception)
{
  List<const String*> *ret=getList(id);
  if(ret->empty()) {
    delete ret;
    ret=other.getList(id);
    if(ret->empty()) {
      delete ret;
      throw Exception(ERROR_ID_NOT_FOUND,title,id);
    }
  }
  return ret;
}



void SectionAssoc::setWithMaximumNumber(const String& id,const String& val) throw()
{
  isUsed=true;
  l.place();
  while(l.walk());
  ValueString *v=new ValueString(id,val,*new String(""));
  v->used=true;
  add(*v);
}

