/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



#include "SectionFileModifier.h"
#include "File.h"
#include "util.h"

SectionFileModifier::SectionFileModifier(const String& nameFile,
				   List<const String *>& markList,
				   const String& commentSymbols) throw(Exception)
  :mark(&markList),commentMark(&commentSymbols),name(&nameFile),endMark(0)
{
  File f(nameFile);
  while(!f.eof()) {
    char *s=0;
    f.read(s);
    insertRaw(*new String(s));
  }
  list.place();

  List<char> listForMark;
  listForMark.put('#');
  listForMark.put(' ');
  listForMark.put('e');
  listForMark.put('n');
  listForMark.put('d');
  listForMark.put(':');
  mark->place();
  while(mark->walk()) {
    for(int k=0;k<mark->get()->size();k++) {
      char c=(*(mark->get()))[k];
      if(c!='\n') listForMark.put(c);
    }
  }
  listForMark.put('\n');
  endMark=new String(listForMark);
}

SectionFileModifier::~SectionFileModifier() throw()
{
  list.place();
  while(list.walk()) delete list.get();
  delete endMark;
}

void SectionFileModifier::save() throw(Exception)
{
  save(*name);
}

void SectionFileModifier::save(const String& nameFile) throw(Exception)
{
  File f(nameFile,File::WRITE_ONLY);
  //  if(list.empty()) return;
  list.place();
  while(list.walk()) {
    f.write((const char*)(*list.get()));
  }
}


void SectionFileModifier::place() throw()
{
  list.place();
}

bool SectionFileModifier::walk() throw()
{
  if(list.walk()) return true;
  return false;
}

bool SectionFileModifier::walkInvers() throw()
{
  if(list.walkHeadToQueue()) return true;
  return false;
}

const String *SectionFileModifier::current() const throw(Exception)
{
  return list.get();
}


bool SectionFileModifier::after(List< const String * >& afterLines) throw()
{
  afterLines.place();
  if(!afterLines.walk()) return true;
  bool together=false;
  for(;;) {
    if(!walk()) return false;
    String *s0=simplifyBlank(*current());
    String *s1=simplifyBlank(*afterLines.get());
    if(*s0==*s1) together=true;
    else {
      together=false;
      afterLines.place();
      afterLines.walk();
    }
    if(together) if(!afterLines.walk()) return true;
    delete s0;
    delete s1;
  }
}

bool SectionFileModifier::after(const String& afterLine) throw()
{
  String *s1=simplifyBlank(afterLine);
  for(;;) {
    if(!walk()) {
      delete s1;
      return false;
    }
    String *s0=simplifyBlank(*current());
    if(*s0==*s1) {
      delete s0;
      delete s1;
      return true;
    }
    delete s0;
  }
}

bool SectionFileModifier::before(List< const String * >& beforeLines) throw()
{
  if(beforeLines.empty()) return true;
  beforeLines.placeCurrentOnHead();
  //%%  if(!beforeLines.walk()) return true;

  if(list.atTheQueue()) return false;

  bool together=false;
  for(;;) {
    String *s0=simplifyBlank(*current());
    String *s1=simplifyBlank(*beforeLines.get());
    if(*s0==*s1) together=true;
    else {
      together=false;
      beforeLines.placeCurrentOnHead();
      //%%      beforeLines.walkHeadToQueue();
    }
    if(together) if(!beforeLines.walkHeadToQueue()) {
      walkInvers();
      return true;
    }
    if(!walkInvers()) return false;
    delete s0;
    delete s1;
  }
}

bool SectionFileModifier::before(const String& beforeLine) throw()
{
  if(list.atTheQueue()) return false;

  String *s1=simplifyBlank(beforeLine);
  for(;;) {
    String *s0=simplifyBlank(*current());
    if(*s0==*s1) {
      delete s0;
      delete s1;
      walkInvers();
      return true;
    }
    if(!walkInvers()) {
      delete s1;
      return false;
    }
    delete s0;
  }
}


void SectionFileModifier::insert(List<const String*>& lines) throw()
{
  //  if(putNewLine) insertRaw(*new String("\n"));
  mark->place();
  while(mark->walk()) insertRaw(*(mark->get())+*new String("\n"));
  
  /*
  int cpt=0;
  lines.place();
  while(lines.walk()) cpt++;
  if(cpt<=1) insertRaw(*commentMark+*new String(cpt)+*new String(" line added\n"));
  else insertRaw(*commentMark+*new String(cpt)+*new String(" lines added\n"));
  */

  lines.place();
  while(lines.walk()) insertRaw(*(lines.get())+*new String("\n"));

  insertRaw(*new String(*endMark));//%%new String("# end\n"));
  
}

void SectionFileModifier::insertRaw(const String& line) throw()
{
  list.put(&line);
}


void SectionFileModifier::suppress() throw()
{
  if(list.empty()) return;

  place();
  while(walk());

  while(before(*mark)) {
    
    //enlever le mark    
    after(*mark);

    mark->place();
    while(mark->walk()) {
      list.suppr();
    }
    while(list.walk()) {
      if((*list.get())==*endMark) {//"# end\n") {
	list.suppr();
	break;
      }
      else list.suppr();
    }

  }
}

void SectionFileModifier::suppressLine() throw()
{
  list.suppr();
}
