/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



#include "Sounder.h"
#include "File.h"
#include "display.h"
#include "error.h"

Sounder::Sounder( void )
  throw()
  :nbChannel(16),
   sampleRate(11025),
   countForId(0)
{
  if(nbChannel<=1) nbChannel=2;
  ok=enableSound();

  for(int k=0;k<nbChannel;k++) freeChannels.put(k);
  freeChannels.place();
  freeChannels.walk();
}

Sounder::~Sounder() throw()
{
  if(ok) disableSound();

  list.place();
  while(list.walk()) delete list.get();

}

int Sounder::add(const String& nameFile,bool ecrasable) throw(Exception)
{
  List<const String*> l;
  l.put(&nameFile);
  return add(l,ecrasable);
}

int Sounder::reserveChannel() throw()
{
  if(!freeChannels.empty()) {
    freeChannels.place();
    freeChannels.walk();
    int r=freeChannels.get();
    freeChannels.suppr();
    freeChannels.place();
    freeChannels.walk();
    return r;
  }
  else return -1;
}

int Sounder::add(List<const String*>& nameFile,bool ecrasable) throw(Exception)
{
  if(nameFile.empty()) return -1;

  SounderAssoc *assoc=new SounderAssoc(countForId);

  nameFile.place();
  while(nameFile.walk()) {
    int i=-1;

    if(ok) i=addSoundFile(*(nameFile.get()));

    File f(*(nameFile.get()));
    int sizeFile=f.getSize();

    assoc->add(i,sizeFile);

  }

  if(!ecrasable && !freeChannels.empty()) {
    freeChannels.place();
    freeChannels.walk();
    assoc->reserveChannel(freeChannels.get());
    freeChannels.suppr();
    freeChannels.place();
    freeChannels.walk();
  }

  list.put(assoc);

  int ret=countForId;
  countForId++;

  return ret;
}

int Sounder::add(List<const String*>& nameFile,int channelReservedBefore) throw(Exception)
{
  if(nameFile.empty()) return -1;

  SounderAssoc *assoc=new SounderAssoc(countForId);
  
  nameFile.place();
  while(nameFile.walk()) {

    int i=-1;

    if(ok) i=addSoundFile(*(nameFile.get()));

    File f(*(nameFile.get()));
    int sizeFile=f.getSize();

    assoc->add(i,sizeFile);
  }

  assoc->reserveChannel(channelReservedBefore);

  list.put(assoc);

  int ret=countForId;
  countForId++;

  return ret;
}

void Sounder::suppr(int id) throw()
{
  SounderAssoc *assoc=0;

  list.place();
  while(list.walk()) {
    if(list.get()->logicalId==id) {
      assoc=list.get();
      list.suppr();
      break;
    }
  }
  if(!assoc) return;

  assoc->physicalIds.place();
  while(assoc->physicalIds.walk()) {
    if(ok) supprSound(assoc->physicalIds.get());
  }

  if(assoc->channelReserving) {
    freeChannels.put(assoc->reservedChannel);
  }

  delete assoc;
}


int Sounder::play(int id) throw()
{
  SounderAssoc *assoc=0;

  list.place();
  while(list.walk()) {
    if(list.get()->logicalId==id) {
      assoc=list.get();
      break;
    }
  }
  if(!assoc) return 0;
  if(assoc->nbPhysicalIds==0) return 0;

  int k=1+rand()%assoc->nbPhysicalIds;

  assoc->physicalIds.place();
  assoc->timeFile.place();
  while(k) {
    assoc->physicalIds.walk();
    assoc->timeFile.walk();
    k--;
  }

  int t=assoc->timeFile.get()/(sampleRate/1000);

  if(assoc->channelReserving) {
    if(ok) playSound(assoc->physicalIds.get(),assoc->reservedChannel);
  }
  else {
    if(!freeChannels.empty()) {
      if(ok) playSound(assoc->physicalIds.get(),freeChannels.get());
      if(!freeChannels.walk()) {
	freeChannels.place();
	freeChannels.walk();
      }
    }
  }

  return t;
}



void Sounder::setVolume(int id,int volume) throw()
{
  SounderAssoc *assoc=0;

  list.place();
  while(list.walk()) {
    if(list.get()->logicalId==id) {
      assoc=list.get();
      break;
    }
  }
  if(!assoc) return;

  assoc->physicalIds.place();  
  while(assoc->physicalIds.walk()) 
    if(ok) {
      if(!assoc->channelReserving || volume>0)
	setVolumeSound(assoc->physicalIds.get(),volume);
      else 
	stopSoundChannel(assoc->reservedChannel);
    }
}



