/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



class Sounder;

#ifndef __Sounder_h__
#define __Sounder_h__

//#include "soundHim.h"
#include "List.h"
#include "SounderAssoc.h"

// On peut ajouter a cette classe une liste de sons, elle renvoie un id,
// ainsi, lorsque qu'on 'play(id)', elle joue un son de la liste au hasard.


/**
   Manages the sound playing.
   Use it by loading a sound and playing it with the id returns.
   Thanks to Brad Pitzel for "soundIt" !
   Uses soundHim.c
   @author David FAUTHOUX
*/
class Sounder {
 protected:
  ///
  List< int > freeChannels;

  ///
  List< SounderAssoc * > list;

  ///
  bool ok;

  ///
  int nbChannel;

  ///
  int sampleRate;

  ///
  int countForId;

 public:
  ///
  Sounder( void ) throw();
  ///
  virtual ~Sounder() throw();

  ///
  virtual int add(const String& nameFile,bool ecrasable=true) throw(Exception);
  //renvoie l'id
  ///ecrasable=false forbids the sound to be removed when too many sounds are playing and another is added
  virtual int add(List< const String * >& nameFile,bool ecrasable=true)
    throw(Exception);
  //renvoie l'id

  ///Reserves a channel to put sounds played not simultaneously (see addInChannel)
  virtual int reserveChannel() throw();
  ///Places the sounds in a channel reserved before this call by reserveChannel, enables that a Bonhomme doesn't speak on more than one channel (not multiple voices !)
  virtual int add(List<const String*>& nameFile,int channelReservedBefore) throw(Exception);

  ///
  virtual void suppr(int id) throw();
  ///When too many sounds are already playing, removes the oldest (if not "ecrasable")
  virtual int play(int id) throw();
  //utilise LRU pour connaitre le channel ou jouer
  //play renvoie le temps du son en millisec

  ///Does not work well
  virtual void setVolume(int id,int volume) throw();
};



#endif
