/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



#include <string.h>
#include <iostream.h>
#include "Sprite.h"
//#include "GraphicMode.h"

Sprite::Sprite(int jSize,int iSize) throw ()
  //  :Array3<Pixel>(2,jSize,iSize)
  :size(jSize,iSize)
{
  data=new PixelMatrix(size.j,size.i);
  mask=new PixelMatrix(size.j,size.i);
  for(int j=0;j<size.j;j++)
    for(int i=0;i<size.i;i++) {
      (*data)[j][i]=toPixel(0,0,0,0);
      (*mask)[j][i]=PIXEL_MASK_EMPTY;
    }
}

Sprite::Sprite(const Sprite& sprite) throw()
  :size(sprite.size)
{
  data=new PixelMatrix(size.j,size.i);
  mask=new PixelMatrix(size.j,size.i);
  for(int j=0;j<size.j;j++)
    for(int i=0;i<size.i;i++) {
      (*data)[j][i]=(*sprite.data)[j][i];
      (*mask)[j][i]=(*sprite.mask)[j][i];
    }
}

Sprite::~Sprite() throw ()
{
  delete data;
  delete mask;
}

void Sprite::set(Pixel p,int j,int i) throw()
{
  if(getRedFromPixel(p)<=SPRITE_BOUND_LUZ && getGreenFromPixel(p)<=SPRITE_BOUND_LUZ && getBlueFromPixel(p)<=SPRITE_BOUND_LUZ) {
    setLuz(j,i);
  }
  else {
    (*data)[j][i]=p;
    (*mask)[j][i]=PIXEL_MASK_FULL;
  }
}

void Sprite::set(int r,int g,int b,int j,int i) throw()
{
  if(r<=SPRITE_BOUND_LUZ && g<=SPRITE_BOUND_LUZ && b<=SPRITE_BOUND_LUZ) {
    setLuz(j,i);
  }
  else {
    (*data)[j][i]=toPixel(r,g,b,0);
    (*mask)[j][i]=PIXEL_MASK_FULL;
  }
}

void Sprite::setWithoutShadow(int r,int g,int b,int j,int i) throw()
{
  (*data)[j][i]=toPixel(r,g,b,0);
  (*mask)[j][i]=PIXEL_MASK_FULL_WITHOUT_SHADOW;
}

void Sprite::setLuz(int j,int i) throw()
{
  (*data)[j][i]=PIXEL_DATA_ONLY_SHADOW;
  (*mask)[j][i]=PIXEL_MASK_ONLY_SHADOW;
}

void Sprite::noTakeLuz() throw()
{
  for(int j=0;j<mask->nbLine();j++)
    for(int i=0;i<mask->nbColumn();i++) {
      //%%      (*data)[j][i]=toPixel(0,0,0,0);
      //PIXEL      (*mask)[j][i] &= 0x7fff;//%%%%%%%%%%PIXEL_MASK_EMPTY;
      (*mask)[j][i] &= ~pixelGetLuz(toPixel(maxRedValue,maxGreenValue,maxBlueValue,1));//0x00ff;//%%%%%%%%%%PIXEL_MASK_EMPTY;
    }
}




