/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



#include <stdlib.h>
#include "String.h"
#include "List.h"
#include "error.h"

String::String() throw()
{
  str=new Array<char>(1);
  (*str)[0]=0;
}

String::String(const char *s) throw()
{
  if(!s) {
    str=new Array<char>(1);
    (*str)[0]=0;
  }
  int i=0;
  while(s[i]) i++;
  str=new Array<char>(i+1);
  for(int k=0;k<i;k++) (*str)[k]=s[k];
  (*str)[i]=0;
}

String::String(const String& s) throw()
{
  int i=s.size();
  str=new Array<char>(i+1);
  for(int k=0;k<i;k++) (*str)[k]=(*(s.str))[k];
  (*str)[i]=0;
}

String::String(List<char>& l) throw()
{
  int i=0;
  l.place();
  while(l.walk()) i++;
  str=new Array<char>(i+1);
  l.place();
  i=0;
  while(l.walk()) {
    (*str)[i]=l.get();
    i++;
  }
  (*str)[i]=0;
}

String& String::operator=(List<char>& l) throw()
{
  delete str;
  int i=0;
  l.place();
  while(l.walk()) i++;
  str=new Array<char>(i+1);
  l.place();
  i=0;
  while(l.walk()) {
    (*str)[i]=l.get();
    i++;
  }
  (*str)[i]=0;
  return *this;
}

String& String::operator=(const String& s) throw()
{
  delete str;
  int i=s.size();
  str=new Array<char>(i+1);
  for(int k=0;k<i;k++) (*str)[k]=(*(s.str))[k];
  (*str)[i]=0;
  return *this;
}


String::~String() throw()
{
  delete str;
}

String::operator const char*() const throw()
{
  return (const char*)(*str);
}

String::operator int() const throw()
{
  return toInt();
}

int String::toInt() const throw()
{
  bool withStar;
  int ret=privateToInt((const char*)(*str),withStar);
  if(withStar==true)
    throw Exception(ERROR_INT_CONVERSION,(const char*)(*str));
  return ret;
}

int String::toInt(bool& withStar) const throw()
{
  return privateToInt((const char*)(*str),withStar);
}

int String::privateToInt(const char *s,bool& withStar) const throw()
{
  if(s[0]=='r') {  //random
    //trouver le '-'
    int i=1;
    while(s[i]!=0) {
      if(s[i]=='-') {
	char *cbeg=new char[i];
	for(int k=0;k<i-1;k++) cbeg[k]=s[k+1];
	cbeg[i-1]=0;
	int beg=privateToInt(cbeg,withStar);
	int end=privateToInt(s+i+1,withStar)+1;//le nb marque est inclu
	if(beg>end)
	  throw Exception(ERROR_INT_CONVERSION,(const char*)(*str));
	return rand()%(end-beg)+beg;
      }
      i++;
    }
    throw Exception(ERROR_INT_CONVERSION,(const char*)(*str));
  }

  int i=0,base;
  bool signe=false;

  if(s[i]=='-') {
    signe=true;
    i++;
  }

  if(s[i]=='0') {
    i++;
    if(s[i]=='x') {
      base=16;
      i++;
    } else base=8;
  } else base=10;

  unsigned int ret=0;
  withStar=false;

  while(s[i]) {
    if(s[i]=='*')
      if(s[i+1]!=0)
	throw Exception(ERROR_INT_CONVERSION,s);
      else withStar=true;
    else {
      if(s[i]<'0' || s[i]>'9') {
	throw Exception(ERROR_INT_CONVERSION,s);
      }
      ret=ret*base+int(s[i]-'0');
    }
    i++;
  }
  if(signe) return -ret;
  return ret;
}

String::String(int i) throw()
{
  if(i==0) {
    str=new Array<char>(2);
    (*str)[0]='0';
    (*str)[1]=0;
    return;
  }
  List<char> l;
  bool neg=false;
  if(i<0) {
    i=-i;
    neg=true;
  }
  while(i>0) {
    l.placeCurrentOnQueue();
    l.put(char(i%10+int('0')));
    i/=10;
  }
  if(neg) {
    l.place();
    l.put('-');
  }
  int cpt=0;
  l.place();
  while(l.walk()) cpt++;
  str=new Array<char>(cpt+1);
  l.place();
  cpt=0;
  while(l.walk()) {
    (*str)[cpt]=l.get();
    cpt++;
  }
  (*str)[cpt]=0;
}

int String::size() const throw()
{
  return str->size()-1;
}

char String::operator[](int i) const throw()
{
  return (*str)[i];
}

char& String::operator[](int i) throw()
{
  return (*str)[i];
}

int String::operator==(const String& s) const throw()
{
  if(s.size()!=size()) return 0;
  for(int k=0;k<size();k++)
    if(s[k]!=(*this)[k]) return 0;
  return 1;
}

int String::operator==(const char *s) const throw()
{
  for(int k=0;k<size();k++) {
    if(s[k]==0) return 0;
    if(s[k]!=(*this)[k]) return 0;
  }
  if(s[size()]!=0) return 0;
  return 1;
}

String& String::operator+(const String& s) const throw()
{
  char *aux=new char[s.size()+size()+1];
  for(int k=0;k<size();k++) aux[k]=(*this)[k];
  for(int k=0;k<s.size();k++) aux[size()+k]=s[k];
  aux[s.size()+size()]=0;
  String *ret=new String(aux);
  delete[] aux;
  return *ret;
}
