/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



class TextFileModifier;

#ifndef __TextFileModifier_h__
#define __TextFileModifier_h__

#include "List.h"
#include "String.h"

//%%#define ERROR_TEXT_FILE_MODIFIER_END_OF_FILE "File modifier error : end of file"

/**
   Powerful class wich is able to modify a text file, and suppress the modifications if asked.
   I am happy, it works ! It was difficult !
   @author David FAUTHOUX
 */
class TextFileModifier {
 protected:
  ///
  enum Sens {
    REAL,
    IGNORE
  };
  ///
  struct SensChar {
    ///
    char c;
    ///
    Sens s;
    ///
    bool added;
  };

  ///char contained by the file
  List< SensChar * > list;

  ///list of the lines added when a line is added
  List< const String * > *mark;

  ///name of the file, used for save()
  const String *name;

 private:
  /*%%%
  ///
  bool slashFound;
  ///
  bool starFound;
  ///
  bool inString;
  ///
  bool blanking;
  ///
  int nbSeparator;
  ///
  int comment;//0:no, 1:line//, 2:big/ * * /
  */

  ///
  int cptWalk;

  ///
  int minLimit;
  ///
  int maxLimit;
  ///
  int tmpLimit;

 protected:
  ///
  List< int > listPosition;

  //%%  virtual void putChar(char c) throw();

  ///
  virtual void insertRaw(const String& line) throw();

  ///
  virtual bool walk() throw();
  ///
  virtual bool walkInvers() throw();

  ///
  virtual void reparse() throw();

 public:
  ///
  TextFileModifier(const String& nameFile,
		   List<const String *>& markList,
		   bool createFile=false) throw(Exception);
  ///
  virtual ~TextFileModifier() throw();

  ///
  virtual void save() throw(Exception);
  ///
  virtual void save(const String& nameFile) throw(Exception);

  ///
  virtual void place() throw();
  ///
  virtual bool walkIgnoring() throw();
  ///
  virtual bool walkInversIgnoring() throw();
  ///
  virtual char current() const throw(Exception);

  ///go begin to end (to find s and place after - on the final char of s)
  virtual bool after(const String& s) throw();
  ///go end to begin (to find s and place before - before the initial char of s)
  virtual bool before(const String& s) throw();

  ///go begin to end (to find s and place after - on the final char of s)
  virtual bool after(List< const String * >& l) throw();
  ///go end to begin (to find s and place before - before the initial char of s)
  virtual bool before(List< const String * >& l) throw();

  ///
  virtual void insert(List<const String*>& lines) throw();

  ///
  virtual void insert(const String& line) throw();


  ///
  virtual void jumpBlank() throw();
  ///
  virtual void jumpBlankInvers() throw();

  
  /// return deleted by caller
  virtual String *nextWord() throw();


  ///
  virtual void keepPosition() throw();
  ///
  virtual void recallPosition() throw(Exception);

  ///put a limit not outrunnable, one limit only can be put
  virtual void limit() throw();
  ///
  virtual void unlimit() throw();

  ///
  virtual void goEndLine() throw();

  ///
  bool findCloseBrace() throw();
};

#endif
