/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



class TextPainter;

#ifndef __TextPainter_h__
#define __TextPainter_h__

#include "Pixel.h"
#include "Font.h"
#include "Ini.h"

/**
   This class can paint words on screen. It is capable to create needed fonts (colored) from the font taken in .ini file.
   @author David FAUTHOUX
 */
class TextPainter {
 protected:
  ///
  List< Font * > list;
  ///Reference to the first element of the list loaded with default values in constructor
  Font *defaultFont;

  ///
  void paint(WindowPipe& windowPipe,int j,int i,const String& text,const Font& font,bool to) throw();

 public:
  ///
  TextPainter(Ini& ini) throw(Exception);

  ///
  ~TextPainter() throw();

  ///
  void paint(WindowPipe& windowPipe,int j,int i,const String& text,Pixel fontColor,Pixel backgroundColor,bool to=false) throw();
  ///
  void paint(WindowPipe& windowPipe,int j,int i,const String& text,Pixel fontColor,bool to=false) throw();
  ///
  void paint(WindowPipe& windowPipe,int j,int i,const String& text,bool to=false) throw();

  ///
  int nbLine() const throw() { return defaultFont->nbLine(); }
  ///
  int nbColumn() const throw() { return defaultFont->nbColumn(); }

  ///
  Pixel getDefaultFontColor() const throw() { return defaultFont->getFontColor(); }
  ///
  Pixel getDefaultBackgroundColor() const throw() { return defaultFont->getBackgroundColor(); }

};

#endif
