/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



#include "Trou.h"
#include "Grille.h"
#include "LoopAnim.h"

#define DEFAULT_SLOW_ANIMATION 0
#define DEFAULT_SLOW_TROU 500
#define DEFAULT_DECAL_TROU 40
#define DEFAULT_RANDOM_TROU 20
#define DEFAULT_TREMBLE_TROU 10
#define DEFAULT_COLOR_PIXEL_TROU toPixel(15,15,15,0)
#define DEFAULT_AMOUNT_PIXEL_TROU 20
#define INIT_VIT_PIXEL 10

void Trou::walkPixel() throw()
{
  listPixel.place();
  while(listPixel.walk()) {
    Cinq *q=listPixel.get();
    q->j+=q->vj;
    q->i+=q->vi;
    q->vj++;
  }
}

int Trou::virtualWalk() throw()
{
  walkPixel();
  listToBePaint.place();
  while(listToBePaint.walk())
    if(!listToBePaint.get()->walk()) {
      delete listToBePaint.get();
      listToBePaint.suppr();
    }
  return 1;
}

Trou::Trou(Ini& ini,Sounder& sounder,Grille& grille) throw(Exception)
  :sounderRef(&sounder),
   grilleRef(&grille),
   colPixel(ini.getWithDefaultColor("[Trou]",0,"color pixel trou",0,DEFAULT_COLOR_PIXEL_TROU)),
   amountPixel(ini.getWithDefault("[Trou]",0,"amount pixel trou",0,DEFAULT_AMOUNT_PIXEL_TROU)),
   soundId(sounder.add(*ini.get("[Trou]",0,"sound trou",0))),
   cptAnim(0)
{
  setUppest();
  setTo();
  List<const String *> *l=ini.getList("[Trou]",0,"trou","[Trou]");
  l->place();
  while(l->walk())
    MultiAnim::add(*new Anim(*(l->get()),
			     ini.getWithDefault("[Slow]",0,
						"Animation",0,
						DEFAULT_SLOW_ANIMATION),
			     ini.getListSeparator()));
  delete l;

  decal.j=ini.getWithDefault("[Trou]",0,
			     "decal trou j",0,DEFAULT_DECAL_TROU);
  decal.i=ini.getWithDefault("[Trou]",0,
			     "decal trou i",0,DEFAULT_DECAL_TROU);
  decalRand.j=ini.getWithDefault("[Trou]",0,
				 "random trou j",0,DEFAULT_RANDOM_TROU);
  decalRand.i=ini.getWithDefault("[Trou]",0,
				 "random trou i",0,DEFAULT_RANDOM_TROU);
  tremble.j=ini.getWithDefault("[Trou]",0,
			       "tremble trou j",0,DEFAULT_TREMBLE_TROU);
  tremble.i=ini.getWithDefault("[Trou]",0,
			       "tremble trou i",0,DEFAULT_TREMBLE_TROU);
  addLoop(ini.getWithDefault("[Trou]",0,"slow trou",0,DEFAULT_SLOW_TROU),0);
}

Trou::~Trou() throw()
{
  listToBePaint.place();
  while(listToBePaint.walk()) delete listToBePaint.get();
  listPixel.place();
  while(listPixel.walk()) delete listPixel.get();
}

void Trou::fire() throw()
{
  CounterContainer::inc();
}

void Trou::addPixel(int j,int i,Pixel col) throw()
{
  for(int k=0;k<amountPixel;k++)
    listPixel.put(new Cinq(j,i,
			   -rand()%INIT_VIT_PIXEL,
			   -INIT_VIT_PIXEL/2+rand()%INIT_VIT_PIXEL,col));
}

void Trou::endOfCount(int) throw()
{
  sounderRef->play(soundId);
  int j=pos.j+rand()%(decalRand.j+1);
  int i=pos.i+rand()%(decalRand.i+1);
  addPixel(j,i,colPixel);
  Anim *a=new LoopAnim(*getAnim(cptAnim));
  a->move(j,i);
  listToBePaint.put(a);

  Trembleur *trembleurRef=grilleRef->getTrembleur();
  if(trembleurRef) trembleurRef->inc(tremble.j,tremble.i);

  move(pos.j-decal.j/2+rand()%(decal.j+1),
       pos.i+rand()%(decal.i+1));
  cptAnim++;
  if(cptAnim==length) cptAnim=0;
}

void Trou::paint(WindowPipe& windowPipe) throw()
{
  int j,i,sj,si;
  windowPipe.getTo(j,i,sj,si);
  int jPos=sj/2;
  listToBePaint.place();
  while(listToBePaint.walk()) {
    Anim *a=listToBePaint.get();
    int j,i;
    a->getPos(j,i);
    if(i>=si) {
      delete listToBePaint.get();
      listToBePaint.suppr();
      move(0,0);
    }
    else {
      a->move(jPos+j,i);
      a->paint(windowPipe);
      a->move(j,i);
    }
  }

  listPixel.place();
  while(listPixel.walk()) {
    int j=listPixel.get()->j;
    if(j>=sj) {
      delete listPixel.get();
      listPixel.suppr();
    }
    else {
      //%%      a.set(listPixel.get()->col,0,0);
      windowPipe.putTo(*(listPixel.get()->s),
		       jPos+j,
		       listPixel.get()->i,true);
    }
  }

}

void Trou::reinit() throw()
{
  move(0,0);
  listToBePaint.place();
  while(listToBePaint.walk()) {
    delete listToBePaint.get();
    listToBePaint.suppr();
  }
  MultiAnim::reinit();
}
