/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



/*
  Code of my friend Ulrich ROISSARD - 1999
 */

class Vector;

#ifndef __Vector_h__
#define __Vector_h__

#include <math.h>
#include <iostream.h>

///
class Vector {
 protected:
  ///
  float vector[3];
 public:
  ///
  Vector() throw() {
    vector[0]=0.0;
    vector[1]=0.0;
    vector[2]=0.0;
  }

  ///
  Vector(float x,float y,float z) throw() {
    vector[0]=x;
    vector[1]=y;
    vector[2]=z;
  }  

  ///
  ~Vector() throw() {}

  ///
  float operator[](int i) const throw() {
    return vector[i];
  }

  ///
  float &operator[](int i) throw() {
    return vector[i];
  }

  ///
  Vector operator+(const Vector &v) const throw() {
    Vector tmp;

    for (int i=0;i<3;i++)
      tmp.vector[i]=vector[i]+v.vector[i];

    return tmp;
  }

  ///
  Vector operator-(const Vector &v) const throw() {
    Vector tmp;

    for (int i=0;i<3;i++)
      tmp.vector[i]=vector[i]-v.vector[i];

    return tmp;
  }

  ///
  Vector operator-() const throw() {
    Vector tmp;

    for (int i=0;i<3;i++)
      tmp.vector[i]=-vector[i];

    return tmp;
  }

  ///
  float operator*(const Vector &v) const throw() {
    float tmp=0;

    for (int i=0;i<3;i++)
      tmp=tmp+vector[i]*v.vector[i];

    return tmp;
  }

  ///
  Vector operator*(float v) const throw() {
    Vector tmp;

    for (int i=0;i<3;i++)
      tmp.vector[i]=vector[i]*v;

    return tmp;
  }

  ///
  Vector operator/(float v) const throw() {
    Vector tmp;

    for (int i=0;i<3;i++)
      tmp.vector[i]=vector[i]/v;

    return tmp;
  }

  ///
  float norm() const throw() {
    float tmp=0;

    for (int i=0;i<3;i++)
      tmp=tmp+pow(vector[i],2);

    return sqrt(tmp);
  }

  ///
  Vector normalize() const throw() {
    if (norm()==0)
      return Vector(0.0,0.0,0.0);
    return (*this)/norm();
  }

  ///
  friend Vector operator*(float f,const Vector &v) throw() {
    return v*f;
  }

  ///
  friend ostream &operator<<(ostream &os,const Vector &v) throw() {
    return os << "<" << 
      v.vector[0] << "," <<
      v.vector[1] << "," <<
      v.vector[2] << ">";
  }
};


#include "Vector.cc" 
//useless

#endif
