/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */


class Window;

#ifndef __Window_h__
#define __Window_h__

#include "PixelMatrix.h"
//#include "WindowPipe.h"
#include "Tji.h"
#include "Sprite.h"

// Cette classe est une couche de gestion sur tableau, elle clippe et 
// elle appelle les fonctions assembleur.

// Quand elle a besoin d'un Array3, c'est que la premiere dimension est
// de 2 : donnee sprite + masque

//ATTENTION recouvrement des window non gere (pour copies)

/**
   Used to clip Sprite painting. Very useful.
   Attention : Window overlay not managed !
   @author David FAUTHOUX
 */
class Window {
 private:

  ///
  friend class WindowPipe;

  ///
  struct WindowDef {
    ///
    Tji pos;
    ///
    Tji size;
    ///
    Tji saveSize;
    ///
    Tji savePos;
    ///
    bool print;
    ///
    WindowDef(int jSize,int iSize) {
      savePos.j=savePos.i=0;
      pos.j=pos.i=0;
      saveSize.j=size.j=jSize;
      saveSize.i=size.i=iSize;
      print=true;
    }
  };
  ///
  WindowDef w;

  ///
  PixelMatrix *const tab;

  ///
  void clip(int& jPos,int& iPos,
	    int& jSize,int& iSize,
	    int& jDecal,int& iDecal) const throw ();

 public:
  ///ref is used to know the limits
  Window(PixelMatrix& ref) throw ();

  ///
  ~Window() throw() {}

  ///
  void set(int jPos,int iPos,int jSize,int iSize) throw ();

  ///
  void get(int& jPos,int& iPos,int& jSize,int& iSize) const throw();

  ///
  void set(int jPos,int iPos) throw ();

  ///
  void copy(const Window& a) throw ();

  ///
  void putSpriteColor(Pixel color,const Sprite& a,int jPos,int iPos) throw();

  ///
  void putSprite(const Sprite& a,int jPos,int iPos) throw();

  ///
  void putSpriteUnderMask(const Sprite& a,int jPos,int iPos,
			  const UnderMaskMatrix& underMask,
			  int jUnderMask,int iUnderMask) throw ();

  ///
  void putSpriteUnderMaskPalette(const Sprite& a,int jPos,int iPos,
				 const UnderMaskMatrix& underMask,
				 int jUnderMask,int iUnderMask,
				 const PixelPalette& pal) throw ();


  ///
  void clear() throw();

};

#endif





