/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



class WindowPipe;

#ifndef __WindowPipe_h__
#define __WindowPipe_h__

#include "Window.h"
#include "List.h"

// Cette classe est un tube de connexion entre le fond et l'ecran ('from'/'to')
// elle clippe tout et modifie les positions pour afficher au bon endroit.
// Il suffit d'appeler ses methodes avec des positions relatives au tableau
// sur lequel on travaille, ce tableau etant connecte a WindowPipe a travers
// la couche Window qui clippe.

// Ce tube permet de bouger autant la window 'to' que la window 'from' tout
// en gardant la coherence des positions.

// On peut donc scroller le fond ('from'), trembler l'ecran ('to'), faire
// tomber des pages ('to'), etc... (a votre imagination...)

/**
   This class connect two Windows in order to put Sprite relatively to the Windows positions. Very useful.
   With this class, no clipping or position calculating problem !
   You can scroll and paint everything you want without any problem.
   Just call refresh() at the end (before the GraphicMode.refresh()).
   Usually, the windowTo is the window for GraphicMode (new Window(graphicMode));
   @see Window
   @see GraphicMode
   @see Sprite
   @see Anim
   @author David FAUTHOUX
 */
struct WindowPipe {
 private:
  ///
  Window *const to;
  ///
  Window *const from;
  ///
  const UnderMaskMatrix *const mask;

  ///
  bool windowToToClear;

  ///
  struct T_Zlist {
    ///
    const Sprite *s;
    ///
    int j;
    ///
    int i;
    ///
    const PixelPalette *pal;
  };
  ///
  struct T_ZColorlist {
    ///
    const Sprite *s;
    ///
    int j;
    ///
    int i;
    ///
    Pixel color;
  };
  ///
  List< T_Zlist * > Zlist;
  ///
  List< T_Zlist * > ZlistOn;
  ///
  List< T_Zlist * > ZlistTo;
  ///
  List< T_Zlist * > ZlistToUppest;
  ///
  List< T_ZColorlist * > ZlistColorTo;

  ///
  void realPaintWindowFrom() throw();
  ///
  void realPut(const Sprite& s,int jFromRelativePos,int iFromRelativePos,const PixelPalette *pal) throw ();
  ///
  void realPutOn(const Sprite& s,int jFromRelativePos,int iFromRelativePos) throw();
  ///
  void realPutTo(const Sprite& s,int jPos,int iPos) throw();
  ///
  void realPutColorTo(Pixel color,const Sprite& s,int jPos,int iPos) throw();

 public:
  ///
  WindowPipe(Window& aTo,Window& aFrom,const UnderMaskMatrix& underMask) throw ();

  /*
  WindowPipe(Window& aTo,
	     Window& aFromData,Window& aFromMask) throw ();
  */

  ///
  ~WindowPipe() throw ();

  ///
  void paintWindowFrom() throw();//vide la liste
  ///Relative under mask
  void put(const Sprite& s,int jFromRelativePos,int iFromRelativePos) throw ();//ajoute a la liste
  ///Relative
  void putOn(const Sprite& s,int jFromRelativePos,int iFromRelativePos) throw ();
  ///No relative
  void putTo(const Sprite& s,int jPos,int iPos,bool uppest) throw ();
  ///under mask
  void putUnder(const Sprite& s,int jPos,int iPos) throw ();

  ///No relative
  void putColorTo(Pixel color,const Sprite& s,int jPos,int iPos) throw ();

  ///Relative under mask
  void putWithPalette(const Sprite& s,int jFromRelativePos,int iFromRelativePos,const PixelPalette& pal) throw ();//ajoute a la liste

  ///Paint the background (windowFrom) and paint the Sprite according to the put order
  void refreshAll() throw();

  ///
  void moveFrom(int j,int i) throw ();
  ///
  void moveTo(int j,int i) throw ();

  ///
  void getFrom(int& jPos,int& iPos,int& jSize,int& iSize) const throw();
  ///
  void getTo(int& jPos,int& iPos,int& jSize,int& iSize) const throw();

  ///
  void clearTo() throw();

};



#endif



