/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



#include <stdlib.h>
#include "util.h"
#include "File.h"
#include "GifIn.h"
#include "GraphicMode.h"
#include "Keyer.h"
#include "Tji.h"
#include "Background.h"
#include "Ini.h"
#include "TextPainter.h"
#include "GrilleUtilCase.h"

/*%%%
char *choice[]={
"vide",
"mur indestructible",
"mur haut",
"vide force",
"escalier",
"steady flamme",
"steady option bombe",
"steady option flamme",
"steady option ouba",
"steady option armageddon",
"steady option chicken",
"steady option photo",
"steady option feu au cul",
"steady option super bombe",
"steady option mine",
"steady option dynamite",
"steady option invincible",
"steady option invisible",
"steady option pose mur",
"steady option surpuissance",
"steady option win95",
"steady option traverse mur",
"steady option femme",
"steady option teleport",
"steady option resurrect",
"steady option speed",
0};
*/

class ImmobileTextContainer {
protected:
  TextPainter *const textPainterRef;
  List< String * > list;
  GraphicMode *const gRef;
  int current,active;
  Pixel activeFontColor;
  Pixel currentFontColor;
  int maxActive,maxOnLine;

public:
  ImmobileTextContainer(TextPainter& textPainter,GraphicMode& g,Pixel activeColor,Pixel currentColor) throw()
    :textPainterRef(&textPainter),gRef(&g),current(0),active(0),activeFontColor(activeColor),currentFontColor(currentColor),maxActive(0),maxOnLine(gRef->nbLine()/textPainterRef->nbLine()) {
  }
  virtual ~ImmobileTextContainer() throw() {
    list.place();
    while(list.walk()) delete list.get();
  }

  virtual void add(const String& mot) throw() {
    list.placeCurrentOnHead();
    list.put(new String(mot));
    maxActive++;
  }

  virtual void up() throw() {
    if(active>0) {
      active--;
    }
  }
  virtual void down() throw() {
    if(active+1<maxActive) {
      active++;
    }
  }
  virtual void left() throw() {
    if(active>=maxOnLine) {
      active-=maxOnLine;
    }
  }
  virtual void right() throw() {
    if(active+maxOnLine<maxActive) {
      active+=maxOnLine;
    }
  }

  virtual void setCurrent(int cur) throw() {
    current=cur;
  }

  virtual int getActive() throw() { return active; }

  virtual void paint(WindowPipe& windowPipe) throw() {
    int k=0,j=0,i=0,iMax=0;
    list.place();
    while(list.walk()) {
      if(k==current) textPainterRef->paint(windowPipe,j,i,*list.get(),currentFontColor,true);
      else if(k==active) textPainterRef->paint(windowPipe,j,i,*list.get(),activeFontColor,true);
      else textPainterRef->paint(windowPipe,j,i,*list.get(),true);
      if(iMax<list.get()->size()) iMax=list.get()->size();
      k++;
      j+=textPainterRef->nbLine();
      if(j==gRef->nbLine()) {
	j=0;
	i+=(iMax+1)*textPainterRef->nbColumn();
	iMax=0;
      }
    }
  }

};


class KeyTaker:public Keyable {
public:
  Tji *posRef;
  bool stop;
  bool canceled;
  Tji max;
  bool inChoice;
  ImmobileTextContainer *const textContainer;
  Array< int > *tab;
  int cptEsc;
  int tabSize;
  Tji caseSize;
  bool upNiv,downNiv;

  KeyTaker(Tji& pos,
	   int jMax,int iMax,
	   ImmobileTextContainer& immobileTextContainer,
	   Array< int >& tabValue,int iSize,
	   int jCaseSize,int iCaseSize) throw()
    :posRef(&pos),
     stop(false),
     canceled(false),
     max(jMax,iMax),
     inChoice(false),
     textContainer(&immobileTextContainer),
     tab(&tabValue),
     cptEsc(0),
     tabSize(iSize),
     caseSize(jCaseSize,iCaseSize),upNiv(false),downNiv(false) {
    textContainer->setCurrent((*tab)[(posRef->j/caseSize.j)*tabSize+posRef->i/caseSize.i]);
  }

  virtual ~KeyTaker() throw() {}

  void keyAction(const Key& key) throw() {
    if(key.isUp()) {
      switch(key.getCode()) {
      case SPACE:
	inChoice=false;
	(*tab)[(posRef->j/caseSize.j)*tabSize+posRef->i/caseSize.i]=textContainer->getActive();
	break;
      case SCROLLOCK:
	canceled=true;
	stop=true;
	return;
      default:
	break;
      }
    }
    if(key.isDown()) {
      switch(key.getCode()) {
      case PAGEUP:
	upNiv=true;
	break;
      case PAGEDOWN:
	downNiv=true;
	break;
      case ESCAPE:
	if(inChoice) inChoice=false;
	else {
	  cptEsc++;
	  if(cptEsc>=3) stop=true;
	  return;
	}
	break;
      case RETURN:
	if(inChoice) {
	  inChoice=false;
	  (*tab)[(posRef->j/caseSize.j)*tabSize+posRef->i/caseSize.i]=textContainer->getActive();
	} else {
	  inChoice=true;
	  textContainer->setCurrent((*tab)[(posRef->j/caseSize.j)*tabSize+posRef->i/caseSize.i]);
	}
	break;
      case SPACE:
	inChoice=true;
	textContainer->setCurrent((*tab)[(posRef->j/caseSize.j)*tabSize+posRef->i/caseSize.i]);
	break;
      case LEFT:
	if(inChoice) textContainer->left();
	else if(posRef->i>0) posRef->i-=caseSize.i;
	break;
      case RIGHT:
	if(inChoice) textContainer->right();
	else if(posRef->i+caseSize.i<max.i) posRef->i+=caseSize.i;
	break;
      case UP:
	if(inChoice) textContainer->up();
	else if(posRef->j>0) posRef->j-=caseSize.j;
	break;
      case DOWN:
	if(inChoice) textContainer->down();
	else if(posRef->j+caseSize.j<max.j) posRef->j+=caseSize.j;
	break;
      default:
	break;
      }
      cptEsc=0;
    }
  }
};

//%%typedef const String * constStringStar;

#define DEFAULT_DECAL_SHADOW -10

int main(int argc,char **argv)
{
  //%%  constStringStar *choice=0;

  if(argc!=4) {
    cerr<<"This program enables you to automatically create the file wich describes a \"Fond\":\n";
    cerr<<"it defines where are the walls, the grounds...\n";
    cerr<<"Use it with cursor keys and 'space' or 'enter', and 'page up' 'page down' for levels\n";
    cerr<<"Exit with 'Esc' (press it three times)\n";
    cerr<<"It automatically saves the file,\nif you want to cancel, do not press 'Esc', but press 'Scroll Lock'\n";
    cerr<<"Usage :"<<argv[0]<<" bomb.ini Fond.gif Fond.grille\n";
    cerr.flush();
    return 1;
  }

  Tji caseSize(20,20);

  Ini ini(argv[1]);

  GraphicMode graphicMode(ini.getWithDefault("[Palette]",0,"decal shadow",
					     0,DEFAULT_DECAL_SHADOW));

  TextPainter textPainter(ini);

  ImmobileTextContainer textContainer(textPainter,graphicMode,toPixel(255,0,0,0),toPixel(0,255,0,0));

  //

  GrilleUtilCase grilleUtilCase;

  /*%%
  int nbChoice=0;
  grilleUtilCase.list.place();
  while(grilleUtilCase.list.walk()) nbChoice++;

  choice=new constStringStar[nbChoice];

  int kChoice=0;
  grilleUtilCase.list.place();
  while(grilleUtilCase.list.walk())
    choice[kChoice++]=&grilleUtilCase.list.get()->name;
    //%%    textContainer.add(grilleUtilCase.list.get()->name);

  for(int k=0;k<nbChoice;k++) textContainer.add(*(choice[k]));
  */

  grilleUtilCase.list.place();
  while(grilleUtilCase.list.walk())
    textContainer.add(grilleUtilCase.list.get()->name);


    //

  Tji pos(0,0);

  //%%  List< Sprite * > background;
  GifIn gif(argv[2]);//%%Background background(new String(argv[2]),graphicMode,30,30);

  List< PixelMatrix * > gifList;

  gif.place();
  while(gif.walk()) gifList.put(gif.get()->getData());

  gifList.place();
  gifList.walk();

  Tji backgroundSize(gifList.get()->nbLine(),gifList.get()->nbColumn());
  Tji backSize(backgroundSize.j/caseSize.j,backgroundSize.i/caseSize.i);

  Array< int > tab(backSize.j*backSize.i);

  tab=0;

  if(File::exists(argv[3])) {

    try {
      Ini iniGrille(argv[3]);

      int k=0;
      grilleUtilCase.list.place();
      while(grilleUtilCase.list.walk()) {
	//%%      for(int k=1;k<nbChoice;k++) {
	
	List<const String *> *coord=iniGrille.getListWithoutException("[Grille]",0,
								      //%%      *(choice[k]));
								      grilleUtilCase.list.get()->name);
	coord->place();
	while(coord->walk()) {
	  
	  List<int> *listInt=new List<int>;
	  if(extractInt(*(coord->get()),iniGrille.getListSeparator(),*listInt)) {
	    listInt->place();
	    if(listInt->walk() && listInt->walk()) {
	      
	      listInt->place();
	      listInt->walk();
	      int j=listInt->get();
	      listInt->walk();
	      int i=listInt->get();
	      
	      if(j>=0 && j<backSize.j && i>=0 && i<backSize.i)
		tab[i+j*backSize.i]=k;
	    }
	  }
	}

	k++;
      }
    }
    catch(Exception) {
      cerr<<"Continuing...\n";cerr.flush();
    }
  }


  Keyer keyer;
  KeyTaker keyTaker(pos,backgroundSize.j,backgroundSize.i,textContainer,tab,backSize.i,caseSize.j,caseSize.i);
  keyer.add(keyTaker);

  graphicMode.on();

  Tji center(graphicMode.nbLine()/2,graphicMode.nbColumn()/2);
  Tji realPos(center.j,center.i);

  Sprite sprite(caseSize.j,caseSize.i);

  for(int j=0;j<caseSize.j;j++)
    for(int i=0;i<caseSize.i;i++)
      sprite.set(0,0,0,j,i);

  for(int i=0;i<caseSize.i;i++)
    sprite.set(255,0,0,caseSize.j/2,i);
  for(int j=0;j<caseSize.j;j++)
    sprite.set(255,0,0,j,caseSize.i/2);

  PixelMatrix matrix(backgroundSize.j,backgroundSize.i);
  gifList.place();
  gifList.walk();
  matrix=*gifList.get();

  gifList.place();
  gifList.walk();
  int cptGifList=0;

  Window wg(graphicMode);
  Window wb(matrix);
  UnderMaskMatrix underMaskMatrix(graphicMode.nbLine(),graphicMode.nbColumn());
  WindowPipe windowPipe(wg,wb,underMaskMatrix);

  while(!keyTaker.stop) {

    if(keyTaker.upNiv) {
      if(gifList.walk()) {
	matrix=*gifList.get();
	cptGifList++;
      }
      keyTaker.upNiv=false;
    }
    if(keyTaker.downNiv) {
      if(cptGifList>0) {
	if(gifList.walkHeadToQueue()) {
	  matrix=*gifList.get();
	  cptGifList--;
	}
      }
      keyTaker.downNiv=false;
    }

    if(realPos.j<pos.j && realPos.j-center.j+graphicMode.nbLine()<backgroundSize.j) realPos.j++;
    if(realPos.j>pos.j && realPos.j-center.j>0) realPos.j--;
    if(realPos.i<pos.i && realPos.i-center.i+graphicMode.nbColumn()<backgroundSize.i) realPos.i++;
    if(realPos.i>pos.i && realPos.i-center.i>0) realPos.i--;

    //%%    background.getWindowPipe()->moveFrom(realPos.j-center.j,realPos.i-center.i);
    windowPipe.moveFrom(realPos.j-center.j,realPos.i-center.i);

    //%%    background.paint();
    //%%    background.getWindowPipe()->putOn(sprite,pos.j,pos.i);
    windowPipe.putOn(sprite,pos.j,pos.i);

    if(keyTaker.inChoice)
      textContainer.paint(windowPipe);
    //%%      textContainer.paint(*background.getWindowPipe());
    
    //%%    background.getWindowPipe()->refreshAll();
    windowPipe.refreshAll();

    graphicMode.refresh();
    keyer.walk();
  }

  graphicMode.off();

  if(!keyTaker.canceled) {
    
    File fileGrille(argv[3],File::WRITE_ONLY);
    
    cout<<"Saving ini file, size="<<backSize.j<<","<<backSize.i<<"\n";cout.flush();
    
    fileGrille.write("# See this file in color : in emacs, type M-x makefile-mode\n[Grille]=\n");
    
    char sep[2]={ini.getListSeparator(),0};
    
    for(int j=0;j<backSize.j;j++)
      for(int i=0;i<backSize.i;i++) {
	int k=tab[i+j*backSize.i];

	if(k>0) {
	  int cptAux=0;
	  grilleUtilCase.list.place();
	  grilleUtilCase.list.walk();
	  while(cptAux<k) {
	    grilleUtilCase.list.walk();
	    cptAux++;
	  }
	  fileGrille.write((const char *)(grilleUtilCase.list.get()->name));
	  
	  //%%	  fileGrille.write((const char *)*(choice[k]));
	  fileGrille.write(":");
	  fileGrille.write((const char*)*new String(j));
	  fileGrille.write(sep);
	  fileGrille.write((const char*)*new String(i));
	  fileGrille.write("\n");
	}
      }
    
    fileGrille.write("[]\n[Global]=\nlist separator:,\n[]\n");
    
    cout<<"Ok.\n";cout.flush();
  }

  return 0;
}
