/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */

//#define FPS  // complie le compteur de fps


#include "display.h"
#include "SDL.h"
#include "SDL_endian.h" /* Used for the endian-dependent 24 bpp mode */
#include <string.h>
#include "Fifo.h"

#ifdef FPS
  #include <time.h>
  time_t temps = time(NULL);
  int frame=0;
#endif

#define FACTOR 2
SDL_Surface *screen;
int bpp;


bool graphicActif=false;
int oldMode=0;
bool keyboardActif=false;

Fifo<int> keyboardCode;
Fifo<bool> keyboardPressed;

//#define NBKEY 255
#define NBKEY 1024
bool switchKeyboardCode[NBKEY];//true if pressing

void keyInBuffer(int key,int press)
{
  keyboardCode.put(key);
  if(press==SDL_RELEASED) keyboardPressed.put(false);
  else keyboardPressed.put(true);
}

bool enableKeyboard()
{
  keyboardActif=true;
  for(int k=0;k<NBKEY;k++) switchKeyboardCode[k]=false;
  return true;
}

void disableKeyboard()
{
  if(keyboardActif) {
    keyboardActif=false;
    while(!keyboardCode.empty()) keyboardCode.get();
    while(!keyboardPressed.empty()) keyboardPressed.get();
    for(int k=0;k<NBKEY;k++)
      if(switchKeyboardCode[k]) {
	switchKeyboardCode[k]=false;
	keyboardCode.put(k);
	keyboardPressed.put(false);
      }
  }
}

bool keyboardRefresh()
{
  // On appelle ici keyInBuff (meme effet que le handler )
  SDL_Event event;
  while( SDL_PollEvent( &event ) )
    {
     if( event.type==SDL_KEYDOWN || event.type==SDL_KEYUP )
         keyInBuffer( event.key.keysym.sym, event.key.state );
    }
  return true;
}

void getKeyboardCode(int& code,bool& pressed)
{
  //  keyboard_update();
  if(keyboardCode.empty()) {
    code=-1;
    return;
  }
  code=keyboardCode.get();
  pressed=keyboardPressed.get();
  if(pressed) switchKeyboardCode[code]=true;
  else switchKeyboardCode[code]=false;
}

//


PhysicPixel *video=new PhysicPixel[320*200];
Pixel *buffer=new Pixel[320*200];


bool enableGraphic()
{
  SDL_Init( SDL_INIT_VIDEO );
  atexit(SDL_Quit);

  SDL_EnableKeyRepeat( 1, SDL_DEFAULT_REPEAT_INTERVAL );

  bool remettreKeyboard=false;
  if(graphicActif) return false;
  if(keyboardActif) {
    remettreKeyboard=true;
    disableKeyboard();
  }

  bool ret=true;
// pas sur que ca marche partout
// barqk......................  // FIX : il faut metre le choix de l'ico dans le .ini
SDL_WM_SetIcon( SDL_LoadBMP("Page/.bmp"), NULL );
SDL_WM_SetCaption( "Bomb  Bomb", "BB" );
// ........................

      if( (screen=SDL_SetVideoMode( graphicWidth()*FACTOR, graphicHeight()*FACTOR, 0, SDL_SWSURFACE )) )
          { ret=true; bpp = screen->format->BytesPerPixel; }
        else
          ret=false;

  if(remettreKeyboard) enableKeyboard();
  graphicActif=true;
  return ret;
}

void disableGraphic( int mode )
{
  bool remettreKeyboard=false;
  if(graphicActif) {
    if(keyboardActif) {
      remettreKeyboard=true;
      disableKeyboard();
    }
    // Le mode text n'est pas actif (il le faudrait dans la fenetre SDL (????)
    if( mode == 0 )
      {
       SDL_FreeSurface( screen );//vga_setmode(TEXT);
       SDL_Quit();
      }

    if(remettreKeyboard) enableKeyboard();
    graphicActif=false;
  }
}

int graphicWidth()
{
  return 320;
}

int graphicHeight()
{
  return 200;
}


void graphicRefresh(const Pixel *buffer)
  //buffer size=sizeof(Pixel)*graphicWitdth()*graphicHeight();
{
#ifdef FPS
  // Calcule les FPS
  if( time(NULL) != temps )
    { temps = time(NULL); printf( "FPS: %i\n", frame ); frame = 0; }
   else
    { frame++; }
#endif

  if(!graphicActif) return;
  for(int k=0;k<320*200;k++)
    {
     Uint32 color = SDL_MapRGB( screen->format,
                      getRedFromPixel(buffer[k]),
                      getGreenFromPixel(buffer[k]),
                      getBlueFromPixel(buffer[k]) );

#if FACTOR == 1
    Uint16 x = k%320, y = k/320;

    switch (bpp) {
        case 2: { // Certainement 15 ou 16 bpp
            Uint16 *bufp;
            bufp = (Uint16 *)screen->pixels + y*screen->pitch/2 + x;
            *bufp = color;
        } break;
        case 3: { // 24 bpp lent et gnralement pas utilis
            Uint8 *bufp;
            bufp = (Uint8 *)screen->pixels + y*screen->pitch + x * 3;
            if(SDL_BYTEORDER == SDL_LIL_ENDIAN) {
                bufp[0] = color;
                bufp[1] = color >> 8;
                bufp[2] = color >> 16;
            } else {
                    bufp[2] = color;
                bufp[1] = color >> 8;
                bufp[0] = color >> 16;
            }
        } break;
        case 4: { // Probablement 32 bpp alors
            Uint32 *bufp;
            bufp = (Uint32 *)screen->pixels + y*screen->pitch/4 + x;
            *bufp = color;
        }
        break;
    }
#endif
#if FACTOR == 2
     Uint16 x = k%320*2, y = k/320*2;
    switch (bpp) {
        case 2: { // Certainement 15 ou 16 bpp
            Uint16 *bufp = (Uint16 *)screen->pixels + x + y*screen->pitch/2;
            *bufp = color;
            bufp++;
            *bufp = color;
            bufp += 320*2;
            *bufp = color;
            bufp--;
            *bufp = color;
        } break;
        case 3: { // 24 bpp lent et gnralement pas utilis
            Uint32 *bufp, col;
            Uint8 col0=color, col1=color>>8, col2=color>>16;
            if(SDL_BYTEORDER == SDL_LIL_ENDIAN)
                col = col0 & col1<<8 & col2<<16;
              else
                col = col2 & col1<<8 & col0<<16;
            bufp = (Uint32 *)screen->pixels + y*screen->pitch + x * 3;
            *bufp = col;
            bufp++;
            *bufp = col;
            bufp += 320*3;
            *bufp = col;
            bufp--;
            *bufp = col;
        } break;
        case 4: { // Probablement 32 bpp alors
            Uint32 *bufp;
            bufp = (Uint32 *)screen->pixels + y*screen->pitch/4 + x;
            *bufp = color;
            bufp++;
            *bufp = color;
            bufp += 320*2;
            *bufp = color;
            bufp--;
            *bufp = color;
        }
        break;
    }
#endif
#if FACTOR > 2 // Marche dans touts les cas mais plus lent
     SDL_Rect r = { (k%320)*FACTOR, (k/320)*FACTOR, FACTOR, FACTOR };
     SDL_FillRect( screen, &r, SDL_MapRGB( screen->format,
                     getRedFromPixel(buffer[k]),
                     getGreenFromPixel(buffer[k]),
                     getBlueFromPixel(buffer[k])    ) );
#endif
    }
  SDL_UpdateRect( screen, 0, 0, 0, 0 );
}


